/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.emf.type.core.commands.EditElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.objectweb.jasmine.ui.jasminemodel.DbLink;
import org.objectweb.jasmine.ui.jasminemodel.Domain;
import org.objectweb.jasmine.ui.jasminemodel.IJavaEEServer;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.policies.JasmineBaseItemSemanticEditPolicy;

/**
 * @generated
 */
public class DbLink2ReorientCommand extends EditElementCommand {

	/**
	 * @generated
	 */
	private final int reorientDirection;

	/**
	 * @generated
	 */
	private final EObject oldEnd;

	/**
	 * @generated
	 */
	private final EObject newEnd;

	/**
	 * @generated
	 */
	public DbLink2ReorientCommand(ReorientRelationshipRequest request) {
		super(request.getLabel(), request.getRelationship(), request);
		reorientDirection = request.getDirection();
		oldEnd = request.getOldRelationshipEnd();
		newEnd = request.getNewRelationshipEnd();
	}

	/**
	 * @generated
	 */
	public boolean canExecute() {
		if (!(getElementToEdit() instanceof DbLink)) {
			return false;
		}
		if (reorientDirection == ReorientRelationshipRequest.REORIENT_SOURCE) {
			return canReorientSource();
		}
		if (reorientDirection == ReorientRelationshipRequest.REORIENT_TARGET) {
			return canReorientTarget();
		}
		return false;
	}

	/**
	 * @generated
	 */
	protected boolean canReorientSource() {
		if (!(oldEnd instanceof org.objectweb.jasmine.ui.jasminemodel.DB && newEnd instanceof org.objectweb.jasmine.ui.jasminemodel.DB)) {
			return false;
		}
		IJavaEEServer target = getLink().getTarget();
		if (!(getLink().eContainer() instanceof Domain)) {
			return false;
		}
		Domain container = (Domain) getLink().eContainer();
		return JasmineBaseItemSemanticEditPolicy.LinkConstraints
				.canExistDbLink_3008(container, getNewSource(), target);
	}

	/**
	 * @generated
	 */
	protected boolean canReorientTarget() {
		if (!(oldEnd instanceof IJavaEEServer && newEnd instanceof IJavaEEServer)) {
			return false;
		}
		org.objectweb.jasmine.ui.jasminemodel.DB source = getLink().getSource();
		if (!(getLink().eContainer() instanceof Domain)) {
			return false;
		}
		Domain container = (Domain) getLink().eContainer();
		return JasmineBaseItemSemanticEditPolicy.LinkConstraints
				.canExistDbLink_3008(container, source, getNewTarget());
	}

	/**
	 * @generated
	 */
	protected CommandResult doExecuteWithResult(IProgressMonitor monitor,
			IAdaptable info) throws ExecutionException {
		if (!canExecute()) {
			throw new ExecutionException(
					"Invalid arguments in reorient link command"); //$NON-NLS-1$
		}
		if (reorientDirection == ReorientRelationshipRequest.REORIENT_SOURCE) {
			return reorientSource();
		}
		if (reorientDirection == ReorientRelationshipRequest.REORIENT_TARGET) {
			return reorientTarget();
		}
		throw new IllegalStateException();
	}

	/**
	 * @generated
	 */
	protected CommandResult reorientSource() throws ExecutionException {
		getLink().setSource(getNewSource());
		return CommandResult.newOKCommandResult(getLink());
	}

	/**
	 * @generated
	 */
	protected CommandResult reorientTarget() throws ExecutionException {
		getLink().setTarget(getNewTarget());
		return CommandResult.newOKCommandResult(getLink());
	}

	/**
	 * @generated
	 */
	protected DbLink getLink() {
		return (DbLink) getElementToEdit();
	}

	/**
	 * @generated
	 */
	protected org.objectweb.jasmine.ui.jasminemodel.DB getOldSource() {
		return (org.objectweb.jasmine.ui.jasminemodel.DB) oldEnd;
	}

	/**
	 * @generated
	 */
	protected org.objectweb.jasmine.ui.jasminemodel.DB getNewSource() {
		return (org.objectweb.jasmine.ui.jasminemodel.DB) newEnd;
	}

	/**
	 * @generated
	 */
	protected IJavaEEServer getOldTarget() {
		return (IJavaEEServer) oldEnd;
	}

	/**
	 * @generated
	 */
	protected IJavaEEServer getNewTarget() {
		return (IJavaEEServer) newEnd;
	}
}
