/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.edit.policies;

import java.util.Iterator;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientReferenceRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.gmf.runtime.notation.View;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.ModjkConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.ModjkConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RepHTTPConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RepHTTPConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.Apache_HTTPdModulesEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModJKModuleEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModjkConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RepHTTPConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineVisualIDRegistry;
import org.objectweb.jasmine.ui.jasminemodel.diagram.providers.JasmineElementTypes;

/**
 * @generated
 */
public class Apache_HTTPdItemSemanticEditPolicy extends
		JasmineBaseItemSemanticEditPolicy {

	/**
	 * @generated
	 */
	protected Command getDestroyElementCommand(DestroyElementRequest req) {
		CompoundCommand cc = getDestroyEdgesCommand();
		addDestroyChildNodesCommand(cc);
		addDestroyShortcutsCommand(cc);
		View view = (View) getHost().getModel();
		if (view.getEAnnotation("Shortcut") != null) { //$NON-NLS-1$
			req.setElementToDestroy(view);
		}
		cc.add(getGEFWrapper(new DestroyElementCommand(req)));
		return cc.unwrap();
	}

	/**
	 * @generated
	 */
	protected void addDestroyChildNodesCommand(CompoundCommand cmd) {
		View view = (View) getHost().getModel();
		EAnnotation annotation = view.getEAnnotation("Shortcut"); //$NON-NLS-1$
		if (annotation != null) {
			return;
		}
		for (Iterator it = view.getChildren().iterator(); it.hasNext();) {
			Node node = (Node) it.next();
			switch (JasmineVisualIDRegistry.getVisualID(node)) {
			case Apache_HTTPdModulesEditPart.VISUAL_ID:
				for (Iterator cit = node.getChildren().iterator(); cit
						.hasNext();) {
					Node cnode = (Node) cit.next();
					switch (JasmineVisualIDRegistry.getVisualID(cnode)) {
					case ModJKModuleEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					}
				}
				break;
			}
		}
	}

	/**
	 * @generated
	 */
	protected Command getCreateRelationshipCommand(CreateRelationshipRequest req) {
		Command command = req.getTarget() == null ? getStartCreateRelationshipCommand(req)
				: getCompleteCreateRelationshipCommand(req);
		return command != null ? command : super
				.getCreateRelationshipCommand(req);
	}

	/**
	 * @generated
	 */
	protected Command getStartCreateRelationshipCommand(
			CreateRelationshipRequest req) {
		if (JasmineElementTypes.IHTTPServerJavaeeServers_3002 == req
				.getElementType()) {
			return getGEFWrapper(new ModjkConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		if (JasmineElementTypes.JBossModJKConnector_3003 == req
				.getElementType()) {
			return getGEFWrapper(new RepHTTPConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCompleteCreateRelationshipCommand(
			CreateRelationshipRequest req) {
		if (JasmineElementTypes.IHTTPServerJavaeeServers_3002 == req
				.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.JBossModJKConnector_3003 == req
				.getElementType()) {
			return null;
		}
		return null;
	}

	/**
	 * Returns command to reorient EClass based link. New link target or source
	 * should be the domain model element associated with this node.
	 * 
	 * @generated
	 */
	protected Command getReorientRelationshipCommand(
			ReorientRelationshipRequest req) {
		switch (getVisualID(req)) {
		case RepHTTPConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new RepHTTPConnectorReorientCommand(req));
		}
		return super.getReorientRelationshipCommand(req);
	}

	/**
	 * Returns command to reorient EReference based link. New link target or source
	 * should be the domain model element associated with this node.
	 * 
	 * @generated
	 */
	protected Command getReorientReferenceRelationshipCommand(
			ReorientReferenceRelationshipRequest req) {
		switch (getVisualID(req)) {
		case ModjkConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new ModjkConnectorReorientCommand(req));
		}
		return super.getReorientReferenceRelationshipCommand(req);
	}

}
