/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.edit.policies;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gmf.runtime.diagram.core.util.ViewUtil;
import org.eclipse.gmf.runtime.diagram.ui.commands.DeferredLayoutCommand;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.CanonicalConnectionEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateConnectionViewRequest;
import org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.View;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelPackage;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.AJP2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.AJPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.Apache_HTTPdEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ClusterContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ClusterEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CmiGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DBEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLink2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLinkEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DiscoveryServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DomainEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.EJBContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HAConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTP2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPS2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPSEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTP_RepGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HaGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IIOPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IRMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossASEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossBindingEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossHAConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossTryTryEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JRMPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JmsServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModJKModuleEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModjkConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RMIGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RepHTTPConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnector2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.WebContainer2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.WebContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineDiagramUpdater;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineLinkDescriptor;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineNodeDescriptor;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineVisualIDRegistry;

/**
 * @generated
 */
public class DomainCanonicalEditPolicy extends CanonicalConnectionEditPolicy {

	/**
	 * @generated
	 */
	Set myFeaturesToSynchronize;

	/**
	 * @generated
	 */
	protected List getSemanticChildrenList() {
		View viewObject = (View) getHost().getModel();
		List result = new LinkedList();
		for (Iterator it = JasmineDiagramUpdater.getDomain_79SemanticChildren(
				viewObject).iterator(); it.hasNext();) {
			result.add(((JasmineNodeDescriptor) it.next()).getModelElement());
		}
		return result;
	}

	/**
	 * @generated
	 */
	protected boolean shouldDeleteView(View view) {
		return true;
	}

	/**
	 * @generated
	 */
	protected boolean isOrphaned(Collection semanticChildren, final View view) {
		int visualID = JasmineVisualIDRegistry.getVisualID(view);
		switch (visualID) {
		case JOnASEditPart.VISUAL_ID:
		case Apache_HTTPdEditPart.VISUAL_ID:
		case CmiGroupEditPart.VISUAL_ID:
		case HaGroupEditPart.VISUAL_ID:
		case HTTP_RepGroupEditPart.VISUAL_ID:
		case RMIGroupEditPart.VISUAL_ID:
		case ClusterContainerEditPart.VISUAL_ID:
		case DBEditPart.VISUAL_ID:
		case JBossASEditPart.VISUAL_ID:
		case JBossTryTryEditPart.VISUAL_ID:
			return !semanticChildren.contains(view.getElement())
					|| visualID != JasmineVisualIDRegistry.getNodeVisualID(
							(View) getHost().getModel(), view.getElement());
		}
		return false;
	}

	/**
	 * @generated
	 */
	protected String getDefaultFactoryHint() {
		return null;
	}

	/**
	 * @generated
	 */
	protected Set getFeaturesToSynchronize() {
		if (myFeaturesToSynchronize == null) {
			myFeaturesToSynchronize = new HashSet();
			myFeaturesToSynchronize.add(JasminemodelPackage.eINSTANCE
					.getDomain_MiddlewareComponents());
			myFeaturesToSynchronize.add(JasminemodelPackage.eINSTANCE
					.getDomain_ConfigurationNodes());
			myFeaturesToSynchronize.add(JasminemodelPackage.eINSTANCE
					.getDomain_ClusterContainers());
			myFeaturesToSynchronize.add(JasminemodelPackage.eINSTANCE
					.getDomain_Db());
		}
		return myFeaturesToSynchronize;
	}

	/**
	 * @generated
	 */
	protected List getSemanticConnectionsList() {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	protected EObject getSourceElement(EObject relationship) {
		return null;
	}

	/**
	 * @generated
	 */
	protected EObject getTargetElement(EObject relationship) {
		return null;
	}

	/**
	 * @generated
	 */
	protected boolean shouldIncludeConnection(Edge connector,
			Collection children) {
		return false;
	}

	/**
	 * @generated
	 */
	protected void refreshSemantic() {
		List createdViews = new LinkedList();
		createdViews.addAll(refreshSemanticChildren());
		List createdConnectionViews = new LinkedList();
		createdConnectionViews.addAll(refreshSemanticConnections());
		createdConnectionViews.addAll(refreshConnections());

		if (createdViews.size() > 1) {
			// perform a layout of the container
			DeferredLayoutCommand layoutCmd = new DeferredLayoutCommand(host()
					.getEditingDomain(), createdViews, host());
			executeCommand(new ICommandProxy(layoutCmd));
		}

		createdViews.addAll(createdConnectionViews);
		makeViewsImmutable(createdViews);
	}

	/**
	 * @generated
	 */
	private Diagram getDiagram() {
		return ((View) getHost().getModel()).getDiagram();
	}

	/**
	 * @generated
	 */
	private Collection refreshConnections() {
		Map domain2NotationMap = new HashMap();
		Collection linkDescriptors = collectAllLinks(getDiagram(),
				domain2NotationMap);
		Collection existingLinks = new LinkedList(getDiagram().getEdges());
		for (Iterator linksIterator = existingLinks.iterator(); linksIterator
				.hasNext();) {
			Edge nextDiagramLink = (Edge) linksIterator.next();
			EObject diagramLinkObject = nextDiagramLink.getElement();
			EObject diagramLinkSrc = nextDiagramLink.getSource().getElement();
			EObject diagramLinkDst = nextDiagramLink.getTarget().getElement();
			int diagramLinkVisualID = JasmineVisualIDRegistry
					.getVisualID(nextDiagramLink);
			for (Iterator LinkDescriptorsIterator = linkDescriptors.iterator(); LinkDescriptorsIterator
					.hasNext();) {
				JasmineLinkDescriptor nextLinkDescriptor = (JasmineLinkDescriptor) LinkDescriptorsIterator
						.next();
				if (diagramLinkObject == nextLinkDescriptor.getModelElement()
						&& diagramLinkSrc == nextLinkDescriptor.getSource()
						&& diagramLinkDst == nextLinkDescriptor
								.getDestination()
						&& diagramLinkVisualID == nextLinkDescriptor
								.getVisualID()) {
					linksIterator.remove();
					LinkDescriptorsIterator.remove();
				}
			}
		}
		deleteViews(existingLinks.iterator());
		return createConnections(linkDescriptors, domain2NotationMap);
	}

	/**
	 * @generated
	 */
	private Collection collectAllLinks(View view, Map domain2NotationMap) {
		if (!DomainEditPart.MODEL_ID.equals(JasmineVisualIDRegistry
				.getModelID(view))) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		switch (JasmineVisualIDRegistry.getVisualID(view)) {
		case DomainEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getDomain_79ContainedLinks(view));
			break;
		}
		case JOnASEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJOnAS_1001ContainedLinks(view));
			break;
		}
		case Apache_HTTPdEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getApache_HTTPd_1002ContainedLinks(view));
			break;
		}
		case CmiGroupEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getCmiGroup_1003ContainedLinks(view));
			break;
		}
		case HaGroupEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHaGroup_1004ContainedLinks(view));
			break;
		}
		case HTTP_RepGroupEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHTTP_RepGroup_1005ContainedLinks(view));
			break;
		}
		case RMIGroupEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getRMIGroup_1006ContainedLinks(view));
			break;
		}
		case ClusterContainerEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getClusterContainer_1007ContainedLinks(view));
			break;
		}
		case DBEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater.getDB_1008ContainedLinks(view));
			break;
		}
		case JBossASEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossAS_1009ContainedLinks(view));
			break;
		}
		case JBossTryTryEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossHAGroup_1010ContainedLinks(view));
			break;
		}
		case WebContainerEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getWebContainer_2001ContainedLinks(view));
			break;
		}
		case EJBContainerEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getEJBContainer_2002ContainedLinks(view));
			break;
		}
		case DiscoveryServiceEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getDiscoveryService_2003ContainedLinks(view));
			break;
		}
		case JmsServiceEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJmsService_2004ContainedLinks(view));
			break;
		}
		case IIOPEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getIIOP_2005ContainedLinks(view));
			break;
		}
		case IRMIEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getIRMI_2006ContainedLinks(view));
			break;
		}
		case JRMPEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJRMP_2007ContainedLinks(view));
			break;
		}
		case CMIEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result
					.addAll(JasmineDiagramUpdater
							.getCMI_2008ContainedLinks(view));
			break;
		}
		case AJPEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result
					.addAll(JasmineDiagramUpdater
							.getAJP_2009ContainedLinks(view));
			break;
		}
		case HTTPEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHTTP_2010ContainedLinks(view));
			break;
		}
		case HTTPSEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHTTPS_2011ContainedLinks(view));
			break;
		}
		case ModJKModuleEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getModJKModule_2012ContainedLinks(view));
			break;
		}
		case ClusterEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getCluster_2013ContainedLinks(view));
			break;
		}
		case JBossBindingEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossBinding_2014ContainedLinks(view));
			break;
		}
		case WebContainer2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getWebContainer_2015ContainedLinks(view));
			break;
		}
		case HTTPS2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHTTPS_2016ContainedLinks(view));
			break;
		}
		case HTTP2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHTTP_2017ContainedLinks(view));
			break;
		}
		case AJP2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result
					.addAll(JasmineDiagramUpdater
							.getAJP_2018ContainedLinks(view));
			break;
		}
		case RepHTTPConnectorEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossModJKConnector_3003ContainedLinks(view));
			break;
		}
		case HAConnectorEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getRepHTTPConnector_3004ContainedLinks(view));
			break;
		}
		case DbLinkEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossHTTPRepConnector_3005ContainedLinks(view));
			break;
		}
		case RmiConnectorEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getHAConnector_3006ContainedLinks(view));
			break;
		}
		case JBossHAConnectorEditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getJBossHAConnector_3007ContainedLinks(view));
			break;
		}
		case DbLink2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getDbLink_3008ContainedLinks(view));
			break;
		}
		case RmiConnector2EditPart.VISUAL_ID: {
			domain2NotationMap.put(view.getElement(), view);
			result.addAll(JasmineDiagramUpdater
					.getRmiConnector_3009ContainedLinks(view));
			break;
		}
		}
		for (Iterator children = view.getChildren().iterator(); children
				.hasNext();) {
			result.addAll(collectAllLinks((View) children.next(),
					domain2NotationMap));
		}
		for (Iterator edges = view.getSourceEdges().iterator(); edges.hasNext();) {
			result.addAll(collectAllLinks((View) edges.next(),
					domain2NotationMap));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private Collection createConnections(Collection linkDescriptors,
			Map domain2NotationMap) {
		List adapters = new LinkedList();
		for (Iterator linkDescriptorsIterator = linkDescriptors.iterator(); linkDescriptorsIterator
				.hasNext();) {
			final JasmineLinkDescriptor nextLinkDescriptor = (JasmineLinkDescriptor) linkDescriptorsIterator
					.next();
			EditPart sourceEditPart = getEditPart(nextLinkDescriptor
					.getSource(), domain2NotationMap);
			EditPart targetEditPart = getEditPart(nextLinkDescriptor
					.getDestination(), domain2NotationMap);
			if (sourceEditPart == null || targetEditPart == null) {
				continue;
			}
			CreateConnectionViewRequest.ConnectionViewDescriptor descriptor = new CreateConnectionViewRequest.ConnectionViewDescriptor(
					nextLinkDescriptor.getSemanticAdapter(), null,
					ViewUtil.APPEND, false, ((IGraphicalEditPart) getHost())
							.getDiagramPreferencesHint());
			CreateConnectionViewRequest ccr = new CreateConnectionViewRequest(
					descriptor);
			ccr.setType(RequestConstants.REQ_CONNECTION_START);
			ccr.setSourceEditPart(sourceEditPart);
			sourceEditPart.getCommand(ccr);
			ccr.setTargetEditPart(targetEditPart);
			ccr.setType(RequestConstants.REQ_CONNECTION_END);
			Command cmd = targetEditPart.getCommand(ccr);
			if (cmd != null && cmd.canExecute()) {
				executeCommand(cmd);
				IAdaptable viewAdapter = (IAdaptable) ccr.getNewObject();
				if (viewAdapter != null) {
					adapters.add(viewAdapter);
				}
			}
		}
		return adapters;
	}

	/**
	 * @generated
	 */
	private EditPart getEditPart(EObject domainModelElement,
			Map domain2NotationMap) {
		View view = (View) domain2NotationMap.get(domainModelElement);
		if (view != null) {
			return (EditPart) getHost().getViewer().getEditPartRegistry().get(
					view);
		}
		return null;
	}
}
