/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.edit.policies;

import java.util.Iterator;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientReferenceRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.gmf.runtime.notation.View;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.CMIConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.CMIConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.DbLink2CreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.DbLink2ReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.DbLinkCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.DbLinkReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.HAConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.HAConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.ModjkConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.ModjkConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RepHTTPConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RepHTTPConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RmiConnector2CreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RmiConnector2ReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RmiConnectorCreateCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.commands.RmiConnectorReorientCommand;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.AJPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLink2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLinkEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DiscoveryServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.EJBContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HAConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPSEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IIOPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IRMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASProtocolsEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASServicesEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASWeb_ConnectorsEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JRMPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JmsServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModjkConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RepHTTPConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnector2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.WebContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineVisualIDRegistry;
import org.objectweb.jasmine.ui.jasminemodel.diagram.providers.JasmineElementTypes;

/**
 * @generated
 */
public class JOnASItemSemanticEditPolicy extends
		JasmineBaseItemSemanticEditPolicy {

	/**
	 * @generated
	 */
	protected Command getDestroyElementCommand(DestroyElementRequest req) {
		CompoundCommand cc = getDestroyEdgesCommand();
		addDestroyChildNodesCommand(cc);
		addDestroyShortcutsCommand(cc);
		View view = (View) getHost().getModel();
		if (view.getEAnnotation("Shortcut") != null) { //$NON-NLS-1$
			req.setElementToDestroy(view);
		}
		cc.add(getGEFWrapper(new DestroyElementCommand(req)));
		return cc.unwrap();
	}

	/**
	 * @generated
	 */
	protected void addDestroyChildNodesCommand(CompoundCommand cmd) {
		View view = (View) getHost().getModel();
		EAnnotation annotation = view.getEAnnotation("Shortcut"); //$NON-NLS-1$
		if (annotation != null) {
			return;
		}
		for (Iterator it = view.getChildren().iterator(); it.hasNext();) {
			Node node = (Node) it.next();
			switch (JasmineVisualIDRegistry.getVisualID(node)) {
			case JOnASServicesEditPart.VISUAL_ID:
				for (Iterator cit = node.getChildren().iterator(); cit
						.hasNext();) {
					Node cnode = (Node) cit.next();
					switch (JasmineVisualIDRegistry.getVisualID(cnode)) {
					case WebContainerEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case EJBContainerEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case DiscoveryServiceEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case JmsServiceEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					}
				}
				break;
			case JOnASProtocolsEditPart.VISUAL_ID:
				for (Iterator cit = node.getChildren().iterator(); cit
						.hasNext();) {
					Node cnode = (Node) cit.next();
					switch (JasmineVisualIDRegistry.getVisualID(cnode)) {
					case IIOPEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case IRMIEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case JRMPEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case CMIEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					}
				}
				break;
			case JOnASWeb_ConnectorsEditPart.VISUAL_ID:
				for (Iterator cit = node.getChildren().iterator(); cit
						.hasNext();) {
					Node cnode = (Node) cit.next();
					switch (JasmineVisualIDRegistry.getVisualID(cnode)) {
					case AJPEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case HTTPEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					case HTTPSEditPart.VISUAL_ID:
						cmd.add(getDestroyElementCommand(cnode));
						break;
					}
				}
				break;
			}
		}
	}

	/**
	 * @generated
	 */
	protected Command getCreateRelationshipCommand(CreateRelationshipRequest req) {
		Command command = req.getTarget() == null ? getStartCreateRelationshipCommand(req)
				: getCompleteCreateRelationshipCommand(req);
		return command != null ? command : super
				.getCreateRelationshipCommand(req);
	}

	/**
	 * @generated
	 */
	protected Command getStartCreateRelationshipCommand(
			CreateRelationshipRequest req) {
		if (JasmineElementTypes.JOnASCmigroup_3001 == req.getElementType()) {
			return getGEFWrapper(new CMIConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		if (JasmineElementTypes.IHTTPServerJavaeeServers_3002 == req
				.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.RepHTTPConnector_3004 == req.getElementType()) {
			return getGEFWrapper(new HAConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		if (JasmineElementTypes.HAConnector_3006 == req.getElementType()) {
			return getGEFWrapper(new RmiConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		if (JasmineElementTypes.DbLink_3008 == req.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.RmiConnector_3009 == req.getElementType()) {
			return getGEFWrapper(new RmiConnector2CreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCompleteCreateRelationshipCommand(
			CreateRelationshipRequest req) {
		if (JasmineElementTypes.JOnASCmigroup_3001 == req.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.IHTTPServerJavaeeServers_3002 == req
				.getElementType()) {
			return getGEFWrapper(new ModjkConnectorCreateCommand(req, req
					.getSource(), req.getTarget()));
		}
		if (JasmineElementTypes.RepHTTPConnector_3004 == req.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.HAConnector_3006 == req.getElementType()) {
			return null;
		}
		if (JasmineElementTypes.DbLink_3008 == req.getElementType()) {
			return getGEFWrapper(new DbLink2CreateCommand(req, req.getSource(),
					req.getTarget()));
		}
		if (JasmineElementTypes.RmiConnector_3009 == req.getElementType()) {
			return null;
		}
		return null;
	}

	/**
	 * Returns command to reorient EClass based link. New link target or source
	 * should be the domain model element associated with this node.
	 * 
	 * @generated
	 */
	protected Command getReorientRelationshipCommand(
			ReorientRelationshipRequest req) {
		switch (getVisualID(req)) {
		case HAConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new HAConnectorReorientCommand(req));
		case RmiConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new RmiConnectorReorientCommand(req));
		case DbLink2EditPart.VISUAL_ID:
			return getGEFWrapper(new DbLink2ReorientCommand(req));
		case RmiConnector2EditPart.VISUAL_ID:
			return getGEFWrapper(new RmiConnector2ReorientCommand(req));
		}
		return super.getReorientRelationshipCommand(req);
	}

	/**
	 * Returns command to reorient EReference based link. New link target or source
	 * should be the domain model element associated with this node.
	 * 
	 * @generated
	 */
	protected Command getReorientReferenceRelationshipCommand(
			ReorientReferenceRelationshipRequest req) {
		switch (getVisualID(req)) {
		case CMIConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new CMIConnectorReorientCommand(req));
		case ModjkConnectorEditPart.VISUAL_ID:
			return getGEFWrapper(new ModjkConnectorReorientCommand(req));
		}
		return super.getReorientReferenceRelationshipCommand(req);
	}

}
