/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.edit.policies;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;

import java.util.Map;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.diagram.core.commands.DeleteCommand;
import org.eclipse.gmf.runtime.diagram.core.util.ViewUtil;
import org.eclipse.gmf.runtime.diagram.ui.commands.CommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.SemanticEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.requests.EditCommandRequestWrapper;
import org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IEditHelperContext;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DuplicateElementsRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.GetEditContextRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.MoveRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientReferenceRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.ReorientRelationshipRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.View;
import org.objectweb.jasmine.ui.jasminemodel.Apache_HTTPd;
import org.objectweb.jasmine.ui.jasminemodel.CmiGroup;
import org.objectweb.jasmine.ui.jasminemodel.Domain;
import org.objectweb.jasmine.ui.jasminemodel.HTTP_RepGroup;
import org.objectweb.jasmine.ui.jasminemodel.HaGroup;
import org.objectweb.jasmine.ui.jasminemodel.IHTTPServer;
import org.objectweb.jasmine.ui.jasminemodel.IJavaEEServer;
import org.objectweb.jasmine.ui.jasminemodel.JBossAS;
import org.objectweb.jasmine.ui.jasminemodel.JBossHAGroup;
import org.objectweb.jasmine.ui.jasminemodel.JOnAS;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelPackage;
import org.objectweb.jasmine.ui.jasminemodel.RMIGroup;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.helpers.JasmineBaseEditHelper;
import org.objectweb.jasmine.ui.jasminemodel.diagram.expressions.JasmineAbstractExpression;
import org.objectweb.jasmine.ui.jasminemodel.diagram.expressions.JasmineOCLFactory;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineDiagramEditorPlugin;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineVisualIDRegistry;

/**
 * @generated
 */
public class JasmineBaseItemSemanticEditPolicy extends SemanticEditPolicy {

	/**
	 * Extended request data key to hold editpart visual id.
	 * 
	 * @generated
	 */
	public static final String VISUAL_ID_KEY = "visual_id"; //$NON-NLS-1$

	/**
	 * Extended request data key to hold editpart visual id.
	 * Add visual id of edited editpart to extended data of the request
	 * so command switch can decide what kind of diagram element is being edited.
	 * It is done in those cases when it's not possible to deduce diagram
	 * element kind from domain element.
	 * 
	 * @generated
	 */
	public Command getCommand(Request request) {
		if (request instanceof ReconnectRequest) {
			Object view = ((ReconnectRequest) request).getConnectionEditPart()
					.getModel();
			if (view instanceof View) {
				Integer id = new Integer(JasmineVisualIDRegistry
						.getVisualID((View) view));
				request.getExtendedData().put(VISUAL_ID_KEY, id);
			}
		}
		return super.getCommand(request);
	}

	/**
	 * Returns visual id from request parameters.
	 * 
	 * @generated
	 */
	protected int getVisualID(IEditCommandRequest request) {
		Object id = request.getParameter(VISUAL_ID_KEY);
		return id instanceof Integer ? ((Integer) id).intValue() : -1;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommand(IEditCommandRequest request) {
		IEditCommandRequest completedRequest = completeRequest(request);
		Object editHelperContext = completedRequest.getEditHelperContext();
		if (editHelperContext instanceof View
				|| (editHelperContext instanceof IEditHelperContext && ((IEditHelperContext) editHelperContext)
						.getEObject() instanceof View)) {
			// no semantic commands are provided for pure design elements
			return null;
		}
		if (editHelperContext == null) {
			editHelperContext = ViewUtil
					.resolveSemanticElement((View) getHost().getModel());
		}
		IElementType elementType = ElementTypeRegistry.getInstance()
				.getElementType(editHelperContext);
		if (elementType == ElementTypeRegistry.getInstance().getType(
				"org.eclipse.gmf.runtime.emf.type.core.default")) { //$NON-NLS-1$ 
			elementType = null;
		}
		Command semanticCommand = getSemanticCommandSwitch(completedRequest);
		if (semanticCommand != null) {
			ICommand command = semanticCommand instanceof ICommandProxy ? ((ICommandProxy) semanticCommand)
					.getICommand()
					: new CommandProxy(semanticCommand);
			completedRequest.setParameter(
					JasmineBaseEditHelper.EDIT_POLICY_COMMAND, command);
		}
		if (elementType != null) {
			ICommand command = elementType.getEditCommand(completedRequest);
			if (command != null) {
				if (!(command instanceof CompositeTransactionalCommand)) {
					TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) getHost())
							.getEditingDomain();
					command = new CompositeTransactionalCommand(editingDomain,
							null).compose(command);
				}
				semanticCommand = new ICommandProxy(command);
			}
		}
		boolean shouldProceed = true;
		if (completedRequest instanceof DestroyRequest) {
			shouldProceed = shouldProceed((DestroyRequest) completedRequest);
		}
		if (shouldProceed) {
			if (completedRequest instanceof DestroyRequest) {
				TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) getHost())
						.getEditingDomain();
				Command deleteViewCommand = new ICommandProxy(
						new DeleteCommand(editingDomain, (View) getHost()
								.getModel()));
				semanticCommand = semanticCommand == null ? deleteViewCommand
						: semanticCommand.chain(deleteViewCommand);
			}
			return semanticCommand;
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getSemanticCommandSwitch(IEditCommandRequest req) {
		if (req instanceof CreateRelationshipRequest) {
			return getCreateRelationshipCommand((CreateRelationshipRequest) req);
		} else if (req instanceof CreateElementRequest) {
			return getCreateCommand((CreateElementRequest) req);
		} else if (req instanceof ConfigureRequest) {
			return getConfigureCommand((ConfigureRequest) req);
		} else if (req instanceof DestroyElementRequest) {
			return getDestroyElementCommand((DestroyElementRequest) req);
		} else if (req instanceof DestroyReferenceRequest) {
			return getDestroyReferenceCommand((DestroyReferenceRequest) req);
		} else if (req instanceof DuplicateElementsRequest) {
			return getDuplicateCommand((DuplicateElementsRequest) req);
		} else if (req instanceof GetEditContextRequest) {
			return getEditContextCommand((GetEditContextRequest) req);
		} else if (req instanceof MoveRequest) {
			return getMoveCommand((MoveRequest) req);
		} else if (req instanceof ReorientReferenceRelationshipRequest) {
			return getReorientReferenceRelationshipCommand((ReorientReferenceRelationshipRequest) req);
		} else if (req instanceof ReorientRelationshipRequest) {
			return getReorientRelationshipCommand((ReorientRelationshipRequest) req);
		} else if (req instanceof SetRequest) {
			return getSetCommand((SetRequest) req);
		}
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getConfigureCommand(ConfigureRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateRelationshipCommand(CreateRelationshipRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getCreateCommand(CreateElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getSetCommand(SetRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getEditContextCommand(GetEditContextRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyElementCommand(DestroyElementRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDestroyReferenceCommand(DestroyReferenceRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getDuplicateCommand(DuplicateElementsRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getMoveCommand(MoveRequest req) {
		return null;
	}

	/**
	 * @generated
	 */
	protected Command getReorientReferenceRelationshipCommand(
			ReorientReferenceRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected Command getReorientRelationshipCommand(
			ReorientRelationshipRequest req) {
		return UnexecutableCommand.INSTANCE;
	}

	/**
	 * @generated
	 */
	protected final Command getGEFWrapper(ICommand cmd) {
		return new ICommandProxy(cmd);
	}

	/**
	 * @deprecated use getGEFWrapper() instead
	 * @generated
	 */
	protected final Command getMSLWrapper(ICommand cmd) {
		// XXX deprecated: use getGEFWrapper() instead
		return getGEFWrapper(cmd);
	}

	/**
	 * @generated
	 */
	protected EObject getSemanticElement() {
		return ViewUtil.resolveSemanticElement((View) getHost().getModel());
	}

	/**
	 * Returns editing domain from the host edit part.
	 * 
	 * @generated
	 */
	protected TransactionalEditingDomain getEditingDomain() {
		return ((IGraphicalEditPart) getHost()).getEditingDomain();
	}

	/**
	 * Creates command to destroy the link.
	 * 
	 * @generated
	 */
	protected Command getDestroyElementCommand(View view) {
		EditPart editPart = (EditPart) getHost().getViewer()
				.getEditPartRegistry().get(view);
		DestroyElementRequest request = new DestroyElementRequest(
				getEditingDomain(), false);
		return editPart.getCommand(new EditCommandRequestWrapper(request,
				Collections.EMPTY_MAP));
	}

	/**
	 * Creates commands to destroy all host incoming and outgoing links.
	 * 
	 * @generated
	 */
	protected CompoundCommand getDestroyEdgesCommand() {
		CompoundCommand cmd = new CompoundCommand();
		View view = (View) getHost().getModel();
		for (Iterator it = view.getSourceEdges().iterator(); it.hasNext();) {
			cmd.add(getDestroyElementCommand((Edge) it.next()));
		}
		for (Iterator it = view.getTargetEdges().iterator(); it.hasNext();) {
			cmd.add(getDestroyElementCommand((Edge) it.next()));
		}
		return cmd;
	}

	/**
	 * @generated
	 */
	protected void addDestroyShortcutsCommand(CompoundCommand command) {
		View view = (View) getHost().getModel();
		if (view.getEAnnotation("Shortcut") != null) { //$NON-NLS-1$
			return;
		}
		for (Iterator it = view.getDiagram().getChildren().iterator(); it
				.hasNext();) {
			View nextView = (View) it.next();
			if (nextView.getEAnnotation("Shortcut") == null || !nextView.isSetElement() || nextView.getElement() != view.getElement()) { //$NON-NLS-1$
				continue;
			}
			command.add(getDestroyElementCommand(nextView));
		}
	}

	/**
	 * @generated
	 */
	public static class LinkConstraints {

		/**
		 * @generated
		 */
		private static final String OPPOSITE_END_VAR = "oppositeEnd"; //$NON-NLS-1$

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression JOnASCmigroup_3001_SourceExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getCmiGroup());
			JOnASCmigroup_3001_SourceExpression = JasmineOCLFactory
					.getExpression(
							"self.rmiRegistries->select(oclIsTypeOf(CMI))->size()>0", JasminemodelPackage.eINSTANCE.getJOnAS(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression IHTTPServerJavaeeServers_3002_SourceExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getIJavaEEServer());
			IHTTPServerJavaeeServers_3002_SourceExpression = JasmineOCLFactory
					.getExpression(
							"self.oclIsKindOf(Apache_HTTPd) and self.oclAsType(Apache_HTTPd).modules->select(oclIsTypeOf(ModJKModule))->size()>0", JasminemodelPackage.eINSTANCE.getIHTTPServer(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression IHTTPServerJavaeeServers_3002_TargetExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getIHTTPServer());
			IHTTPServerJavaeeServers_3002_TargetExpression = JasmineOCLFactory
					.getExpression(
							"self.oclIsKindOf(JOnAS) and self.oclAsType(JOnAS).webconnectors->select(oclIsTypeOf(AJP))->size()>0", JasminemodelPackage.eINSTANCE.getIJavaEEServer(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression RepHTTPConnector_3004_SourceExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getHTTP_RepGroup());
			RepHTTPConnector_3004_SourceExpression = JasmineOCLFactory
					.getExpression(
							"self->webContainers->size()>0\nand (\n\tself->webconnectors->select(oclIsTypeOf(HTTP))->size()>0\n\tor self->webconnectors->select(oclIsTypeOf(HTTPS))->size()>0\n)", JasminemodelPackage.eINSTANCE.getJOnAS(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression JBossHTTPRepConnector_3005_SourceExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getHTTP_RepGroup());
			JBossHTTPRepConnector_3005_SourceExpression = JasmineOCLFactory
					.getExpression(
							"self->webconnectors->select(oclIsTypeOf(HTTP))->size()>0\nor\nself->webconnectors->select(oclIsTypeOf(HTTPS))->size()>0", JasminemodelPackage.eINSTANCE.getJBossAS(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		private static final JasmineAbstractExpression HAConnector_3006_SourceExpression;

		/**
		 * @generated
		 */
		static {
			Map env = new HashMap(3);
			env.put(OPPOSITE_END_VAR, JasminemodelPackage.eINSTANCE
					.getHaGroup());
			HAConnector_3006_SourceExpression = JasmineOCLFactory
					.getExpression(
							"self->ejbContainers->size()>0", JasminemodelPackage.eINSTANCE.getJOnAS(), env); //$NON-NLS-1$
		}

		/**
		 * @generated
		 */
		public static boolean canCreateJOnASCmigroup_3001(JOnAS source,
				CmiGroup target) {
			if (source != null) {
				if (source.getCmigroup() != null) {
					return false;
				}
			}
			return canExistJOnASCmigroup_3001(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateIHTTPServerJavaeeServers_3002(
				IHTTPServer source, IJavaEEServer target) {
			if (source != null) {
				if (source.getJavaeeServers().contains(target)) {
					return false;
				}
			}
			return canExistIHTTPServerJavaeeServers_3002(source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateJBossModJKConnector_3003(
				Domain container, Apache_HTTPd source, JBossAS target) {
			return canExistJBossModJKConnector_3003(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateRepHTTPConnector_3004(Domain container,
				JOnAS source, HTTP_RepGroup target) {
			return canExistRepHTTPConnector_3004(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateJBossHTTPRepConnector_3005(
				Domain container, JBossAS source, HTTP_RepGroup target) {
			return canExistJBossHTTPRepConnector_3005(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateHAConnector_3006(Domain container,
				JOnAS source, HaGroup target) {
			return canExistHAConnector_3006(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateJBossHAConnector_3007(Domain container,
				JBossAS source, JBossHAGroup target) {
			return canExistJBossHAConnector_3007(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateDbLink_3008(Domain container,
				org.objectweb.jasmine.ui.jasminemodel.DB source,
				IJavaEEServer target) {
			return canExistDbLink_3008(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canCreateRmiConnector_3009(Domain container,
				JOnAS source, RMIGroup target) {
			return canExistRmiConnector_3009(container, source, target);
		}

		/**
		 * @generated
		 */
		public static boolean canExistJOnASCmigroup_3001(JOnAS source,
				CmiGroup target) {
			if (!evaluate(JOnASCmigroup_3001_SourceExpression, source, target,
					false)) {
				return false;
			}
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistIHTTPServerJavaeeServers_3002(
				IHTTPServer source, IJavaEEServer target) {
			if (!evaluate(IHTTPServerJavaeeServers_3002_SourceExpression,
					source, target, false)) {
				return false;
			}
			if (!evaluate(IHTTPServerJavaeeServers_3002_TargetExpression,
					target, source, true)) {
				return false;
			}
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistJBossModJKConnector_3003(
				Domain container, Apache_HTTPd source, JBossAS target) {
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistRepHTTPConnector_3004(Domain container,
				JOnAS source, HTTP_RepGroup target) {
			if (!evaluate(RepHTTPConnector_3004_SourceExpression, source,
					target, false)) {
				return false;
			}
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistJBossHTTPRepConnector_3005(
				Domain container, JBossAS source, HTTP_RepGroup target) {
			if (!evaluate(JBossHTTPRepConnector_3005_SourceExpression, source,
					target, false)) {
				return false;
			}
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistHAConnector_3006(Domain container,
				JOnAS source, HaGroup target) {
			if (!evaluate(HAConnector_3006_SourceExpression, source, target,
					false)) {
				return false;
			}
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistJBossHAConnector_3007(Domain container,
				JBossAS source, JBossHAGroup target) {
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistDbLink_3008(Domain container,
				org.objectweb.jasmine.ui.jasminemodel.DB source,
				IJavaEEServer target) {
			return true;
		}

		/**
		 * @generated
		 */
		public static boolean canExistRmiConnector_3009(Domain container,
				JOnAS source, RMIGroup target) {
			return true;
		}

		/**
		 * @generated
		 */
		private static boolean evaluate(JasmineAbstractExpression constraint,
				Object sourceEnd, Object oppositeEnd, boolean clearEnv) {
			if (sourceEnd == null) {
				return true;
			}
			Map evalEnv = Collections.singletonMap(OPPOSITE_END_VAR,
					oppositeEnd);
			try {
				Object val = constraint.evaluate(sourceEnd, evalEnv);
				return (val instanceof Boolean) ? ((Boolean) val)
						.booleanValue() : false;
			} catch (Exception e) {
				JasmineDiagramEditorPlugin.getInstance().logError(
						"Link constraint evaluation error", e); //$NON-NLS-1$
				return false;
			}
		}

	}

}
