/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.part;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gmf.runtime.notation.View;
import org.objectweb.jasmine.ui.jasminemodel.Apache_HTTPd;
import org.objectweb.jasmine.ui.jasminemodel.CMIConnector;
import org.objectweb.jasmine.ui.jasminemodel.Cluster;
import org.objectweb.jasmine.ui.jasminemodel.ClusterContainer;
import org.objectweb.jasmine.ui.jasminemodel.CmiGroup;
import org.objectweb.jasmine.ui.jasminemodel.ConfigurationNode;
import org.objectweb.jasmine.ui.jasminemodel.DbLink;
import org.objectweb.jasmine.ui.jasminemodel.DiscoveryService;
import org.objectweb.jasmine.ui.jasminemodel.Domain;
import org.objectweb.jasmine.ui.jasminemodel.EJBContainer;
import org.objectweb.jasmine.ui.jasminemodel.HAConnector;
import org.objectweb.jasmine.ui.jasminemodel.HTTP_RepGroup;
import org.objectweb.jasmine.ui.jasminemodel.HaGroup;
import org.objectweb.jasmine.ui.jasminemodel.IHTTPServer;
import org.objectweb.jasmine.ui.jasminemodel.IJavaEEServer;
import org.objectweb.jasmine.ui.jasminemodel.IMiddlewareComponent;
import org.objectweb.jasmine.ui.jasminemodel.IModule;
import org.objectweb.jasmine.ui.jasminemodel.IRMIRegistry;
import org.objectweb.jasmine.ui.jasminemodel.JBossAS;
import org.objectweb.jasmine.ui.jasminemodel.JBossBinding;
import org.objectweb.jasmine.ui.jasminemodel.JBossHAConnector;
import org.objectweb.jasmine.ui.jasminemodel.JBossHAGroup;
import org.objectweb.jasmine.ui.jasminemodel.JBossHTTPRepConnector;
import org.objectweb.jasmine.ui.jasminemodel.JBossModJKConnector;
import org.objectweb.jasmine.ui.jasminemodel.JOnAS;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelPackage;
import org.objectweb.jasmine.ui.jasminemodel.JmsService;
import org.objectweb.jasmine.ui.jasminemodel.RMIGroup;
import org.objectweb.jasmine.ui.jasminemodel.RepHTTPConnector;
import org.objectweb.jasmine.ui.jasminemodel.RmiConnector;
import org.objectweb.jasmine.ui.jasminemodel.WebConnector;
import org.objectweb.jasmine.ui.jasminemodel.WebContainer;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.AJP2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.AJPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.Apache_HTTPdEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.Apache_HTTPdModulesEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ClusterContainerClusterSetEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ClusterContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ClusterEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.CmiGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DBEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLink2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DbLinkEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DiscoveryServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DomainEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.EJBContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HAConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTP2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPS2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTPSEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HTTP_RepGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.HaGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IIOPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.IRMIEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossASEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossASServicesEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossASWeb_ConnectorsEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossBindingEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossHAConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JBossTryTryEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASProtocolsEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASServicesEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JOnASWeb_ConnectorsEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JRMPEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.JmsServiceEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModJKModuleEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.ModjkConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RMIGroupEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RepHTTPConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnector2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.RmiConnectorEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.WebContainer2EditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.WebContainerEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.providers.JasmineElementTypes;

/**
 * @generated
 */
public class JasmineDiagramUpdater {

	/**
	 * @generated
	 */
	public static List getSemanticChildren(View view) {
		switch (JasmineVisualIDRegistry.getVisualID(view)) {
		case JOnASServicesEditPart.VISUAL_ID:
			return getJOnASServices_5001SemanticChildren(view);
		case JOnASProtocolsEditPart.VISUAL_ID:
			return getJOnASProtocols_5002SemanticChildren(view);
		case JOnASWeb_ConnectorsEditPart.VISUAL_ID:
			return getJOnASWeb_Connectors_5003SemanticChildren(view);
		case Apache_HTTPdModulesEditPart.VISUAL_ID:
			return getApache_HTTPdModules_5004SemanticChildren(view);
		case ClusterContainerClusterSetEditPart.VISUAL_ID:
			return getClusterContainerClusterSet_5005SemanticChildren(view);
		case JBossASServicesEditPart.VISUAL_ID:
			return getJBossASServices_5006SemanticChildren(view);
		case JBossASWeb_ConnectorsEditPart.VISUAL_ID:
			return getJBossASWeb_Connectors_5007SemanticChildren(view);
		case DomainEditPart.VISUAL_ID:
			return getDomain_79SemanticChildren(view);
		}
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJOnASServices_5001SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		JOnAS modelElement = (JOnAS) containerView.getElement();
		List result = new LinkedList();
		{
			WebContainer childElement = modelElement.getWebContainers();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == WebContainerEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		{
			EJBContainer childElement = modelElement.getEjbContainers();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == EJBContainerEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		{
			DiscoveryService childElement = modelElement.getDiscoveryServices();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == DiscoveryServiceEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		{
			JmsService childElement = modelElement.getJmsServices();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == JmsServiceEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJOnASProtocols_5002SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		JOnAS modelElement = (JOnAS) containerView.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getRmiRegistries().iterator(); it
				.hasNext();) {
			IRMIRegistry childElement = (IRMIRegistry) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == IIOPEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == IRMIEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == JRMPEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == CMIEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJOnASWeb_Connectors_5003SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		JOnAS modelElement = (JOnAS) containerView.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getWebconnectors().iterator(); it
				.hasNext();) {
			WebConnector childElement = (WebConnector) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == AJPEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == HTTPEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == HTTPSEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getApache_HTTPdModules_5004SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		Apache_HTTPd modelElement = (Apache_HTTPd) containerView.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getModules().iterator(); it.hasNext();) {
			IModule childElement = (IModule) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == ModJKModuleEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getClusterContainerClusterSet_5005SemanticChildren(
			View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		ClusterContainer modelElement = (ClusterContainer) containerView
				.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getClusters().iterator(); it.hasNext();) {
			Cluster childElement = (Cluster) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == ClusterEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJBossASServices_5006SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		JBossAS modelElement = (JBossAS) containerView.getElement();
		List result = new LinkedList();
		{
			JBossBinding childElement = modelElement.getBinding();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == JBossBindingEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		{
			WebContainer childElement = modelElement.getWebContainers();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == WebContainer2EditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJBossASWeb_Connectors_5007SemanticChildren(View view) {
		if (false == view.eContainer() instanceof View) {
			return Collections.EMPTY_LIST;
		}
		View containerView = (View) view.eContainer();
		if (!containerView.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		JBossAS modelElement = (JBossAS) containerView.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getWebconnectors().iterator(); it
				.hasNext();) {
			WebConnector childElement = (WebConnector) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == HTTPS2EditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == HTTP2EditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == AJP2EditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getDomain_79SemanticChildren(View view) {
		if (!view.isSetElement()) {
			return Collections.EMPTY_LIST;
		}
		Domain modelElement = (Domain) view.getElement();
		List result = new LinkedList();
		for (Iterator it = modelElement.getMiddlewareComponents().iterator(); it
				.hasNext();) {
			IMiddlewareComponent childElement = (IMiddlewareComponent) it
					.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == JOnASEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == Apache_HTTPdEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == JBossASEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		for (Iterator it = modelElement.getConfigurationNodes().iterator(); it
				.hasNext();) {
			ConfigurationNode childElement = (ConfigurationNode) it.next();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == CmiGroupEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == HaGroupEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == HTTP_RepGroupEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == RMIGroupEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
			if (visualID == JBossTryTryEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
				continue;
			}
		}
		{
			ClusterContainer childElement = modelElement.getClusterContainers();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == ClusterContainerEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		{
			org.objectweb.jasmine.ui.jasminemodel.DB childElement = modelElement
					.getDb();
			int visualID = JasmineVisualIDRegistry.getNodeVisualID(view,
					childElement);
			if (visualID == DBEditPart.VISUAL_ID) {
				result.add(new JasmineNodeDescriptor(childElement, visualID));
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	public static List getContainedLinks(View view) {
		switch (JasmineVisualIDRegistry.getVisualID(view)) {
		case DomainEditPart.VISUAL_ID:
			return getDomain_79ContainedLinks(view);
		case JOnASEditPart.VISUAL_ID:
			return getJOnAS_1001ContainedLinks(view);
		case Apache_HTTPdEditPart.VISUAL_ID:
			return getApache_HTTPd_1002ContainedLinks(view);
		case CmiGroupEditPart.VISUAL_ID:
			return getCmiGroup_1003ContainedLinks(view);
		case HaGroupEditPart.VISUAL_ID:
			return getHaGroup_1004ContainedLinks(view);
		case HTTP_RepGroupEditPart.VISUAL_ID:
			return getHTTP_RepGroup_1005ContainedLinks(view);
		case RMIGroupEditPart.VISUAL_ID:
			return getRMIGroup_1006ContainedLinks(view);
		case ClusterContainerEditPart.VISUAL_ID:
			return getClusterContainer_1007ContainedLinks(view);
		case DBEditPart.VISUAL_ID:
			return getDB_1008ContainedLinks(view);
		case JBossASEditPart.VISUAL_ID:
			return getJBossAS_1009ContainedLinks(view);
		case JBossTryTryEditPart.VISUAL_ID:
			return getJBossHAGroup_1010ContainedLinks(view);
		case WebContainerEditPart.VISUAL_ID:
			return getWebContainer_2001ContainedLinks(view);
		case EJBContainerEditPart.VISUAL_ID:
			return getEJBContainer_2002ContainedLinks(view);
		case DiscoveryServiceEditPart.VISUAL_ID:
			return getDiscoveryService_2003ContainedLinks(view);
		case JmsServiceEditPart.VISUAL_ID:
			return getJmsService_2004ContainedLinks(view);
		case IIOPEditPart.VISUAL_ID:
			return getIIOP_2005ContainedLinks(view);
		case IRMIEditPart.VISUAL_ID:
			return getIRMI_2006ContainedLinks(view);
		case JRMPEditPart.VISUAL_ID:
			return getJRMP_2007ContainedLinks(view);
		case CMIEditPart.VISUAL_ID:
			return getCMI_2008ContainedLinks(view);
		case AJPEditPart.VISUAL_ID:
			return getAJP_2009ContainedLinks(view);
		case HTTPEditPart.VISUAL_ID:
			return getHTTP_2010ContainedLinks(view);
		case HTTPSEditPart.VISUAL_ID:
			return getHTTPS_2011ContainedLinks(view);
		case ModJKModuleEditPart.VISUAL_ID:
			return getModJKModule_2012ContainedLinks(view);
		case ClusterEditPart.VISUAL_ID:
			return getCluster_2013ContainedLinks(view);
		case JBossBindingEditPart.VISUAL_ID:
			return getJBossBinding_2014ContainedLinks(view);
		case WebContainer2EditPart.VISUAL_ID:
			return getWebContainer_2015ContainedLinks(view);
		case HTTPS2EditPart.VISUAL_ID:
			return getHTTPS_2016ContainedLinks(view);
		case HTTP2EditPart.VISUAL_ID:
			return getHTTP_2017ContainedLinks(view);
		case AJP2EditPart.VISUAL_ID:
			return getAJP_2018ContainedLinks(view);
		case RepHTTPConnectorEditPart.VISUAL_ID:
			return getJBossModJKConnector_3003ContainedLinks(view);
		case HAConnectorEditPart.VISUAL_ID:
			return getRepHTTPConnector_3004ContainedLinks(view);
		case DbLinkEditPart.VISUAL_ID:
			return getJBossHTTPRepConnector_3005ContainedLinks(view);
		case RmiConnectorEditPart.VISUAL_ID:
			return getHAConnector_3006ContainedLinks(view);
		case JBossHAConnectorEditPart.VISUAL_ID:
			return getJBossHAConnector_3007ContainedLinks(view);
		case DbLink2EditPart.VISUAL_ID:
			return getDbLink_3008ContainedLinks(view);
		case RmiConnector2EditPart.VISUAL_ID:
			return getRmiConnector_3009ContainedLinks(view);
		}
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIncomingLinks(View view) {
		switch (JasmineVisualIDRegistry.getVisualID(view)) {
		case JOnASEditPart.VISUAL_ID:
			return getJOnAS_1001IncomingLinks(view);
		case Apache_HTTPdEditPart.VISUAL_ID:
			return getApache_HTTPd_1002IncomingLinks(view);
		case CmiGroupEditPart.VISUAL_ID:
			return getCmiGroup_1003IncomingLinks(view);
		case HaGroupEditPart.VISUAL_ID:
			return getHaGroup_1004IncomingLinks(view);
		case HTTP_RepGroupEditPart.VISUAL_ID:
			return getHTTP_RepGroup_1005IncomingLinks(view);
		case RMIGroupEditPart.VISUAL_ID:
			return getRMIGroup_1006IncomingLinks(view);
		case ClusterContainerEditPart.VISUAL_ID:
			return getClusterContainer_1007IncomingLinks(view);
		case DBEditPart.VISUAL_ID:
			return getDB_1008IncomingLinks(view);
		case JBossASEditPart.VISUAL_ID:
			return getJBossAS_1009IncomingLinks(view);
		case JBossTryTryEditPart.VISUAL_ID:
			return getJBossHAGroup_1010IncomingLinks(view);
		case WebContainerEditPart.VISUAL_ID:
			return getWebContainer_2001IncomingLinks(view);
		case EJBContainerEditPart.VISUAL_ID:
			return getEJBContainer_2002IncomingLinks(view);
		case DiscoveryServiceEditPart.VISUAL_ID:
			return getDiscoveryService_2003IncomingLinks(view);
		case JmsServiceEditPart.VISUAL_ID:
			return getJmsService_2004IncomingLinks(view);
		case IIOPEditPart.VISUAL_ID:
			return getIIOP_2005IncomingLinks(view);
		case IRMIEditPart.VISUAL_ID:
			return getIRMI_2006IncomingLinks(view);
		case JRMPEditPart.VISUAL_ID:
			return getJRMP_2007IncomingLinks(view);
		case CMIEditPart.VISUAL_ID:
			return getCMI_2008IncomingLinks(view);
		case AJPEditPart.VISUAL_ID:
			return getAJP_2009IncomingLinks(view);
		case HTTPEditPart.VISUAL_ID:
			return getHTTP_2010IncomingLinks(view);
		case HTTPSEditPart.VISUAL_ID:
			return getHTTPS_2011IncomingLinks(view);
		case ModJKModuleEditPart.VISUAL_ID:
			return getModJKModule_2012IncomingLinks(view);
		case ClusterEditPart.VISUAL_ID:
			return getCluster_2013IncomingLinks(view);
		case JBossBindingEditPart.VISUAL_ID:
			return getJBossBinding_2014IncomingLinks(view);
		case WebContainer2EditPart.VISUAL_ID:
			return getWebContainer_2015IncomingLinks(view);
		case HTTPS2EditPart.VISUAL_ID:
			return getHTTPS_2016IncomingLinks(view);
		case HTTP2EditPart.VISUAL_ID:
			return getHTTP_2017IncomingLinks(view);
		case AJP2EditPart.VISUAL_ID:
			return getAJP_2018IncomingLinks(view);
		case RepHTTPConnectorEditPart.VISUAL_ID:
			return getJBossModJKConnector_3003IncomingLinks(view);
		case HAConnectorEditPart.VISUAL_ID:
			return getRepHTTPConnector_3004IncomingLinks(view);
		case DbLinkEditPart.VISUAL_ID:
			return getJBossHTTPRepConnector_3005IncomingLinks(view);
		case RmiConnectorEditPart.VISUAL_ID:
			return getHAConnector_3006IncomingLinks(view);
		case JBossHAConnectorEditPart.VISUAL_ID:
			return getJBossHAConnector_3007IncomingLinks(view);
		case DbLink2EditPart.VISUAL_ID:
			return getDbLink_3008IncomingLinks(view);
		case RmiConnector2EditPart.VISUAL_ID:
			return getRmiConnector_3009IncomingLinks(view);
		}
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getOutgoingLinks(View view) {
		switch (JasmineVisualIDRegistry.getVisualID(view)) {
		case JOnASEditPart.VISUAL_ID:
			return getJOnAS_1001OutgoingLinks(view);
		case Apache_HTTPdEditPart.VISUAL_ID:
			return getApache_HTTPd_1002OutgoingLinks(view);
		case CmiGroupEditPart.VISUAL_ID:
			return getCmiGroup_1003OutgoingLinks(view);
		case HaGroupEditPart.VISUAL_ID:
			return getHaGroup_1004OutgoingLinks(view);
		case HTTP_RepGroupEditPart.VISUAL_ID:
			return getHTTP_RepGroup_1005OutgoingLinks(view);
		case RMIGroupEditPart.VISUAL_ID:
			return getRMIGroup_1006OutgoingLinks(view);
		case ClusterContainerEditPart.VISUAL_ID:
			return getClusterContainer_1007OutgoingLinks(view);
		case DBEditPart.VISUAL_ID:
			return getDB_1008OutgoingLinks(view);
		case JBossASEditPart.VISUAL_ID:
			return getJBossAS_1009OutgoingLinks(view);
		case JBossTryTryEditPart.VISUAL_ID:
			return getJBossHAGroup_1010OutgoingLinks(view);
		case WebContainerEditPart.VISUAL_ID:
			return getWebContainer_2001OutgoingLinks(view);
		case EJBContainerEditPart.VISUAL_ID:
			return getEJBContainer_2002OutgoingLinks(view);
		case DiscoveryServiceEditPart.VISUAL_ID:
			return getDiscoveryService_2003OutgoingLinks(view);
		case JmsServiceEditPart.VISUAL_ID:
			return getJmsService_2004OutgoingLinks(view);
		case IIOPEditPart.VISUAL_ID:
			return getIIOP_2005OutgoingLinks(view);
		case IRMIEditPart.VISUAL_ID:
			return getIRMI_2006OutgoingLinks(view);
		case JRMPEditPart.VISUAL_ID:
			return getJRMP_2007OutgoingLinks(view);
		case CMIEditPart.VISUAL_ID:
			return getCMI_2008OutgoingLinks(view);
		case AJPEditPart.VISUAL_ID:
			return getAJP_2009OutgoingLinks(view);
		case HTTPEditPart.VISUAL_ID:
			return getHTTP_2010OutgoingLinks(view);
		case HTTPSEditPart.VISUAL_ID:
			return getHTTPS_2011OutgoingLinks(view);
		case ModJKModuleEditPart.VISUAL_ID:
			return getModJKModule_2012OutgoingLinks(view);
		case ClusterEditPart.VISUAL_ID:
			return getCluster_2013OutgoingLinks(view);
		case JBossBindingEditPart.VISUAL_ID:
			return getJBossBinding_2014OutgoingLinks(view);
		case WebContainer2EditPart.VISUAL_ID:
			return getWebContainer_2015OutgoingLinks(view);
		case HTTPS2EditPart.VISUAL_ID:
			return getHTTPS_2016OutgoingLinks(view);
		case HTTP2EditPart.VISUAL_ID:
			return getHTTP_2017OutgoingLinks(view);
		case AJP2EditPart.VISUAL_ID:
			return getAJP_2018OutgoingLinks(view);
		case RepHTTPConnectorEditPart.VISUAL_ID:
			return getJBossModJKConnector_3003OutgoingLinks(view);
		case HAConnectorEditPart.VISUAL_ID:
			return getRepHTTPConnector_3004OutgoingLinks(view);
		case DbLinkEditPart.VISUAL_ID:
			return getJBossHTTPRepConnector_3005OutgoingLinks(view);
		case RmiConnectorEditPart.VISUAL_ID:
			return getHAConnector_3006OutgoingLinks(view);
		case JBossHAConnectorEditPart.VISUAL_ID:
			return getJBossHAConnector_3007OutgoingLinks(view);
		case DbLink2EditPart.VISUAL_ID:
			return getDbLink_3008OutgoingLinks(view);
		case RmiConnector2EditPart.VISUAL_ID:
			return getRmiConnector_3009OutgoingLinks(view);
		}
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDomain_79ContainedLinks(View view) {
		Domain modelElement = (Domain) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getContainedTypeModelFacetLinks_JBossModJKConnector_3003(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_RepHTTPConnector_3004(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_JBossHTTPRepConnector_3005(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_HAConnector_3006(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_JBossHAConnector_3007(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_DbLink_3008(modelElement));
		result
				.addAll(getContainedTypeModelFacetLinks_RmiConnector_3009(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJOnAS_1001ContainedLinks(View view) {
		JOnAS modelElement = (JOnAS) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getOutgoingFeatureModelFacetLinks_JOnAS_Cmigroup_3001(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getApache_HTTPd_1002ContainedLinks(View view) {
		Apache_HTTPd modelElement = (Apache_HTTPd) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getOutgoingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getCmiGroup_1003ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHaGroup_1004ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_RepGroup_1005ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRMIGroup_1006ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getClusterContainer_1007ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDB_1008ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossAS_1009ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAGroup_1010ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2001ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getEJBContainer_2002ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDiscoveryService_2003ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJmsService_2004ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIIOP_2005ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIRMI_2006ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJRMP_2007ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCMI_2008ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2009ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2010ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2011ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getModJKModule_2012ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCluster_2013ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossBinding_2014ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2015ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2016ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2017ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2018ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossModJKConnector_3003ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRepHTTPConnector_3004ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHTTPRepConnector_3005ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHAConnector_3006ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAConnector_3007ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDbLink_3008ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRmiConnector_3009ContainedLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJOnAS_1001IncomingLinks(View view) {
		JOnAS modelElement = (JOnAS) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result
				.addAll(getIncomingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(
						modelElement, crossReferences));
		result.addAll(getIncomingTypeModelFacetLinks_DbLink_3008(modelElement,
				crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getApache_HTTPd_1002IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCmiGroup_1003IncomingLinks(View view) {
		CmiGroup modelElement = (CmiGroup) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result.addAll(getIncomingFeatureModelFacetLinks_JOnAS_Cmigroup_3001(
				modelElement, crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getHaGroup_1004IncomingLinks(View view) {
		HaGroup modelElement = (HaGroup) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result.addAll(getIncomingTypeModelFacetLinks_HAConnector_3006(
				modelElement, crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_RepGroup_1005IncomingLinks(View view) {
		HTTP_RepGroup modelElement = (HTTP_RepGroup) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result.addAll(getIncomingTypeModelFacetLinks_RepHTTPConnector_3004(
				modelElement, crossReferences));
		result
				.addAll(getIncomingTypeModelFacetLinks_JBossHTTPRepConnector_3005(
						modelElement, crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getRMIGroup_1006IncomingLinks(View view) {
		RMIGroup modelElement = (RMIGroup) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result.addAll(getIncomingTypeModelFacetLinks_RmiConnector_3009(
				modelElement, crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getClusterContainer_1007IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDB_1008IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossAS_1009IncomingLinks(View view) {
		JBossAS modelElement = (JBossAS) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result
				.addAll(getIncomingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(
						modelElement, crossReferences));
		result.addAll(getIncomingTypeModelFacetLinks_JBossModJKConnector_3003(
				modelElement, crossReferences));
		result.addAll(getIncomingTypeModelFacetLinks_DbLink_3008(modelElement,
				crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAGroup_1010IncomingLinks(View view) {
		JBossHAGroup modelElement = (JBossHAGroup) view.getElement();
		Map crossReferences = EcoreUtil.CrossReferencer.find(view.eResource()
				.getResourceSet().getResources());
		List result = new LinkedList();
		result.addAll(getIncomingTypeModelFacetLinks_JBossHAConnector_3007(
				modelElement, crossReferences));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2001IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getEJBContainer_2002IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDiscoveryService_2003IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJmsService_2004IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIIOP_2005IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIRMI_2006IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJRMP_2007IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCMI_2008IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2009IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2010IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2011IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getModJKModule_2012IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCluster_2013IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossBinding_2014IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2015IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2016IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2017IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2018IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossModJKConnector_3003IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRepHTTPConnector_3004IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHTTPRepConnector_3005IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHAConnector_3006IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAConnector_3007IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDbLink_3008IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRmiConnector_3009IncomingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJOnAS_1001OutgoingLinks(View view) {
		JOnAS modelElement = (JOnAS) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getOutgoingFeatureModelFacetLinks_JOnAS_Cmigroup_3001(modelElement));
		result
				.addAll(getOutgoingTypeModelFacetLinks_RepHTTPConnector_3004(modelElement));
		result
				.addAll(getOutgoingTypeModelFacetLinks_HAConnector_3006(modelElement));
		result
				.addAll(getOutgoingTypeModelFacetLinks_RmiConnector_3009(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getApache_HTTPd_1002OutgoingLinks(View view) {
		Apache_HTTPd modelElement = (Apache_HTTPd) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getOutgoingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(modelElement));
		result
				.addAll(getOutgoingTypeModelFacetLinks_JBossModJKConnector_3003(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getCmiGroup_1003OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHaGroup_1004OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_RepGroup_1005OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRMIGroup_1006OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getClusterContainer_1007OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDB_1008OutgoingLinks(View view) {
		org.objectweb.jasmine.ui.jasminemodel.DB modelElement = (org.objectweb.jasmine.ui.jasminemodel.DB) view
				.getElement();
		List result = new LinkedList();
		result.addAll(getOutgoingTypeModelFacetLinks_DbLink_3008(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJBossAS_1009OutgoingLinks(View view) {
		JBossAS modelElement = (JBossAS) view.getElement();
		List result = new LinkedList();
		result
				.addAll(getOutgoingTypeModelFacetLinks_JBossHTTPRepConnector_3005(modelElement));
		result
				.addAll(getOutgoingTypeModelFacetLinks_JBossHAConnector_3007(modelElement));
		return result;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAGroup_1010OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2001OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getEJBContainer_2002OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDiscoveryService_2003OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJmsService_2004OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIIOP_2005OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getIRMI_2006OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJRMP_2007OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCMI_2008OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2009OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2010OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2011OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getModJKModule_2012OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getCluster_2013OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossBinding_2014OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getWebContainer_2015OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTPS_2016OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHTTP_2017OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getAJP_2018OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossModJKConnector_3003OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRepHTTPConnector_3004OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHTTPRepConnector_3005OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getHAConnector_3006OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getJBossHAConnector_3007OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getDbLink_3008OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	public static List getRmiConnector_3009OutgoingLinks(View view) {
		return Collections.EMPTY_LIST;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_JBossModJKConnector_3003(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossModJKConnector) {
				continue;
			}
			JBossModJKConnector link = (JBossModJKConnector) linkObject;
			if (RepHTTPConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossAS dst = link.getJboss();
			Apache_HTTPd src = link.getApache();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossModJKConnector_3003,
					RepHTTPConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_RepHTTPConnector_3004(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof RepHTTPConnector) {
				continue;
			}
			RepHTTPConnector link = (RepHTTPConnector) linkObject;
			if (HAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HTTP_RepGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.RepHTTPConnector_3004,
					HAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_JBossHTTPRepConnector_3005(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossHTTPRepConnector) {
				continue;
			}
			JBossHTTPRepConnector link = (JBossHTTPRepConnector) linkObject;
			if (DbLinkEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HTTP_RepGroup dst = link.getGroup();
			JBossAS src = link.getInstance();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossHTTPRepConnector_3005,
					DbLinkEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_HAConnector_3006(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof HAConnector) {
				continue;
			}
			HAConnector link = (HAConnector) linkObject;
			if (RmiConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HaGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.HAConnector_3006,
					RmiConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_JBossHAConnector_3007(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossHAConnector) {
				continue;
			}
			JBossHAConnector link = (JBossHAConnector) linkObject;
			if (JBossHAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossHAGroup dst = link.getGroup();
			JBossAS src = link.getInstance();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossHAConnector_3007,
					JBossHAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_DbLink_3008(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getDblinks().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof DbLink) {
				continue;
			}
			DbLink link = (DbLink) linkObject;
			if (DbLink2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			IJavaEEServer dst = link.getTarget();
			org.objectweb.jasmine.ui.jasminemodel.DB src = link.getSource();
			result
					.add(new JasmineLinkDescriptor(src, dst, link,
							JasmineElementTypes.DbLink_3008,
							DbLink2EditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getContainedTypeModelFacetLinks_RmiConnector_3009(
			Domain container) {
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof RmiConnector) {
				continue;
			}
			RmiConnector link = (RmiConnector) linkObject;
			if (RmiConnector2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			RMIGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.RmiConnector_3009,
					RmiConnector2EditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingFeatureModelFacetLinks_JOnAS_Cmigroup_3001(
			CmiGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() == JasminemodelPackage.eINSTANCE
					.getJOnAS_Cmigroup()) {
				result.add(new JasmineLinkDescriptor(setting.getEObject(),
						target, JasmineElementTypes.JOnASCmigroup_3001,
						CMIConnectorEditPart.VISUAL_ID));
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(
			IJavaEEServer target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() == JasminemodelPackage.eINSTANCE
					.getIHTTPServer_JavaeeServers()) {
				result.add(new JasmineLinkDescriptor(setting.getEObject(),
						target,
						JasmineElementTypes.IHTTPServerJavaeeServers_3002,
						ModjkConnectorEditPart.VISUAL_ID));
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_JBossModJKConnector_3003(
			JBossAS target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getJBossModJKConnector_Jboss()
					|| false == setting.getEObject() instanceof JBossModJKConnector) {
				continue;
			}
			JBossModJKConnector link = (JBossModJKConnector) setting
					.getEObject();
			if (RepHTTPConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			Apache_HTTPd src = link.getApache();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.JBossModJKConnector_3003,
					RepHTTPConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_RepHTTPConnector_3004(
			HTTP_RepGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getRepHTTPConnector_Target()
					|| false == setting.getEObject() instanceof RepHTTPConnector) {
				continue;
			}
			RepHTTPConnector link = (RepHTTPConnector) setting.getEObject();
			if (HAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.RepHTTPConnector_3004,
					HAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_JBossHTTPRepConnector_3005(
			HTTP_RepGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getJBossHTTPRepConnector_Group()
					|| false == setting.getEObject() instanceof JBossHTTPRepConnector) {
				continue;
			}
			JBossHTTPRepConnector link = (JBossHTTPRepConnector) setting
					.getEObject();
			if (DbLinkEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossAS src = link.getInstance();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.JBossHTTPRepConnector_3005,
					DbLinkEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_HAConnector_3006(
			HaGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getHAConnector_Target()
					|| false == setting.getEObject() instanceof HAConnector) {
				continue;
			}
			HAConnector link = (HAConnector) setting.getEObject();
			if (RmiConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.HAConnector_3006,
					RmiConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_JBossHAConnector_3007(
			JBossHAGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getJBossHAConnector_Group()
					|| false == setting.getEObject() instanceof JBossHAConnector) {
				continue;
			}
			JBossHAConnector link = (JBossHAConnector) setting.getEObject();
			if (JBossHAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossAS src = link.getInstance();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.JBossHAConnector_3007,
					JBossHAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_DbLink_3008(
			IJavaEEServer target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getDbLink_Target()
					|| false == setting.getEObject() instanceof DbLink) {
				continue;
			}
			DbLink link = (DbLink) setting.getEObject();
			if (DbLink2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			org.objectweb.jasmine.ui.jasminemodel.DB src = link.getSource();
			result
					.add(new JasmineLinkDescriptor(src, target, link,
							JasmineElementTypes.DbLink_3008,
							DbLink2EditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getIncomingTypeModelFacetLinks_RmiConnector_3009(
			RMIGroup target, Map crossReferences) {
		Collection result = new LinkedList();
		Collection settings = (Collection) crossReferences.get(target);
		for (Iterator it = settings.iterator(); it.hasNext();) {
			EStructuralFeature.Setting setting = (EStructuralFeature.Setting) it
					.next();
			if (setting.getEStructuralFeature() != JasminemodelPackage.eINSTANCE
					.getRmiConnector_Target()
					|| false == setting.getEObject() instanceof RmiConnector) {
				continue;
			}
			RmiConnector link = (RmiConnector) setting.getEObject();
			if (RmiConnector2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JOnAS src = link.getSource();
			result.add(new JasmineLinkDescriptor(src, target, link,
					JasmineElementTypes.RmiConnector_3009,
					RmiConnector2EditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingFeatureModelFacetLinks_JOnAS_Cmigroup_3001(
			JOnAS source) {
		Collection result = new LinkedList();
		CmiGroup destination = source.getCmigroup();
		result.add(new JasmineLinkDescriptor(source, destination,
				JasmineElementTypes.JOnASCmigroup_3001,
				CMIConnectorEditPart.VISUAL_ID));
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingFeatureModelFacetLinks_IHTTPServer_JavaeeServers_3002(
			IHTTPServer source) {
		Collection result = new LinkedList();
		for (Iterator destinations = source.getJavaeeServers().iterator(); destinations
				.hasNext();) {
			IJavaEEServer destination = (IJavaEEServer) destinations.next();
			result.add(new JasmineLinkDescriptor(source, destination,
					JasmineElementTypes.IHTTPServerJavaeeServers_3002,
					ModjkConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_JBossModJKConnector_3003(
			Apache_HTTPd source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossModJKConnector) {
				continue;
			}
			JBossModJKConnector link = (JBossModJKConnector) linkObject;
			if (RepHTTPConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossAS dst = link.getJboss();
			Apache_HTTPd src = link.getApache();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossModJKConnector_3003,
					RepHTTPConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_RepHTTPConnector_3004(
			JOnAS source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof RepHTTPConnector) {
				continue;
			}
			RepHTTPConnector link = (RepHTTPConnector) linkObject;
			if (HAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HTTP_RepGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.RepHTTPConnector_3004,
					HAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_JBossHTTPRepConnector_3005(
			JBossAS source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossHTTPRepConnector) {
				continue;
			}
			JBossHTTPRepConnector link = (JBossHTTPRepConnector) linkObject;
			if (DbLinkEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HTTP_RepGroup dst = link.getGroup();
			JBossAS src = link.getInstance();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossHTTPRepConnector_3005,
					DbLinkEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_HAConnector_3006(
			JOnAS source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof HAConnector) {
				continue;
			}
			HAConnector link = (HAConnector) linkObject;
			if (RmiConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			HaGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.HAConnector_3006,
					RmiConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_JBossHAConnector_3007(
			JBossAS source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof JBossHAConnector) {
				continue;
			}
			JBossHAConnector link = (JBossHAConnector) linkObject;
			if (JBossHAConnectorEditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			JBossHAGroup dst = link.getGroup();
			JBossAS src = link.getInstance();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.JBossHAConnector_3007,
					JBossHAConnectorEditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_DbLink_3008(
			org.objectweb.jasmine.ui.jasminemodel.DB source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getDblinks().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof DbLink) {
				continue;
			}
			DbLink link = (DbLink) linkObject;
			if (DbLink2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			IJavaEEServer dst = link.getTarget();
			org.objectweb.jasmine.ui.jasminemodel.DB src = link.getSource();
			if (src != source) {
				continue;
			}
			result
					.add(new JasmineLinkDescriptor(src, dst, link,
							JasmineElementTypes.DbLink_3008,
							DbLink2EditPart.VISUAL_ID));
		}
		return result;
	}

	/**
	 * @generated
	 */
	private static Collection getOutgoingTypeModelFacetLinks_RmiConnector_3009(
			JOnAS source) {
		Domain container = null;
		// Find container element for the link.
		// Climb up by containment hierarchy starting from the source
		// and return the first element that is instance of the container class.
		for (EObject element = source; element != null && container == null; element = element
				.eContainer()) {
			if (element instanceof Domain) {
				container = (Domain) element;
			}
		}
		if (container == null) {
			return Collections.EMPTY_LIST;
		}
		Collection result = new LinkedList();
		for (Iterator links = container.getConnectors().iterator(); links
				.hasNext();) {
			Object linkObject = links.next();
			if (false == linkObject instanceof RmiConnector) {
				continue;
			}
			RmiConnector link = (RmiConnector) linkObject;
			if (RmiConnector2EditPart.VISUAL_ID != JasmineVisualIDRegistry
					.getLinkWithClassVisualID(link)) {
				continue;
			}
			RMIGroup dst = link.getTarget();
			JOnAS src = link.getSource();
			if (src != source) {
				continue;
			}
			result.add(new JasmineLinkDescriptor(src, dst, link,
					JasmineElementTypes.RmiConnector_3009,
					RmiConnector2EditPart.VISUAL_ID));
		}
		return result;
	}

}
