/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.ui.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.progress.UIJob;
import org.objectweb.jasmine.jade.service.deployer.DeploymentException;
import org.objectweb.jasmine.ui.jasminemodel.IMiddlewareComponent;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineDiagramEditorPlugin;
import org.objectweb.jasmine.ui.jasminemodel.diagram.ui.util.JasmineModelValidator;

public class DeployMiddlewareAction implements IObjectActionDelegate {

	/**
	 *
	 */
	private Object selection;

	public void run(IAction action) {

		if (((IStructuredSelection) selection).getFirstElement() instanceof IMiddlewareComponent) {

			final IMiddlewareComponent theComponent = (IMiddlewareComponent) ((IStructuredSelection) selection)
					.getFirstElement();

			try {
				// Do the work within a job because this is a long running
				// activity
				Job theJob = new UIJob("Deploy") {

					@Override
					public IStatus runInUIThread(IProgressMonitor monitor) {

						List<String> result = new ArrayList<String>();

						int status = JasmineModelValidator.validate(
								theComponent, result);

						/*
						 * the model is valid, launch the deployment
						 */
						if (status == IStatus.OK) {

							try {

								theComponent.deploy();

								MessageDialog.openInformation(Display
										.getCurrent().getActiveShell(),
										"Deployment succeed",
										((IStructuredSelection) selection)
												.getFirstElement().getClass()
												.getSimpleName()
												+ " successfully deployed");

								return Status.OK_STATUS;

							} catch (DeploymentException e) {

								JasmineDiagramEditorPlugin
										.getInstance()
										.logError("Deployment action failed", e);

								MessageDialog.openError(Display.getCurrent()
												.getActiveShell(),
												"Error during deployment ",
												"Deployment failed : "
														+ e.getMessage());

								return Status.CANCEL_STATUS;

							}
						}
						/*
						 * The model is not valid
						 */
						else {

							StringBuffer messages = new StringBuffer();
							for (String s : result) {
								messages.append(s);
								messages.append("\n");
							}

							String message = ((IStructuredSelection) selection)
									.getFirstElement().getClass()
									.getSimpleName()
									+ " is not valid : \n\n"
									+ messages.toString();

							MessageDialog.openInformation(Display.getCurrent()
									.getActiveShell(), "Model not valid",
									message);

							return Status.CANCEL_STATUS;

						}
					}
				};

				theJob.schedule();

			} catch (Exception e) {
				JasmineDiagramEditorPlugin.getInstance().logError(
						"Deployment action failed", e); //$NON-NLS-1$

				MessageDialog.openError(null, "Error",
						"An error has occured during deployment. "
								+ e.getMessage());
			}

		} else {

			// We should never go there
			MessageDialog.openInformation(null, "Popup Menu Action executed",
					"Operation unsupported for object "
							+ ((IStructuredSelection) selection)
									.getFirstElement().getClass()
									.getSimpleName());
			throw new UnsupportedOperationException();
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = (IStructuredSelection) selection;
	}

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {

	}

	public void init(IViewPart view) {

	}
}
