/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.diagram.ui.views;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.BasicCommandStack;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CommandStack;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.ui.viewer.IViewerProvider;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EValidator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.provider.AdapterFactoryItemDelegator;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.celleditor.AdapterFactoryTreeEditor;
import org.eclipse.emf.edit.ui.dnd.EditingDomainViewerDropAdapter;
import org.eclipse.emf.edit.ui.dnd.LocalTransfer;
import org.eclipse.emf.edit.ui.dnd.ViewerDragAdapter;
import org.eclipse.emf.transaction.NotificationFilter;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.ui.provider.TransactionalAdapterFactoryContentProvider;
import org.eclipse.emf.transaction.ui.provider.TransactionalAdapterFactoryLabelProvider;
import org.eclipse.emf.transaction.ui.view.ExtendedPropertySheetPage;
import org.eclipse.emf.workspace.util.WorkspaceSynchronizer;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequestFactory;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IGotoMarker;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.contentoutline.ContentOutline;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.objectweb.jasmine.ui.jasminemodel.JASMINeNode;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelFactory;
import org.objectweb.jasmine.ui.jasminemodel.diagram.ext.presentation.ResourceLoadedListener;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineDiagramEditorPlugin;
import org.objectweb.jasmine.ui.jasminemodel.diagram.ui.treeviews.DefaultNodeExplorerFilter;
import org.objectweb.jasmine.ui.jasminemodel.diagram.ui.treeviews.JasmineNodeFilter;
import org.objectweb.jasmine.ui.jasminemodel.provider.JasmineEditPlugin;
import org.objectweb.jasmine.ui.jasminemodel.provider.JasminemodelItemProviderAdapterFactory;



/**
 * This sample class demonstrates how to plug-in a new workbench view. The view shows data obtained
 * from the model. The sample creates a dummy model on the fly, but a real implementation would
 * connect to the model available either in this or another plug-in (e.g. the workspace). The view
 * is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be presented in the view. Each
 * view can present the same model objects using different labels and icons, if needed.
 * Alternatively, a single label provider can be shared between views in order to ensure that
 * objects of the same type are presented in the same way everywhere.
 * <p>
 */
public class JasmineNodeExplorerView extends ViewPart
implements IEditingDomainProvider,
ISelectionProvider, IMenuListener, IViewerProvider, IGotoMarker,
ISelectionListener
{

	public static final String ID = "org.ow2.jasmine.model.diagram.JasmineNodeExplorerView";

	private DrillDownAdapter drillDownAdapter;

	private Action refreshAction, saveAction, addItemAction, deleteItemAction, onlyJasmineNodesAction;

	// ------------------------------------------------------------

	private JasminemodelActionBarContributor _actionBarContributor;

	/**
	 * This is the one adapter factory used for providing views of the model.
	 */
	protected ComposedAdapterFactory adapterFactory;

	/**
	 * This is the content outline page.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	protected IContentOutlinePage contentOutlinePage;

	/**
	 * This keeps track of the active content viewer, which may be either one of the viewers in the
	 * pages or the content outline viewer.
	 */
	private TreeViewer currentViewer;

	/**
	 * This is the content outline page's viewer.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	protected TreeViewer contentOutlineViewer;

	/**
	 * This is a kludge...
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	protected IStatusLineManager contentOutlineStatusLineManager;

	private ViewerFilter onlyJasmineNodesFilter, defaultFilter;

	/**
	 * This is the viewer that shadows the selection in the content outline.
	 * The parent relation must be correctly defined for this to work.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	protected TreeViewer selectionViewer;

	/**
	 * This is the property sheet page.
	 */
	protected PropertySheetPage propertySheetPage;

	/**
	 * This keeps track of the editing domain that is used to track all changes to the model.
	 */
	protected AdapterFactoryEditingDomain editingDomain;

	private CreateViewRequestFactory viewRequest;

	/**
	 * This listens to which ever viewer is active.
	 */
	protected ISelectionChangedListener selectionChangedListener = new ISelectionChangedListener()
	{
		// This just notifies those things that are affected by the section.
		//
		public void selectionChanged(SelectionChangedEvent selectionChangedEvent)
		{
			setSelection(selectionChangedEvent.getSelection());
		}
	};;

	/**
	 * This keeps track of all the {@link org.eclipse.jface.viewers.ISelectionChangedListener}s that
	 * are listening to this editor.
	 */
	protected Collection selectionChangedListeners = new ArrayList();

	/**
	 * This keeps track of the selection of the editor as a whole.
	 */
	protected ISelection editorSelection = StructuredSelection.EMPTY;

	protected IUndoContext undoContext;

	public static String RESOURCE_URI;

	  /**
	   * Controls whether the problem indication should be updated.
	   * <!-- begin-user-doc -->
	   * <!-- end-user-doc -->
	   */
	  protected boolean updateProblemIndication = true;


	/**
	 * This listens for when the outline becomes active
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	protected IPartListener partListener =
		new IPartListener() {
		public void partActivated(IWorkbenchPart p) {
			if (p instanceof ContentOutline) {
				if (((ContentOutline)p).getCurrentPage() == contentOutlinePage) {
					getActionBarContributor().setActiveView(JasmineNodeExplorerView.this);

//					setCurrentViewer(contentOutlineViewer);
				}
			}
			else if (p instanceof PropertySheet) {
				if (((PropertySheet)p).getCurrentPage() == propertySheetPage) {
					getActionBarContributor().setActiveView(JasmineNodeExplorerView.this);
					handleActivate();
				}
			}
			else if (p == JasmineNodeExplorerView.this) {
				handleActivate();
			}
		}
		public void partBroughtToTop(IWorkbenchPart p) {
		}
		public void partClosed(IWorkbenchPart p) {
		}
		public void partDeactivated(IWorkbenchPart p) {
		}
		public void partOpened(IWorkbenchPart p) {
		}
	};
	class NameSorter extends ViewerSorter
	{
	}

	/**
	 * This creates a model view.
	 */
	public JasmineNodeExplorerView()
	{
		// Create an adapter factory that yields item providers.
		List factories = new ArrayList();
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new JasminemodelItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());

		// Get the registered workbench editing domain.
		//
		editingDomain = (AdapterFactoryEditingDomain) TransactionalEditingDomain.Registry
		.INSTANCE.getEditingDomain("org.ow2.jasmine.model.diagram.JasmineEditingDomain");

		adapterFactory = (ComposedAdapterFactory)editingDomain.getAdapterFactory();


		final NotificationFilter diagramResourceModifiedFilter = NotificationFilter
		.createNotifierFilter(editingDomain.getResourceSet()).and(
				NotificationFilter
				.createEventTypeFilter(Notification.ADD)).and(
						NotificationFilter.createFeatureFilter(
								ResourceSet.class,
								ResourceSet.RESOURCE_SET__RESOURCES));
		editingDomain.getResourceSet().eAdapters().add(new Adapter() {

			private Notifier myTarger;

			public Notifier getTarget() {
				return myTarger;
			}

			public boolean isAdapterForType(Object type) {
				return false;
			}

			public void notifyChanged(Notification notification) {
				if (diagramResourceModifiedFilter.matches(notification)) {
					Object value = notification.getNewValue();
					if (value instanceof Resource) {
						((Resource) value).setTrackingModification(true);
						System.out.println("notifyChanged!!");

					}
				}
			}

			public void setTarget(Notifier newTarget) {
				myTarger = newTarget;
			}
		});


		// Create the command stack that will notify this editor as commands are executed.
		BasicCommandStack commandStack = new BasicCommandStack();

		// Add a listener to set the most recent command's affected objects to be the selection of the
		// viewer with focus.
		commandStack.addCommandStackListener(new CommandStackListener()
		{
			public void commandStackChanged(final EventObject event)
			{
				getSite().getShell().getDisplay().asyncExec(new Runnable()
				{
					public void run()
					{
						firePropertyChange(IEditorPart.PROP_DIRTY);

						// Try to select the affected objects.
						Command mostRecentCommand = ((CommandStack)event.getSource()).getMostRecentCommand();

						if (mostRecentCommand != null)
						{
							setSelectionToViewer(mostRecentCommand.getAffectedObjects());
						}

						if (propertySheetPage != null)
						{
							propertySheetPage.refresh();
						}
					}
				});
			}
		});
	}



	/**
	 * This is here for the listener to be able to call it. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 *
	 */
	protected void firePropertyChange(int action) {
		super.firePropertyChange(action);
	}



	@Override
	public void init(IViewSite site) throws PartInitException
	{
		setSite(site);
		setPartName("Nodes Explorer");
		site.setSelectionProvider(this);
		site.getPage().addPartListener(partListener);

		ResourcesPlugin.getWorkspace()
		.addResourceChangeListener(resourceChangeListener, IResourceChangeEvent.POST_CHANGE);

		workspaceSynchronizer = new WorkspaceSynchronizer(
				(TransactionalEditingDomain) editingDomain,
				createSynchronizationDelegate());
	}

	IResourceChangeListener resourceChangeListener = new IResourceChangeListener() {
		public void resourceChanged(IResourceChangeEvent event) {
			IResourceDelta delta = event.getDelta();
		}
	};



	/**
	 * This sets the selection into whichever viewer is active.
	 */
	public void setSelectionToViewer(Collection collection)
	{
		final Collection theSelection = collection;
		// Make sure it's okay.
		//
		if (theSelection != null && !theSelection.isEmpty())
		{
			// I don't know if this should be run this deferred
			// because we might have to give the editor a chance to process the viewer update events
			// and hence to update the views first.
			//
			//
			Runnable runnable = new Runnable()
			{
				public void run()
				{
					// Try to select the items in the current content viewer of the editor.
					//
					if (currentViewer != null)
					{
						currentViewer.setSelection(new StructuredSelection(theSelection.toArray()), true);
					}
				}
			};
			runnable.run();
		}
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent)
	{
		if (RESOURCE_URI != null) {
			URI uri = URI.createFileURI(this.RESOURCE_URI);
			editingDomain.getResourceSet().getResource(uri, true);
		}


		currentViewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);


		currentViewer.setContentProvider(new TransactionalAdapterFactoryContentProvider((TransactionalEditingDomain) getEditingDomain(), adapterFactory));
		currentViewer.setLabelProvider(new TransactionalAdapterFactoryLabelProvider((TransactionalEditingDomain) getEditingDomain(), adapterFactory));

		new AdapterFactoryTreeEditor(currentViewer.getTree(), adapterFactory);



		currentViewer.setInput(editingDomain.getResourceSet());

		getViewSite().getPage().addSelectionListener(this);

		// ENABLE PROPERTIES PAGE TO BE DISPLAYED
		currentViewer.addSelectionChangedListener(selectionChangedListener);


		drillDownAdapter = new DrillDownAdapter(currentViewer);

		// Add default filter
		createFiltersAndSorters();
		currentViewer.addFilter(defaultFilter);

		createActions();
		hookContextMenu();
		contributeToActionBars();

		setSelection(currentViewer.getSelection());
	}



	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
		}
	}


	/**
	 * This creates a context menu for the viewer and adds a listener as well registering the menu for
	 * extension.
	 */
	private void hookContextMenu()
	{
		MenuManager contextMenu = new MenuManager("#PopUp");
		contextMenu.add(new Separator("additions"));
		contextMenu.setRemoveAllWhenShown(true);

		contextMenu.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager mgr) {
				fillContextMenu(mgr);
			}
		});

		Menu menu = contextMenu.createContextMenu(currentViewer.getControl());
		currentViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(contextMenu, currentViewer);

		int dndOperations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK;
		Transfer[] transfers = new Transfer[] {LocalTransfer.getInstance()};
		currentViewer.addDragSupport(dndOperations, transfers, new ViewerDragAdapter(currentViewer));
		currentViewer.addDropSupport(dndOperations, transfers, new EditingDomainViewerDropAdapter(
				editingDomain, currentViewer));

	}

	private void contributeToActionBars()
	{
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager)
	{
		manager.add(addItemAction);
		manager.add(deleteItemAction);
		manager.add(new Separator());
		manager.add(onlyJasmineNodesAction);
		manager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillContextMenu(IMenuManager manager)
	{
		manager.add(addItemAction);
		manager.add(deleteItemAction);
		manager.add(new Separator());
		drillDownAdapter.addNavigationActions(manager);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager)
	{
		manager.add(refreshAction);
		manager.add(saveAction);
		manager.add(addItemAction);
		manager.add(deleteItemAction);
		manager.add(new Separator());
		drillDownAdapter.addNavigationActions(manager);
	}

	protected void createFiltersAndSorters() {
		onlyJasmineNodesFilter = new JasmineNodeFilter();
		defaultFilter = new DefaultNodeExplorerFilter();
	}


	/**
	 * Create action
	 * This method should implement **all** the actions added in fillContextMenu
	 */
	private void createActions() {
		ImageDescriptor descriptor = null;
		onlyJasmineNodesAction = new Action("Only JASMINe Nodes") {
		public void run() {
		updateFilter(onlyJasmineNodesAction);
		}
		};
		onlyJasmineNodesAction.setChecked(false);



		saveAction = new Action("Save") {
			public void run() {
				doSave();
				getSite().getWorkbenchWindow().getWorkbench().saveAllEditors(false);
			}
		};
		saveAction.setToolTipText("Save");

		descriptor =
			JasmineDiagramEditorPlugin.findImageDescriptor("icons/eview16/save.gif");
		saveAction.setImageDescriptor(descriptor);


		refreshAction = new Action("Refresh") {
			public void run() {
				refreshModel();
				currentViewer.refresh();
			}
		};
		refreshAction.setToolTipText("Refresh");

		descriptor =
			JasmineDiagramEditorPlugin.findImageDescriptor("icons/eview16/refresh.gif");
		refreshAction.setImageDescriptor(descriptor);


		addItemAction = new Action("Add Node") {
			public void run() {
				addNewNode();
			}
		};
		addItemAction.setToolTipText("Add a New Node");

		descriptor =
			JasmineDiagramEditorPlugin.findImageDescriptor("icons/eview16/newNode.gif");
		addItemAction.setImageDescriptor(descriptor);

		deleteItemAction = new Action("Delete Node") {
			public void run() {
				removeSelected();
			}
		};
		deleteItemAction.setToolTipText("Delete Node");
		descriptor =
			JasmineDiagramEditorPlugin.findImageDescriptor("icons/eview16/remove.gif");
		deleteItemAction.setImageDescriptor(descriptor);
	}



	/* Multiple filters can be enabled at a time. */
	protected void updateFilter(Action action) {
		if(action == onlyJasmineNodesAction) {
			if(action.isChecked()) {
				currentViewer.addFilter(onlyJasmineNodesFilter);
			} else {
				currentViewer.removeFilter(onlyJasmineNodesFilter);
			}
		}
	}



	protected void removeSelected() {
		// Implement deletion of a JASMINe Node
		ISelection selection = currentViewer.getSelection();
		Object obj = ((IStructuredSelection)selection).getFirstElement();

		if (obj instanceof JASMINeNode) {
			deleteNode((JASMINeNode)obj);
		}

	}


	private void addNewNode() {
		createNode (getResource());
		currentViewer.refresh();
	}


	private void showMessage(String message)
	{
		MessageDialog
		.openInformation(currentViewer.getControl().getShell(), "System Explorer", message);
	}

	/**
	 * This implements {@link org.eclipse.jface.viewers.ISelectionProvider}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		selectionChangedListeners.add(listener);
	}



	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus()
	{
		currentViewer.getControl().setFocus();
	}

	/**
	 * This accesses a cached version of the property sheet.
	 */
	public IPropertySheetPage getPropertySheetPage()
	{
		if (propertySheetPage == null)
		{
			propertySheetPage =
				new ExtendedPropertySheetPage(editingDomain) {
				public void setSelectionToViewer(List selection) {
					JasmineNodeExplorerView.this.setSelectionToViewer(selection);
					JasmineNodeExplorerView.this.setFocus();
				}

				public void setActionBars(IActionBars actionBars) {
					super.setActionBars(actionBars);
					getActionBarContributor().shareGlobalActions(this, actionBars);
				}
			};

			propertySheetPage
			.setPropertySourceProvider(
					new TransactionalAdapterFactoryContentProvider(
							(TransactionalEditingDomain) getEditingDomain(), adapterFactory));
		}
		return propertySheetPage;
	}

	/**
	 * This returns the editing domain as required by the {@link IEditingDomainProvider} interface.
	 * This is important for implementing the static methods of {@link AdapterFactoryEditingDomain}
	 * and for supporting {@link org.eclipse.emf.edit.ui.action.CommandAction}.
	 */
	public EditingDomain getEditingDomain()
	{
		return editingDomain;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public class ReverseAdapterFactoryContentProvider extends TransactionalAdapterFactoryContentProvider {
		public ReverseAdapterFactoryContentProvider(AdapterFactory adapterFactory) {
			super((TransactionalEditingDomain) getEditingDomain(), adapterFactory);
		}

		public Object [] getElements(Object object) {
			Object parent = super.getParent(object);
			return (parent == null ? Collections.EMPTY_SET : Collections.singleton(parent)).toArray();
		}

		public Object [] getChildren(Object object) {
			Object parent = super.getParent(object);
			return (parent == null ? Collections.EMPTY_SET : Collections.singleton(parent)).toArray();
		}

		public boolean hasChildren(Object object) {
			Object parent = super.getParent(object);
			return parent != null;
		}

		public Object getParent(Object object) {
			return null;
		}
	}

	public synchronized JasminemodelActionBarContributor getActionBarContributor()
	{
		if (_actionBarContributor == null)
		{
			_actionBarContributor = new JasminemodelActionBarContributor();
			_actionBarContributor.init(getActionBars(), getSite().getPage());
			_actionBarContributor.setActiveView(this);
		}

		return _actionBarContributor;
	}

	public IActionBars getActionBars()
	{
		return getViewSite().getActionBars();
	}

	public AdapterFactory getAdapterFactory()
	{
		return adapterFactory;
	}

	public void gotoMarker(IMarker marker)
	{
		try
		{
			if (marker.getType().equals(EValidator.MARKER))
			{
				String uriAttribute = marker.getAttribute(EValidator.URI_ATTRIBUTE, null);
				if (uriAttribute != null)
				{
					URI uri = URI.createURI(uriAttribute);
					EObject eObject = editingDomain.getResourceSet().getEObject(uri, true);
					if (eObject != null)
					{
						setSelectionToViewer(Collections.singleton(editingDomain.getWrapper(eObject)));
					}
				}
			}
		}
		catch (CoreException exception)
		{
			JasmineEditPlugin.INSTANCE.log(exception);
		}
	}

	/**
	 * This implements {@link org.eclipse.jface.action.IMenuListener} to help fill the context menus
	 * with contributions from the Edit menu.
	 */
	public void menuAboutToShow(IMenuManager menuManager)
	{
		getActionBarContributor().menuAboutToShow(menuManager);
	}


	/**
	 * This implements {@link org.eclipse.jface.viewers.ISelectionProvider}.
	 */
	public void removeSelectionChangedListener(ISelectionChangedListener listener)
	{
		selectionChangedListeners.remove(listener);
	}

	/**
	 * This implements {@link org.eclipse.jface.viewers.ISelectionProvider} to return this editor's
	 * overall selection. <!-- begin-user-doc --> <!-- end-user-doc -->
	 *
	 * @generated
	 */
	public ISelection getSelection()
	{
		return editorSelection;
	}

	/**
	 * This implements {@link org.eclipse.jface.viewers.ISelectionProvider} to set this editor's
	 * overall selection. Calling this result will notify the listeners.
	 */
	public void setSelection(ISelection selection)
	{
		editorSelection = selection;

		for (Iterator listeners = selectionChangedListeners.iterator(); listeners.hasNext();)
		{
			ISelectionChangedListener listener = (ISelectionChangedListener)listeners.next();
			listener.selectionChanged(new SelectionChangedEvent(this, selection));
		}

		setStatusLineManager(selection);
	}

	public void setStatusLineManager(ISelection selection)
	{
		IStatusLineManager statusLineManager = getActionBars().getStatusLineManager();

		if (statusLineManager != null)
		{
			if (selection instanceof IStructuredSelection)
			{
				Collection collection = ((IStructuredSelection)selection).toList();
				switch (collection.size())
				{
				case 0:
				{
					statusLineManager.setMessage(getString("_UI_NoObjectSelected"));
					break;
				}
				case 1:
				{
					String text = new AdapterFactoryItemDelegator(adapterFactory).getText(collection
							.iterator().next());
					statusLineManager.setMessage(getString("_UI_SingleObjectSelected", text));
					break;
				}
				default:
				{
					statusLineManager.setMessage(getString("_UI_MultiObjectSelected", Integer
							.toString(collection.size())));
					break;
				}
				}
			}
			else
			{
				statusLineManager.setMessage("");
			}
		}
	}


	/**
	 * This creates a context menu for the viewer and adds a listener as well registering the menu for extension.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void createContextMenuFor(StructuredViewer viewer) {
		MenuManager contextMenu = new MenuManager("#PopUp"); //$NON-NLS-1$
		contextMenu.add(new Separator("additions")); //$NON-NLS-1$
		contextMenu.setRemoveAllWhenShown(true);
		contextMenu.addMenuListener(this);
		Menu menu= contextMenu.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(contextMenu, viewer);

		int dndOperations = DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK;
		Transfer[] transfers = new Transfer[] { LocalTransfer.getInstance() };
		viewer.addDragSupport(dndOperations, transfers, new ViewerDragAdapter(viewer));
		viewer.addDropSupport(dndOperations, transfers, new EditingDomainViewerDropAdapter(editingDomain, viewer));
	}

	/**
	 * This returns the viewer as required by the {@link IViewerProvider} interface.
	 */
	public Viewer getViewer()
	{
		return currentViewer;
	}

	/**
	 * Handles activation of the editor or it's associated views.
	 */
	protected void handleActivate()
	{
		// Recompute the read only state.
		if (editingDomain.getResourceToReadOnlyMap() != null)
		{
			editingDomain.getResourceToReadOnlyMap().clear();

			// Refresh any actions that may become enabled or disabled.
			setSelection(getSelection());

			// Save all dirty editors.
			getSite().getWorkbenchWindow().getWorkbench().saveAllEditors(false);

		}

		if (!removedResources.isEmpty())
		{
			if (handleDirtyConflict())
			{
				//getSite().getPage().closeEditor(JasmineDiagramEditor.this, false);
//				JasmineDiagramEditor.this.close(true);
//				JasmineDiagramEditor.this.dispose();
			}
			else
			{
				removedResources.clear();
				changedResources.clear();
				savedResources.clear();
			}
		}
		else if (!changedResources.isEmpty())
		{
			changedResources.removeAll(savedResources);
			handleChangedResource();
			changedResources.clear();
			savedResources.clear();
		}
	}


	/**
	 * Resources that have been removed since last activation.
	 */
	protected Collection<Resource> removedResources = new ArrayList<Resource>();

	/**
	 * Resources that have been changed since last activation.
	 */
	protected Collection<Resource> changedResources = new ArrayList<Resource>();

	/**
	 * Resources that have been moved since last activation.
	 * Maps {@link Resource resource} to {@link URI new URI}
	 */
	Map movedResources = new HashMap();

	/**
	 * Resources that have been saved.
	 */
	protected Collection<Resource> savedResources = new ArrayList<Resource>();

	private boolean dirty;

//	private IOperationHistoryListener historyListener = new IOperationHistoryListener() {
//	public void historyNotification(final OperationHistoryEvent event) {
//	if (event.getEventType() == OperationHistoryEvent.DONE) {
//	Set affectedResources = ResourceUndoContext.getAffectedResources(
//	event.getOperation());

//	if (affectedResources.contains(getResource())) {
//	final IUndoableOperation operation = event.getOperation();

//	// remove the default undo context so that we can have
//	//     independent undo/redo of independent resource changes
//	operation.removeContext(((IWorkspaceCommandStack)
//	getEditingDomain().getCommandStack()).getDefaultUndoContext());

//	// add our undo context to populate our undo menu
//	operation.addContext(getUndoContext());

//	getSite().getShell().getDisplay().asyncExec(new Runnable() {
//	public void run() {
//	dirty = true;
//	firePropertyChange(IEditorPart.PROP_DIRTY);

//	// Try to select the affected objects.
//	//
//	if (operation instanceof EMFCommandOperation) {
//	Command command = ((EMFCommandOperation) operation).getCommand();

//	if (command != null) {
//	setSelectionToViewer(command
//	.getAffectedObjects());
//	}
//	}

//	if (propertySheetPage != null) {
//	propertySheetPage.refresh();
//	}
//	}
//	});
//	}
//	}
//	}};

	/**
	 * Synchronizes workspace changes with the editing domain.
	 */
	protected WorkspaceSynchronizer workspaceSynchronizer;

	private WorkspaceSynchronizer.Delegate createSynchronizationDelegate() {

		return new WorkspaceSynchronizer.Delegate() {
			public boolean handleResourceDeleted(Resource resource) {
				if ((resource == getResource()) && !isDirty()) {
					// just close now without prompt
					getSite().getShell().getDisplay().asyncExec
					(new Runnable() {
						public void run() {

							getSite().getWorkbenchWindow().getWorkbench().saveAllEditors(false);
							getSite().getWorkbenchWindow().getWorkbench().close();

							JasmineNodeExplorerView.this.dispose();
						}
					});
				} else {
					removedResources.add(resource);
				}

				return true;
			}

			public boolean handleResourceChanged(Resource resource) {

				System.out.println("handleResourceChanged");

				// is this a resource that we just saved?  If so, then this is
				//   notification of that save, so forget it
				if (savedResources.contains(resource)) {
					savedResources.remove(resource);
				} else {
					changedResources.add(resource);
				}

				return true;
			}

			public boolean handleResourceMoved(Resource resource, URI newURI) {
				movedResources.put(resource, newURI);
				System.out.println("newURI "+newURI.path());
				return true;
			}

			public void dispose() {
				removedResources.clear();
				changedResources.clear();
				movedResources.clear();
			}};
	}


	/**
	 * Handles what to do with changed resource on activation.
	 */
	protected void handleChangedResource() {

		System.out.println("handleChangedResource");

		Resource res = getResource();

		if (changedResources.contains(res) && (!isDirty() || handleDirtyConflict())) {
			changedResources.remove(res);

			dirty = false;
			firePropertyChange(IEditorPart.PROP_DIRTY);

			ResourceLoadedListener listener = ResourceLoadedListener.getDefault();
			listener.ignore(res);

			try {
				if (res.isLoaded()) {
					res.unload();
					try {
						res.load(Collections.EMPTY_MAP);
					} catch (IOException exception) {
						exception.printStackTrace();
					}
				}

				currentViewer.setInput(getResource());
			} finally {
				listener.watch(res);
			}
		}
	}

	/**
	 * Handles what to do with moved resource on activation.
	 */
	protected void handleMovedResource() {
		if (!isDirty() || handleDirtyConflict()) {
			Resource res = getResource();
			URI newURI = (URI) movedResources.get(res);

			if (newURI != null) {
				if (res.isLoaded()) {
					// unload
					res.unload();
				}

				// load the new URI in another editor
				res.getResourceSet().getResource(newURI, true);
			}
		}
	}

	/**
	 * Shows a dialog that asks if conflicting changes should be discarded.
	 *
	 */
	protected boolean handleDirtyConflict() {
		return
		MessageDialog.openQuestion
		(getSite().getShell(),
				getString("_UI_FileConflict_label"), //$NON-NLS-1$
				getString("_WARN_FileConflict")); //$NON-NLS-1$
	}

	/**
	 * This is for implementing {@link IEditorPart} and simply tests the command stack.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public boolean isDirty() {

		return ((BasicCommandStack)editingDomain.getCommandStack()).isSaveNeeded();
	}


	/**
	 * This is how the framework determines which interfaces we implement.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public Object getAdapter(Class key) {
		if (key.equals(IContentOutlinePage.class)) {
			return getContentOutlinePage();
		}
		else if (key.equals(IPropertySheetPage.class)) {
			return getPropertySheetPage();
		}
		else if (key.equals(IGotoMarker.class)) {
			return this;
		}
		else if (key.equals(IUndoContext.class)) {
			// used by undo/redo actions to get their undo context
			return undoContext;
		}
		else {
			return super.getAdapter(key);
		}
	}


	/**
	 * This accesses a cached version of the content outliner.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public IContentOutlinePage getContentOutlinePage() {
		if (contentOutlinePage == null) {
			// The content outline is just a tree.
			//
			class MyContentOutlinePage extends ContentOutlinePage {
				public void createControl(Composite parent) {
					super.createControl(parent);
					contentOutlineViewer = getTreeViewer();
					contentOutlineViewer.addSelectionChangedListener(this);

					// Set up the tree viewer.
					//
					contentOutlineViewer.setContentProvider(
							new TransactionalAdapterFactoryContentProvider(
									(TransactionalEditingDomain) getEditingDomain(), adapterFactory));
					contentOutlineViewer.setLabelProvider(
							new TransactionalAdapterFactoryLabelProvider(
									(TransactionalEditingDomain) getEditingDomain(), adapterFactory));

					// unlike other EMF editors, I edit only a single resource, not a resource set
					contentOutlineViewer.setInput(getResource());

					// Make sure our popups work.
					//
					createContextMenuFor(contentOutlineViewer);

					if (!editingDomain.getResourceSet().getResources().isEmpty()) {
						// Select the root object in the view.
						//
						ArrayList selection = new ArrayList();
						selection.add(getResource());
						contentOutlineViewer.setSelection(new StructuredSelection(selection), true);
					}
				}

				public void makeContributions(IMenuManager menuManager, IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
					super.makeContributions(menuManager, toolBarManager, statusLineManager);
					contentOutlineStatusLineManager = statusLineManager;
				}

				public void setActionBars(IActionBars actionBars) {
					super.setActionBars(actionBars);
					getActionBarContributor().shareGlobalActions(this, actionBars);
				}
			}

			contentOutlinePage = new MyContentOutlinePage();

			// Listen to selection so that we can handle it is a special way.
			//
			contentOutlinePage.addSelectionChangedListener
			(new ISelectionChangedListener() {
				// This ensures that we handle selections correctly.
				//
				public void selectionChanged(SelectionChangedEvent event) {
					handleContentOutlineSelection(event.getSelection());
				}
			});
		}
		return contentOutlinePage;
	}



	/**
	 * This deals with how we want selection in the outliner to affect the other views.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void handleContentOutlineSelection(ISelection selection) {
		if (!selection.isEmpty() && selection instanceof IStructuredSelection) {
			Iterator selectedElements = ((IStructuredSelection)selection).iterator();
			if (selectedElements.hasNext()) {
				// Get the first selected element.
				//
				Object selectedElement = selectedElements.next();

				ArrayList selectionList = new ArrayList();
				selectionList.add(selectedElement);
				while (selectedElements.hasNext()) {
					selectionList.add(selectedElements.next());
				}

				// Set the selection to the widget.
				//
				selectionViewer.setSelection(new StructuredSelection(selectionList));
			}
		}
	}


	@Override
	public void dispose()
	{
		System.out.println("dispose (View)");
		updateProblemIndication = false;

		ResourcesPlugin.getWorkspace().removeResourceChangeListener(resourceChangeListener);

		getSite().getPage().removePartListener(partListener);

		adapterFactory.dispose();

		if (getActionBarContributor().getActiveEditor() == this)
		{
			getActionBarContributor().setActiveEditor(null);
		}

		if (propertySheetPage != null)
		{
			propertySheetPage.dispose();
		}

		if (contentOutlinePage != null)
		{
			contentOutlinePage.dispose();
		}

		super.dispose();
	}

	/**
	 * This looks up a string in the plugin's plugin.properties file.
	 */
	private static String getString(String key)
	{
		return JasmineEditPlugin.INSTANCE.getString(key);

	}

	/**
	 * This looks up a string in plugin.properties, making a substitution.
	 */
	private static String getString(String key, Object s1)
	{
		return JasmineEditPlugin.INSTANCE.getString(key, new Object [] { s1 });

	}

	protected Resource getResource() {
		if (RESOURCE_URI != null) {
			URI uri = URI.createFileURI(this.RESOURCE_URI);
			return editingDomain.getResourceSet().getResource(uri, false);  //************************************************
		} else {
			return editingDomain.getResourceSet().getResources().get(0);
		}
	}

	private void createNode (final Resource resource) {
		TransactionalEditingDomain tstack = (TransactionalEditingDomain)getEditingDomain();
		getEditingDomain().getCommandStack().execute(new RecordingCommand(tstack) {
			protected void doExecute() {

				JASMINeNode node = JasminemodelFactory.eINSTANCE.createJASMINeNode();

				try {

					resource.getContents().add(node);
					resource.save(Collections.EMPTY_MAP);

				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
	}

	private void deleteNode (final JASMINeNode node) {
		TransactionalEditingDomain tstack = (TransactionalEditingDomain)getEditingDomain();
		getEditingDomain().getCommandStack().execute(new RecordingCommand(tstack) {
			protected void doExecute() {

				EcoreUtil.delete(node);
				try {
					getResource().getContents().remove(node);
					getResource().save(Collections.EMPTY_MAP);

				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
	}




	/* (non-Javadoc)
	 * @see org.eclipse.ui.IViewPart#saveState(org.eclipse.ui.IMemento)
	 */
	@Override
	public void saveState(IMemento memento) {
		// save the model
		doSave();
	}


	/**
	 * This is for implementing {@link IViewPart} and simply saves the model file.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 *
	 * @generated NOT
	 */
	  public void doSave()
	  {

			System.out.println("doSave called (View)");

	    // Save only resources that have actually changed.
	    //
	    final Map<Object, Object> saveOptions = new HashMap<Object, Object>();
	    saveOptions.put(Resource.OPTION_SAVE_ONLY_IF_CHANGED, Resource.OPTION_SAVE_ONLY_IF_CHANGED_MEMORY_BUFFER);

	    // Do the work within an operation because this is a long running activity that modifies the workbench.
	    //
	    WorkspaceModifyOperation operation =
	      new WorkspaceModifyOperation()
	      {
	        // This is the method that gets invoked when the operation runs.
	        //
	        @Override
	        public void execute(IProgressMonitor monitor)
	        {
	          // Save the resources to the file system.
	          //
	          boolean first = true;
	          for (Resource resource : getEditingDomain().getResourceSet().getResources())
	          {
	            if ((first || !resource.getContents().isEmpty()) && !getEditingDomain().isReadOnly(resource))
	            {
	              try
	              {
	            	System.out.println("add resource "+resource.getURI());


	                savedResources.add(resource);
	                resource.save(saveOptions);

	              }
	              catch (Exception exception)
	              {
//	                resourceToDiagnosticMap.put(resource, analyzeResourceProblems(resource, exception));
	      	    	JasmineEditPlugin.getPlugin().log(exception);
	    	    	exception.printStackTrace();
	              }
	              first = false;
	            }
	          }
	        }
	      };

	    updateProblemIndication = false;
	    try
	    {
	      // This runs the options, and shows progress.
	      //
	      new ProgressMonitorDialog(getSite().getShell()).run(true, false, operation);

	      // Refresh the necessary state.
	      //
	      ((BasicCommandStack)getEditingDomain().getCommandStack()).saveIsDone();
	      firePropertyChange(IEditorPart.PROP_DIRTY);

	    }
	    catch (Exception exception)
	    {
	      // Something went wrong that shouldn't.
	      //
	    	JasmineEditPlugin.getPlugin().log(exception);
	    	exception.printStackTrace();
	    }
	    updateProblemIndication = true;
//	    updateProblemIndication();
	  }


		private void refreshModel() {
			getResource().unload();
			try {
			getResource().load(
					getEditingDomain().getResourceSet().getLoadOptions());

			} catch (IOException e) {
				e.printStackTrace();
			}
		}




}