/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.logs;

import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eclipse.swt.widgets.Display;
import org.objectweb.jasmine.rules.logs.AlarmLog;
import org.objectweb.jasmine.rules.logs.LogInterfaceRemote;
import org.objectweb.jasmine.rules.utils.DBException;

public class ComLog {

    private GuiRuleLogManager manager;

    // connection vars
    private static LogInterfaceRemote logRemote;

    private Display display;

    private enum Level {
        INFO, WARN, ALERT, ACTION
    };

    // level of info < warn < alert
    // action is different

    public ComLog() {
        super();
    }

    public GuiRuleLogManager getManager() {
        return manager;
    }

    public void setManager(GuiRuleLogManager manager) {
        this.manager = manager;
    }

    public Display getDisplay() {
        return display;
    }

    public void setDisplay(Display display) {
        this.display = display;
    }

    public void init() {
        try {
            manager.getLogs().addAll(getLogRules());
            manager.update();

            ComLogListener listener = new ComLogListener(this);
            listener.startListening();

        } catch (DBException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void connection(String url) throws NamingException, Exception {

        System.setProperty(Context.INITIAL_CONTEXT_FACTORY,
                "org.objectweb.carol.jndi.spi.JRMPContextWrapperFactory");
        System.setProperty(Context.PROVIDER_URL, url);

        Context initialContext = new InitialContext();
        logRemote = (LogInterfaceRemote) initialContext
                .lookup("org.objectweb.jasmine.rules.logs.LogInterface" + "_"
                        + LogInterfaceRemote.class.getName() + "@Remote");

    }

    public void refreshLogRulesDiff() throws DBException {
        List<AlarmLog> tmp = logRemote.getAllLogs();
        if (!manager.equals(tmp)) {
            manager.setLogs(tmp);
            manager.update();
        }
    }

    public void refreshNewLogRules(int id) throws DBException {
    	AlarmLog tmp = logRemote.getLog(id);
        manager.addLogRule(tmp);
        manager.update();
    }

    public List<AlarmLog> getLogRules() throws DBException {
        return logRemote.getAllLogs();
    }

    public void removeLogRules(Filter filter) throws DBException {
    	ArrayList<String> levelArray = new ArrayList<String>();
        if (filter.isInfo()) levelArray.add(Level.INFO.toString());
        if (filter.isWarn()) levelArray.add(Level.WARN.toString());
        if (filter.isAlert()) levelArray.add(Level.ALERT.toString());
        if (filter.isAction()) levelArray.add(Level.ACTION.toString());
    	String[] levels = levelArray.toArray(new String[0]);

        if (filter.isDate())
            logRemote.delLogRange(levels, filter.getBegin(), filter.getEnd());
        else
            logRemote.delLogRange(levels, null, null);
    }

}
