/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.logs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class DeleteDialog extends Dialog {

    private Filter filter = null;

    private List<Text> texts;

    private boolean okEnd;

    public DeleteDialog(Shell parent) {
        // Pass the default styles here
        this(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE);
    }

    public DeleteDialog(Shell parent, int style) {
        // Let users override the default styles
        super(parent, style);
        setText("Clean DB");
        texts = new ArrayList<Text>();
    }

    public void setFilter(Filter filtre) {
        this.filter = filtre;
    }

    public Filter getFilter() {
        return filter;
    }

    public boolean isOkEnd() {
        return okEnd;
    }

    public String open() {
        // Create the dialog window
        Shell shell = new Shell(getParent(), getStyle());
        shell.setText(getText());
        createContents(shell);
        shell.pack();
        shell.open();
        Display display = getParent().getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch()) {
                display.sleep();
            }
        }
        // Return the entered value, or null
        return null;
    }

    /**
     * Creates the dialog's contents
     * 
     * @param shell
     *            the dialog window
     */
    @SuppressWarnings("deprecation")
    private void createContents(final Shell shell) {

        // definition of the layout of the view
        GridLayout gl = new GridLayout();
        gl.numColumns = 1;
        shell.setLayout(gl);

        // definition of controls
        Label ruleTitle = new Label(shell, SWT.NONE);
        ruleTitle.setText("Choose what you want to clean");

        new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);

        Button info = new Button(shell, SWT.CHECK);
        info.setText("INFO");
        info.setSelection(filter.isInfo());
        info.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                filter.setInfo(((Button) e.getSource()).getSelection());
            }
        });
        Button warn = new Button(shell, SWT.CHECK);
        warn.setText("WARN");
        warn.setSelection(filter.isWarn());
        warn.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                filter.setWarn(((Button) e.getSource()).getSelection());
            }
        });
        Button alert = new Button(shell, SWT.CHECK);
        alert.setText("ALERT");
        alert.setSelection(filter.isAlert());
        alert.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                filter.setAlert(((Button) e.getSource()).getSelection());
            }
        });
        Button action = new Button(shell, SWT.CHECK);
        action.setSelection(filter.isAction());
        action.setText("ACTION");
        action.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                filter.setAction(((Button) e.getSource()).getSelection());
            }
        });

        new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);

        Button date = new Button(shell, SWT.CHECK);

        date.setSelection(filter.isDate());
        date.setText("Use begin/end date");
        date.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                filter.setDate(((Button) e.getSource()).getSelection());
                for (Text t : texts) {
                    t.setEnabled(filter.isDate());
                }
            }
        });

        Label beginl = new Label(shell, SWT.NONE);
        beginl.setText("Begin");

        Composite c = new Composite(shell, SWT.NONE);
        GridLayout gl2 = new GridLayout();
        gl2.numColumns = 12;
        c.setLayout(gl2);

        number(c, "/", filter.getBegin().getDate()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setDate(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c, "/", filter.getBegin().getMonth()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setMonth(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c, ":", 1900 + filter.getBegin().getYear()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setYear(
                                1900 + Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c, "h", filter.getBegin().getHours()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setHours(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c, "m", filter.getBegin().getMinutes()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setMinutes(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c, "s", filter.getBegin().getSeconds()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getBegin().setSeconds(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });

        Label endl = new Label(shell, SWT.NONE);
        endl.setText("End");

        Composite c2 = new Composite(shell, SWT.NONE);
        GridLayout gl3 = new GridLayout();
        gl3.numColumns = 12;
        c2.setLayout(gl3);

        number(c2, "/", filter.getEnd().getDate()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setDate(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c2, "/", filter.getEnd().getMonth()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setMonth(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c2, ":", 1900 + filter.getEnd().getYear()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setYear(
                                1900 + Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c2, "h", filter.getEnd().getHours()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setHours(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c2, "m", filter.getEnd().getMinutes()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setMinutes(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });
        number(c2, "s", filter.getEnd().getSeconds()).addModifyListener(
                new ModifyListener() {
                    public void modifyText(ModifyEvent e) {
                        filter.getEnd().setSeconds(
                                Integer.valueOf(((Text) e.getSource())
                                        .getText()));
                    }
                });

        // must be donne here
        if (!filter.isDate())
            for (Text t : texts) {
                t.setEnabled(filter.isDate());
            }

        new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);

        Composite c3 = new Composite(shell, SWT.NONE);
        GridLayout gl4 = new GridLayout();
        gl4.numColumns = 2;
        c3.setLayout(gl4);

        Button ok = new Button(c3, SWT.PUSH);
        ok.setText("Clean DB");
        Button cancel = new Button(c3, SWT.PUSH);
        cancel.setText("Cancel");

        ok.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
                shell.close();
                okEnd = true;
            }
        });

        cancel.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
                shell.close();
                okEnd = false;
            }
        });
    }

    private Text number(Composite shell, String name, int current) {
        Text number = new Text(shell, SWT.BORDER);
        number.setText(Integer.toString(current));
        texts.add(number);
        Label label = new Label(shell, SWT.NONE);
        label.setText(name);

        return number;
    }
}
