/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.logs;

import java.io.FileOutputStream;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.objectweb.jasmine.rules.logs.AlarmLog;
import org.objectweb.jasmine.rules.utils.DBException;

public class GuiRuleLogControler implements IGuiRuleObserver {
    GuiRuleLogManager manager;

    GuiRuleLogView view;

    boolean incLevel = true;

    boolean incDate = true;

    public GuiRuleLogControler(GuiRuleLogManager manager, GuiRuleLogView view) {
        super();
        this.manager = manager;
        this.view = view;
    }

    public void addListeners() {
        // filtre button
        view.getFiltre().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                FiltreDialog fd = new FiltreDialog(view.getParent().getShell());
                fd.setFilter(view.getFilterOb());
                fd.open();
                if (fd.isEndOk()) {
                    view.setFilterOb(fd.getFilter());
                    view.refresh();
                }
            }
        });

        // clean db button
        view.getDelete().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                DeleteDialog fd = new DeleteDialog(view.getParent().getShell());
                fd.setFilter(new Filter());
                fd.open();
                if (fd.isOkEnd()) {
                    try {
                        manager.cleanByFilter(fd.getFilter());
                    } catch (DBException e1) {
                        MessageDialog.openError(view.getParent().getShell(),
                                "Error", "Problem  delete logs from db\n"
                                        + "error : " + e1.getMessage());
                        e1.printStackTrace();
                        return;
                    }
                    try {
                        manager.refreshLogs();
                    } catch (DBException e1) {
                        MessageDialog.openError(view.getParent().getShell(),
                                "Error", "Problem load logs from db\n"
                                        + "error : " + e1.getMessage());
                        e1.printStackTrace();
                        return;
                    }
                    view.refresh();
                }
            }
        });

        // export button
        view.getExport().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                ExportDialog fd = new ExportDialog(view.getParent().getShell());
                fd.setFilter(new Filter());
                fd.open();
                if (fd.isOkEnd()) {

                    HSSFWorkbook wb = new HSSFWorkbook();

                    HSSFSheet sheet = wb.createSheet("log of rules engine");
                    HSSFRow row = sheet.createRow((short) 0);
                    row.createCell((short) 0).setCellValue("Level");
                    row.createCell((short) 1).setCellValue("Date");
                    row.createCell((short) 2).setCellValue("Message");
                    int i = 1;
                    for (AlarmLog log : manager.getLogs()) {

                        // create row
                        row = sheet.createRow((short) i);
                        // create cells
                        row.createCell((short) 0).setCellValue(log.getLevel());
                        row.createCell((short) 1).setCellValue(
                                log.getDate().toString());
                        row.createCell((short) 2)
                                .setCellValue(log.getMessage());
                        i++;
                    }

                    // Write the output to a file
                    FileOutputStream fileOut;
                    try {
                        fileOut = new FileOutputStream(fd.getPath());
                        wb.write(fileOut);
                        fileOut.close();
                    } catch (Exception e1) {
                        MessageDialog.openError(view.getParent().getShell(),
                                "Error",
                                "Impossible to export log to xls file\n"
                                        + "error : " + e1.getMessage());
                        e1.printStackTrace();
                    }
                }
            }
        });

        // colonne 1 selection
        view.getColonne1().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                manager.sortByLevel(incLevel);
                incLevel = !incLevel;
                view.refresh();
            }
        });

        // colonne 2 selection
        view.getColonne2().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                manager.sortByDate(incDate);
                incDate = !incDate;
                view.refresh();
            }
        });

    }

    public void update() {

    }
}
