/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.logs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.objectweb.jasmine.rules.logs.AlarmLog;
import org.objectweb.jasmine.rules.utils.DBException;


public class GuiRuleLogManager {

    private boolean tmp;

    private ComLog comLog = null;

    private List<IGuiRuleObserver> obs;

    private List<AlarmLog> logs;

    /*
     * private enum Level { INFO, WARN, ALERT, ACTION}; // level of info < warn <
     * alert // action is different
     */

    public GuiRuleLogManager(ComLog cl) {
        super();
        obs = new ArrayList<IGuiRuleObserver>(2);
        logs = new ArrayList<AlarmLog>();
        comLog = cl;
    }

    void addLogRule(AlarmLog glr) {
        logs.add(glr);
        // TODO : to un-comment
        // update();
    }

    public void addObs(IGuiRuleObserver iobs) {
        obs.add(iobs);
    }

    void update() {
        for (IGuiRuleObserver iobs : obs) {
            iobs.update();
        }
    }

    void sortByDate(boolean inc) {
        tmp = inc;

        Collections.sort(logs, new Comparator<AlarmLog>() {

            public int compare(AlarmLog o1, AlarmLog o2) {
                if (tmp) {
                    if (o1.getDate().before(o2.getDate())) {
                        return -1;
                    }
                    /*
                     * else if((o1.getDate().equals(o2.getDate()))){ return 0; }
                     */
                    else {
                        return 1;
                    }
                } else {
                    if (o1.getDate().before(o2.getDate())) {
                        return 1;
                    }
                    /*
                     * else if((o1.getDate().equals(o2.getDate()))){ return 0; }
                     */
                    else {
                        return -1;
                    }
                }
            }

        });
    }

    void sortByLevel(boolean inc) {
        tmp = inc;

        Collections.sort(logs, new Comparator<AlarmLog>() {

            public int compare(AlarmLog o1, AlarmLog o2) {
                if (tmp) {
                    return (o1.getLevel().compareTo(o2.getLevel())) * (-1);
                } else {
                    return o1.getLevel().compareTo(o2.getLevel());
                }
            }

        });
    }

    public List<AlarmLog> getLogs() {
        return logs;
    }

    public void setLogs(List<AlarmLog> logs) {
        this.logs = logs;
    }

    public void cleanByFilter(Filter filter) throws DBException {
        comLog.removeLogRules(filter);
    }

    public void refreshLogs() throws DBException {
        logs.clear();
        logs.addAll(comLog.getLogRules());
    }

    public boolean equals(List<AlarmLog> l2) {
        boolean res = true;
        for (AlarmLog log : l2) {
            boolean tmp = false;
            ;
            Iterator it = logs.iterator();
            while (it.hasNext()) {
                if (((AlarmLog) it.next()).equals(log)) {
                    tmp = true;
                }
            }
            if (!tmp) {
                res = false;
            }
        }
        return res;
    }

}