/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.logs;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.jasmine.ui.jasminemodel.provider.JasmineEditPlugin;

import org.eclipse.emf.edit.ui.provider.ExtendedImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.objectweb.jasmine.rules.logs.AlarmLog;

public class GuiRuleLogView implements IGuiRuleObserver {

    private GuiRuleLogManager manager;

    private Table table;

    private Composite parent;

    private Filter filterOb = null;

    private Map<String, Image> icons;

    private static String INFO = "full/jasmine/ruleLogs/info";

    private static String WARN = "full/jasmine/ruleLogs/warning";

    private static String ALERT = "full/jasmine/ruleLogs/alert";

    private static String ACTION = "full/jasmine/ruleLogs/action";

    private Button filtre;

    private Button export;

    private Button delete;

    private TableColumn colonne1;

    private TableColumn colonne2;

    public GuiRuleLogView(GuiRuleLogManager manager) {
        super();
        this.manager = manager;
        // initale filtre
        filterOb = new Filter();
    }

    private Image loadIconsHelper(String location) {
        Object i = JasmineEditPlugin.INSTANCE.getImage(location);
        ExtendedImageRegistry.INSTANCE.getImageDescriptor(i);
        return ExtendedImageRegistry.getInstance().getImage(i);
    }

    private void loadIcons() {
        icons = new HashMap<String, Image>();

        icons.put("INFO", loadIconsHelper(INFO));
        icons.put("WARN", loadIconsHelper(WARN));
        icons.put("ALERT", loadIconsHelper(ALERT));
        icons.put("ACTION", loadIconsHelper(ACTION));

    }

    public void createContent(Composite p) {
        parent = p;

        Label label = new Label(parent, SWT.NONE);
        label.setText("Automanagement logs");

        filtre = new Button(parent, SWT.PUSH);
        filtre.setText("Filtre");
        export = new Button(parent, SWT.PUSH);
        export.setText("Export");
        delete = new Button(parent, SWT.PUSH);
        delete.setText("Delete");

        table = new Table(parent, SWT.BORDER);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);
        loadIcons();

        colonne1 = new TableColumn(table, SWT.LEFT);
        colonne1.setText("Level");
        colonne1.setWidth(80);
        colonne2 = new TableColumn(table, SWT.LEFT);
        colonne2.setText("Date");
        colonne2.setWidth(180);
        TableColumn colonne3 = new TableColumn(table, SWT.LEFT);
        colonne3.setText("Message");
        colonne3.setWidth(200);

        // shell.setMinimumSize(300, 300);

        FormLayout formLayout;

        formLayout = new FormLayout();
        formLayout.marginWidth = org.objectweb.jasmine.ui.logs.FormUtil.insetX;
        formLayout.marginHeight = org.objectweb.jasmine.ui.logs.FormUtil.insetY;
        p.setLayout(formLayout);

        FormUtil.setFormLayout(label, 0, null, 0, null);
        FormUtil.setFormLayout(filtre, label, null, 0, null);
        FormUtil.setFormLayout(export, filtre, null, 0, null);
        FormUtil.setFormLayout(delete, export, null, 0, null);

        FormUtil.setFormLayout(table, 0, 100, filtre, 100);

    }

    public Button getDelete() {
        return delete;
    }

    public Button getExport() {
        return export;
    }

    public Button getFiltre() {
        return filtre;
    }

    public Composite getParent() {
        return parent;
    }

    public Filter getFilterOb() {
        return filterOb;
    }

    public void setFilterOb(Filter filtreOb) {
        this.filterOb = filtreOb;
    }

    public TableColumn getColonne1() {
        return colonne1;
    }

    public TableColumn getColonne2() {
        return colonne2;
    }

    public void refresh() {

        // clean the table
        if (table != null) {
            TableItem[] items = table.getItems();
            for (int i = 0; i < items.length; i++) {
                items[i].dispose();
            }

            // fill the table
            for (AlarmLog log : manager.getLogs()) {
                if (logFiltre(log)) {
                    TableItem ligne = new TableItem(table, SWT.NONE);

                    ligne.setText(new String[] { log.getLevel(),
                            log.getDate().toString(), log.getMessage() });
                    ligne.setImage(icons.get(log.getLevel()));
                }
            }
        }
    }

    private boolean logFiltre(AlarmLog log) {

        if (log.getLevel().equals("INFO") && !filterOb.isInfo()) {
            return false;
        }
        if (log.getLevel().equals("WARN") && !filterOb.isWarn()) {
            return false;
        }
        if (log.getLevel().equals("ALERT") && !filterOb.isAlert()) {
            return false;
        }
        if (log.getLevel().equals("ACTION") && !filterOb.isAction()) {
            return false;
        }
        if (filterOb.isDate()
                && (log.getDate().before(filterOb.getBegin()) || log.getDate()
                        .after(filterOb.getEnd()))) {
            return false;
        }

        return true;
    }

    public void update() {
        refresh();
    }
}
