/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.monitoring.CommunicationAndParsing;

import java.util.List;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.xml.xpath.XPathConstants;

import org.objectweb.jasmine.ui.monitoring.NumericDatas;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

class ListenerJMS implements MessageListener {

    public ListenerJMS() {
    }

    private ParserProbe parserProbe;

    public void onMessage(final Message message) {

        // to activate a probe
        // ActiveProbes.addProbe("129.183.91.55", "cpu");

        // Should be instance of text message
        if (message != null) {
            if (message instanceof TextMessage) {
                try {

                    // start message parsing
                    parserProbe = new ParserProbe();
                    String xml = "<root>" + ((TextMessage) message).getText()
                            + "</root>";

                    // cover the nodes
                    NodeList nodes = (NodeList) parserProbe.evaluateSAX(xml,
                            "/root/probe", XPathConstants.NODESET);

                    if (nodes == null) {
                        System.out.println("Parser error");
                    } else {

                        for (int i = 0; i < nodes.getLength(); ++i) {

                            Node node = nodes.item(i);

                            // the attributs of "root" node
                            NamedNodeMap namedNodeMap = node.getAttributes();
                            // String timestamp = null;
                            String hostname = null;
                            String ip = null;
                            for (int l = 0; l < namedNodeMap.getLength(); ++l) {
                                Node attribute = namedNodeMap.item(l);
                                String nodeName = attribute.getNodeName();

                                /*
                                 * if(nodeName.equals("timestamp")) timestamp =
                                 * attribute.getNodeValue();
                                 */

                                if (nodeName.equals("hostname"))
                                    hostname = attribute.getNodeValue();

                                if (nodeName.equals("ip")) {
                                    ip = attribute.getNodeValue();
                                }
                            }

                            if (ActiveProbes
                                    .thisMachineHaveAProbe(ip, hostname)) {
                                // Les noeuds fils
                                NodeList enfants = node.getChildNodes();

                                for (int k = 0; k < enfants.getLength(); ++k) {
                                    Node enfant = enfants.item(k);

                                    if (enfant.getNodeName().equals("name")) {

                                        // CPU

                                        if (enfant.getTextContent().equals(
                                                "cpu")
                                                && ActiveProbes
                                                        .thisMachineHaveAProbe2(
                                                                ip, hostname,
                                                                "cpu")) {
                                            CpuParser cpuParser = new CpuParser();
                                            CpuProbe cpuProbe = cpuParser
                                                    .createCpuProbe(node);

                                            List<ActiveProbes_Probe> activeProbes = ActiveProbes
                                                    .MachineProbes(ip,
                                                            hostname, "cpu");
                                            for (ActiveProbes_Probe app : activeProbes) {
                                                if (app.data != null) {
                                                    NumericDatas cd = ((NumericDatas) app.data);
                                                    cd
                                                            .setValues(
                                                                    "Total load",
                                                                    cpuProbe
                                                                            .getTotalCpu());
                                                    cd
                                                            .setValues(
                                                                    "New load",
                                                                    cpuProbe
                                                                            .getNewCpu());
                                                    cd
                                                            .setValues(
                                                                    "System load",
                                                                    cpuProbe
                                                                            .getSystemCpu());
                                                    cd
                                                            .setValues(
                                                                    "IOWait load",
                                                                    cpuProbe
                                                                            .getIoWaitCpu());
                                                    cd
                                                            .setValues(
                                                                    "User load",
                                                                    cpuProbe
                                                                            .getUserCpu());
                                                    cd
                                                            .setValues(
                                                                    "Idle",
                                                                    cpuProbe
                                                                            .getIdleCpu());
                                                    cd
                                                            .setValues(
                                                                    "Nice",
                                                                    cpuProbe
                                                                            .getNiceCpu());
                                                    cd.newMeasure();
                                                }
                                            }

                                        }

                                        // MEMORY

                                        if (enfant.getTextContent().equals(
                                                "memory")
                                                && ActiveProbes
                                                        .thisMachineHaveAProbe2(
                                                                ip, hostname,
                                                                "memory")) {
                                            MemoryParser memoryParser = new MemoryParser();
                                            MemoryProbe memoryProbe = memoryParser
                                                    .createMemoryProbe(node);

                                            List<ActiveProbes_Probe> activeProbes = ActiveProbes
                                                    .MachineProbes(ip,
                                                            hostname, "memory");
                                            for (ActiveProbes_Probe app : activeProbes) {
                                                if (app.data != null) {
                                                    NumericDatas cd = ((NumericDatas) app.data);
                                                    cd
                                                            .setValues(
                                                                    "Total memory",
                                                                    memoryProbe
                                                                            .getMemTotalMemory());
                                                    cd
                                                            .setValues(
                                                                    "Free memory",
                                                                    memoryProbe
                                                                            .getMemFreeMemory());
                                                    cd
                                                            .setValues(
                                                                    "Used memory",
                                                                    memoryProbe
                                                                            .getMemUsedMemory());
                                                    cd
                                                            .setValues(
                                                                    "Used memory %",
                                                                    memoryProbe
                                                                            .getMemUsedPerMemory());
                                                    cd
                                                            .setValues(
                                                                    "Cached",
                                                                    memoryProbe
                                                                            .getCachedMemory());
                                                    cd
                                                            .setValues(
                                                                    "Buffers",
                                                                    memoryProbe
                                                                            .getBuffersMemory());
                                                    cd
                                                            .setValues(
                                                                    "Swap used",
                                                                    memoryProbe
                                                                            .getSwapUsedMemory());
                                                    cd
                                                            .setValues(
                                                                    "Swap used %",
                                                                    memoryProbe
                                                                            .getSwapUsedPerMemory());
                                                    cd
                                                            .setValues(
                                                                    "Total swap",
                                                                    memoryProbe
                                                                            .getSwapTotalMemory());
                                                    cd
                                                            .setValues(
                                                                    "Free swap",
                                                                    memoryProbe
                                                                            .getSwapFreeMemory());
                                                    cd.newMeasure();
                                                }
                                            }
                                        }

                                        // NETWORK

                                        if (enfant.getTextContent().equals(
                                                "network")
                                                && ActiveProbes
                                                        .thisMachineHaveAProbe2(
                                                                ip, hostname,
                                                                "network")) {
                                            NetworkParser networkParser = new NetworkParser();
                                            NetworkProbe networkProbe = networkParser
                                                    .createNetworkProbe(node);

                                            List<ActiveProbes_Probe> activeProbes = ActiveProbes
                                                    .MachineProbes(ip,
                                                            hostname, "network");
                                            for (ActiveProbes_Probe app : activeProbes) {
                                                if (app.data != null) {
                                                    NumericDatas cd = ((NumericDatas) app.data);
                                                    cd
                                                            .setValues(
                                                                    "Tx",
                                                                    networkProbe
                                                                            .getTxBytesNetwork());
                                                    cd
                                                            .setValues(
                                                                    "Rx",
                                                                    networkProbe
                                                                            .getRxBytesNetwork());
                                                    cd.newMeasure();
                                                }
                                            }
                                        }

                                    }
                                }
                            }
                        }
                    }

                } catch (JMSException e) {
                    System.err
                            .println("Error while getting the content of the message");
                    e.printStackTrace();
                }
            }
        }
    }

}
