/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.monitoring;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.objectweb.jasmine.ui.monitoring.GenericDatas.Category;
import org.objectweb.jasmine.ui.monitoring.GenericDatas.Value;

public class GenericProbe implements Probe {

    public String name = "Generic probe";

    private Composite composite;

    private GenericDatas pdo;

    private Label title;

    private FormData quart1;

    private TreeViewer tv;

    private FormLayout formLayout;

    private FormData quart2;

    public GenericProbe(GenericDatas pdoi) {
        pdo = pdoi;
        pdoi.setProbe(this);
    }

    public void create(Composite c) {

        this.composite = c;

        /*
         * GridLayout gridlayout = new GridLayout(); gridlayout.numColumns = 1;
         * composite.setLayout(gridlayout);
         */
        final int insetX = 2;
        final int insetY = 2;
        formLayout = new FormLayout();
        formLayout.marginWidth = insetX;
        formLayout.marginHeight = insetY;
        composite.setLayout(formLayout);

        title = new Label(composite, SWT.NONE);
        title.setText(name);

        tv = new TreeViewer(composite);
        quart1 = new FormData();
        quart1.left = new FormAttachment(0, insetX);
        quart1.right = new FormAttachment(100, -insetX);
        quart1.top = new FormAttachment(0, insetY);
        quart1.bottom = new FormAttachment(tv.getControl(), -insetY);
        title.setLayoutData(quart1);

        quart2 = new FormData();
        quart2.left = new FormAttachment(0, insetX);
        quart2.right = new FormAttachment(100, -insetX);
        quart2.top = new FormAttachment(title, insetY);
        quart2.bottom = new FormAttachment(100, -insetY);
        tv.getControl().setLayoutData(quart2);

        tv.setContentProvider(new ITreeContentProvider() {

            public Object[] getChildren(Object parentElement) {
                if (parentElement instanceof GenericDatas) {
                    return ((GenericDatas) parentElement).getInfo().toArray();
                } else if (parentElement instanceof Category) {
                    return ((Category) parentElement).info.toArray();
                } else {
                    return null;
                }
            }

            public Object getParent(Object element) {
                // TODO Auto-generated method stub
                return null;
            }

            public boolean hasChildren(Object element) {
                if (element instanceof GenericDatas) {
                    return !((GenericDatas) element).getInfo().isEmpty();
                } else if (element instanceof Category) {
                    return !((Category) element).info.isEmpty();
                } else {
                    return false;
                }
            }

            public Object[] getElements(Object inputElement) {
                return getChildren(inputElement);
            }

            public void dispose() {
                // TODO Auto-generated method stub

            }

            public void inputChanged(Viewer viewer, Object oldInput,
                    Object newInput) {
                // TODO Auto-generated method stub

            }

        });
        tv.setLabelProvider(new ILabelProvider() {

            public Image getImage(Object element) {
                // TODO Auto-generated method stub
                return null;
            }

            public String getText(Object element) {
                if (element instanceof GenericDatas) {
                    return ((GenericDatas) element).getName();
                } else if (element instanceof Category) {
                    return ((Category) element).name;
                } else {
                    return ((Value) element).name + " : "
                            + ((Value) element).value;
                }
            }

            public void addListener(ILabelProviderListener listener) {
                // TODO Auto-generated method stub

            }

            public void dispose() {
                // TODO Auto-generated method stub

            }

            public boolean isLabelProperty(Object element, String property) {
                // TODO Auto-generated method stub
                return false;
            }

            public void removeListener(ILabelProviderListener listener) {
                // TODO Auto-generated method stub

            }

        });

        tv.setInput(pdo);
        tv.expandAll();

        // to do after modifing composite
        composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        composite.layout();

    }

    public String getName() {
        return name;
    }

    public Composite getComposite() {
        return composite;
    }

    public void refresh() {
        title.setText(name);
        tv.refresh();
        tv.expandAll();
        // to do after modifing composite
        composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        composite.layout();
    }

}
