/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.monitoring;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

public class NumericDatas implements Datas {

    public class Value {

        public String name;

        public double value;

        public boolean text = true;

        public boolean curve = true;

        public boolean save = true;

        public Value(String name, double value) {
            super();
            this.name = name;
            this.value = value;
        }

        public Value(String name, double value, boolean text, boolean curve,
                boolean save) {
            super();
            this.name = name;
            this.value = value;
            this.text = text;
            this.curve = curve;
            this.save = save;
        }

        public Value(Value v) {
            super();
            this.name = v.name;
            this.value = v.value;
            this.text = v.text;
            this.curve = v.curve;
            this.save = v.save;
        }

    }

    public class ValuesAtTime {

        private double[] values;

        private int second;

        private int minute;

        private int hour;

        public ValuesAtTime(List<Value> v, int second, int minute, int hour) {
            super();
            values = new double[numberOfFields];
            int i = 0;
            for (Value val : v) {
                this.values[i] = val.value;
                i++;
            }

            this.second = second;
            this.minute = minute;
            this.hour = hour;
        }

        public int getHour() {
            return hour;
        }

        public int getMinute() {
            return minute;
        }

        public int getSecond() {
            return second;
        }

        public double[] getValues() {
            return values;
        }

        public double getValues(int i) {
            return values[i];
        }

    }

    public static String FILE_NAME = "numeric probe";

    public static int MAX_VALUES_AT_TIME = 30;

    private LinkedList<ValuesAtTime> vt = new LinkedList<ValuesAtTime>();

    private List<Value> v = new ArrayList<Value>();

    private List<Probe> cp = new ArrayList<Probe>();

    private int numberOfFields = 0;

    public int getNumberOfFields() {
        return numberOfFields;
    }

    public void setValueAndRefresh(String name, double value) {

        setValues(name, value);

        for (Probe c : cp) {
            c.refresh();
        }

    }

    public void setValues(String name, double value) {
        Iterator<Value> it = v.iterator();
        while (it.hasNext()) {
            Value tmp = it.next();
            if (tmp.name.equals(name)) {
                tmp.value = value;
            }
        }
    }

    public void newMeasure() {
        Calendar c = Calendar.getInstance();

        vt.addLast(new ValuesAtTime(v, c.get(Calendar.SECOND), c
                .get(Calendar.MINUTE), c.get(Calendar.HOUR)));

        if (vt.size() > MAX_VALUES_AT_TIME) {
            vt.removeFirst();
        }

        for (Probe p : cp) {
            p.refresh();
        }
    }

    public List<Value> getV() {
        return v;
    }

    public ValuesAtTime getLastVt() {
        return vt.getLast();
    }

    public LinkedList<ValuesAtTime> getVt() {
        return vt;
    }

    public void addValue(String name, double value) {
        v.add(new Value(name, value));
        numberOfFields++;
    }

    public void addProbe(Probe probe) {
        cp.add(probe);
    }

    public String saveProbe(String file) {

        HSSFWorkbook wb = new HSSFWorkbook();

        HSSFSheet sheet = wb.createSheet(FILE_NAME);

        int i = 0;
        for (Value value : v) {
            if (value.save) {
                HSSFRow row = sheet.createRow((short) i++);
                row.createCell((short) 0).setCellValue(value.name);
                row.createCell((short) 1).setCellValue(value.value);
            }
        }

        // Write the output to a file
        FileOutputStream fileOut;
        try {
            fileOut = new FileOutputStream(file);
            wb.write(fileOut);
            fileOut.close();
        } catch (Exception e) {
            return e.toString();
        }

        return null;
    }

}
