/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.monitoring;

import java.text.DecimalFormat;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.objectweb.jasmine.ui.monitoring.NumericDatas.Value;
import org.objectweb.jasmine.ui.monitoring.NumericDatas.ValuesAtTime;

public class NumericProbe implements Probe {

    // listener for check button of setting view
    public class SAText extends SelectionAdapter {
        private Value value;

        public SAText(Value value) {
            super();
            this.value = value;
        }

        public void widgetSelected(SelectionEvent e) {
            value.text = ((Button) e.getSource()).getSelection();

            while (cpuValue.getChildren().length != 0) {
                cpuValue.getChildren()[0].dispose();
            }
            newText();

        }
    }

    // listener for check button of setting view
    public class SACurve extends SelectionAdapter {
        private Value value;

        public SACurve(Value value) {
            super();
            this.value = value;
        }

        public void widgetSelected(SelectionEvent e) {
            value.curve = ((Button) e.getSource()).getSelection();
            curveModified = true;

        }
    }

    // listener for check button of setting view
    public class SASave extends SelectionAdapter {
        private Value value;

        public SASave(Value value) {
            super();
            this.value = value;
        }

        public void widgetSelected(SelectionEvent e) {
            value.save = ((Button) e.getSource()).getSelection();
        }
    }

    public String name = "Numeric probe";

    public String file_name = "numeric_probe.xls";

    public int number_of_fields;

    public static String DEFAULT_VALUE = "  00.00";

    public static String NULL_VALUE = "  --.--";

    private Composite composite;

    private NumericDatas datas = new NumericDatas();

    private Label[] loadArray = new Label[number_of_fields];

    private Composite cpuSetting;

    private Composite cpuValue;

    private Composite cpuCurve;

    private int currentItem = 0;

    private int numItem;

    private CTabItem item[];

    public static int TEXT = 0;

    public static int CURVE = 1;

    public static int SETTING = 2;

    public boolean curveModified = false;

    public Image image;

    private NumericProbeGraph cpg;

    private Widget curve;

    private CTabFolder ctf;

    private Label[] loadValue;

    private Text rateText;

    private ScrolledComposite utilCurve;

    private String ip;

    private String type;

    private int rate = 0;

    private int max_rate = 1;

    public NumericProbe(NumericDatas d, String ip, String type) {
        datas = d;
        datas.addProbe(this);
        this.ip = ip;
        this.type = type;

        if (type.equals("cpu")) {
            max_rate = 5;
        }

        number_of_fields = d.getNumberOfFields();
    }

    public void create(Composite c) {
        composite = c;

        /*
         * Menu menu = new Menu (getComposite().getShell(), SWT.POP_UP);
         * MenuItem item = new MenuItem (menu, SWT.PUSH); item.setText
         * ("Popup");
         */

        composite.addMouseListener(new MouseListener() {

            public void mouseDoubleClick(MouseEvent e) {
                // TODO Auto-generated method stub
            }

            public void mouseDown(MouseEvent e) {
                if (e.button >= 2) {

                    FileDialog dialog = new FileDialog(new Shell(), SWT.SAVE);
                    dialog.setFilterNames(new String[] { "xls/jpeg Files",
                            "All Files (*.*)" });
                    dialog
                            .setFilterExtensions(new String[] { "*.xls",
                                    "*.jpg" });
                    dialog.setFilterPath("/home");
                    dialog.setFileName(file_name);

                    String ok = dialog.open();
                    if (null != ok) {
                        String tmp = datas.saveProbe(ok);
                        if (null != tmp) {
                            MessageDialog.openError(new Shell(),
                                    "Error : save failed", tmp);
                        }
                    }
                }

            }

            public void mouseUp(MouseEvent e) {
                // TODO Auto-generated method stub
            }

        });

        item = new CTabItem[3];

        ctf = new CTabFolder(composite, SWT.BORDER);
        cpuValue = newCompositeTab(ctf, type + " values");
        cpuCurve = newCompositeTab(ctf, type + " graph");
        cpuSetting = newCompositeTab(ctf, type + " setting");

        // focus on value view
        ctf.setSelection(currentItem);
        ctf.addSelectionListener(new SelectionListener() {

            public void widgetSelected(SelectionEvent e) {
                // TODO Auto-generated method stub
                if (currentItem != ctf.getSelectionIndex()) {
                    currentItem = ctf.getSelectionIndex();
                    if (curveModified) {
                        curveModified = false;
                        curve.dispose();
                        refreshCurve();
                    }
                }

            }

            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub
            }

        });

        // cpu value creation
        newText();

        // cpu curve creation
        newCurve();

        // cpu setting creation
        {
            GridLayout gridlayout = new GridLayout();
            gridlayout.numColumns = 4;
            cpuSetting.setLayout(gridlayout);

            Label label = new Label(cpuSetting, SWT.NONE);
            label.setText("name");
            label = new Label(cpuSetting, SWT.NONE);
            label.setText("text");
            label = new Label(cpuSetting, SWT.NONE);
            label.setText("graph");
            label = new Label(cpuSetting, SWT.NONE);
            label.setText("save");

            for (Value value : datas.getV()) {
                label = new Label(cpuSetting, SWT.NONE);
                label.setText(value.name);

                Button text = new Button(cpuSetting, SWT.CHECK);
                text.setSelection(value.text);
                text.addSelectionListener(new SAText(value));

                Button graph = new Button(cpuSetting, SWT.CHECK);
                graph.setSelection(value.curve);
                graph.addSelectionListener(new SACurve(value));

                Button save = new Button(cpuSetting, SWT.CHECK);
                save.setSelection(value.save);
                save.addSelectionListener(new SASave(value));
            }

            Label rateLabel = new Label(cpuSetting, SWT.NONE);
            rateLabel.setText("Refresh each measure");
            rateText = new Text(cpuSetting, SWT.BORDER);
            rateText.setText(Integer.toString(max_rate));
            rateText.addModifyListener(new ModifyListener() {
                public void modifyText(ModifyEvent e) {
                    max_rate = Integer.valueOf(rateText.getText());
                }
            });

            // to do after modifing composite

            cpuSetting
                    .setSize(cpuSetting.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            cpuSetting.layout();

        }

    }

    private Composite newCompositeTab(CTabFolder ctf, String name) {

        item[numItem] = new CTabItem(ctf, SWT.NONE);
        ScrolledComposite scrolledComposite = new ScrolledComposite(ctf,
                SWT.H_SCROLL | SWT.V_SCROLL);
        scrolledComposite.setLayout(new FillLayout());

        Composite composite = new Composite(scrolledComposite, SWT.NONE);
        // composite.setLayout(new FillLayout());
        scrolledComposite.setContent(composite);

        item[numItem].setControl(scrolledComposite);
        item[numItem].setText(name);

        numItem++;

        if (name.equals("Cpu graph")) {
            utilCurve = scrolledComposite;
        }

        return composite;
    }

    private void newText() {

        // view presentation
        GridLayout gridlayout = new GridLayout();
        gridlayout.numColumns = 2;
        cpuValue.setLayout(gridlayout);

        /*
         * StyledText title = new StyledText(composite,SWT.READ_ONLY);
         * title.setText(NAME); title.setStyleRange(new
         * StyleRange(0,NAME.length(),null,null, SWT.BOLD));
         */

        Label title = new Label(cpuValue, SWT.NONE);
        title.setText(name);
        new Label(cpuValue, SWT.NONE);
        new Label(cpuValue, SWT.NONE);

        Label sep = new Label(cpuValue, SWT.SEPARATOR | SWT.HORIZONTAL);
        GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.horizontalSpan = 3;
        data.grabExcessHorizontalSpace = true;
        sep.setLayoutData(data);

        loadValue = new Label[datas.getNumberOfFields()];

        int i = 0;
        for (Value value : datas.getV()) {
            if (value.text) {
                Label load = new Label(cpuValue, SWT.NONE);
                load.setText(value.name);

                loadValue[i] = new Label(cpuValue, SWT.BORDER);
                loadValue[i].setText(DEFAULT_VALUE);
                // loadValue[i].setText(Double.toString(value.value));
                // loadValue[i].setSize(50, 20);
                // loadValue[i].setLayoutData(new GridData(SWT.FILL, SWT.FILL,
                // true, true, 1, 1));

                /*
                 * Label percent = new Label(cpuValue, SWT.NONE);
                 * percent.setText("%");
                 */
            }
            i++;
        }

        // to do after modifing composite
        cpuValue.setSize(cpuValue.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        cpuValue.layout();
    }

    private void newCurve() {

        // view presentation
        cpuCurve.setLayout(new FillLayout());

        cpg = new NumericProbeGraph();

        refreshCurve();

        cpuCurve.addControlListener(new ControlListener() {

            public void controlMoved(ControlEvent e) {
            }

            public void controlResized(ControlEvent e) {
                if (allowRefresh) {
                    refreshCurve();
                }
            }

        });

    }

    private Label labelChoose = null;

    private boolean allowRefresh = true;

    public void refreshCurve() {
        allowRefresh = false;

        Point size = cpuCurve.getParent().getSize();
        // System.out.println(size.x+":"+size.y);

        // set graph size
        if (size.x < 500) {
            size.x = 500;
        }
        if (size.y < 300) {
            size.y = 300;
        }

        cpg.setHeight(size.x - 10);
        cpg.setLength(size.y - 10);

        // values of data set
        String[] xTime = null;
        double[][] yValues = null;
        String[] legends = null;

        boolean[] displayed = null;
        int fields = 0;

        // get the displayed informations
        displayed = new boolean[datas.getNumberOfFields()];
        int i = 0;
        for (Value val : datas.getV()) {
            displayed[i] = val.curve;
            if (val.curve) {
                fields++;
            }
            i++;
        }

        if (0 == fields) {
            labelChoose = new Label(cpuCurve, SWT.NONE);
            labelChoose.setText("Select datas to display.");

            cpuCurve.setSize(cpuCurve.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            cpuCurve.layout();
        } else {
            if (labelChoose != null && !labelChoose.isDisposed()) {
                labelChoose.dispose();
            }

            // get the legends
            legends = new String[fields];
            i = 0;
            for (Value val : datas.getV()) {
                if (val.curve) {
                    legends[i] = val.name;
                    i++;
                }
            }

            // filling of the data set
            List<ValuesAtTime> l = datas.getVt();

            xTime = new String[l.size()];
            yValues = new double[fields][l.size()];

            i = 0;
            for (ValuesAtTime vt : l) {
                xTime[i] = Integer.toString(vt.getHour()) + "h"
                        + Integer.toString(vt.getMinute()) + "m"
                        + Integer.toString(vt.getSecond());

                int k = 0;
                for (int j = 0; j < datas.getNumberOfFields(); j++) {
                    if (displayed[j]) {
                        yValues[k][i] = vt.getValues()[j];
                        k++;
                    }
                }
                i++;
            }

            // create the graph obejct
            cpg.createGraph(xTime, "Time", "Load", type + " " + ip, legends,
                    yValues);

            // get the image for the canevas
            image = cpg.createImage(cpuCurve.getDisplay());

            Control[] controls = cpuCurve.getChildren();
            for (int k = 0; k < controls.length; k++) {
                controls[k].dispose();
            }

            // create the canevas SWT control
            Canvas canvas = new Canvas(cpuCurve, SWT.NONE);

            canvas.addPaintListener(new PaintListener() {
                public void paintControl(PaintEvent e) {
                    e.gc.drawImage(image, 0, 0);
                }
            });

            if (size.x != 0) {
                cpuCurve.setSize(size.x, size.y);
                canvas.setSize(size.x - 1, size.y - 1);
            }
            cpuCurve.setLayout(new FillLayout());
            cpuCurve.layout();

            Label ll = new Label(cpuCurve, SWT.NONE);
            ll.setImage(image);

            canvas.redraw();

        }
        allowRefresh = true;
    }

    public String getName() {
        return name;
    }

    public Label l;

    public Value v;

    public void refresh() {

        // to avoid clipping one display when receive 5 new refresh from new
        // measure
        rate++;
        if (rate > max_rate) {
            rate = 0;

            if (loadValue.length > 0 && !loadValue[0].isDisposed()) {

                int i = 0;
                for (Value value : datas.getV()) {
                    v = value;

                    l = loadValue[i];

                    // it s a mandatory to synchronise SWT threads
                    l.getDisplay().syncExec(new Runnable() {
                        public void run() {
                            try {
                                // l.setText(Double.toString(v.value));
                                DecimalFormat df = new DecimalFormat(
                                        "######00.00");
                                l.setText(df.format(v.value));
                            } catch (Exception e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
                        }
                    });

                    i++;
                }

                l.getDisplay().syncExec(new Runnable() {
                    public void run() {
                        try {
                            refreshCurve();
                        } catch (Exception e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        }
                    }
                });

            }
        }

    }

    public Composite getComposite() {
        return composite;
    }

}
