/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.monitoring;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.naming.InitialContext;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.chartData.DataSeries;
import org.jCharts.encoders.JPEGEncoder;
import org.jCharts.encoders.PNGEncoder;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.DataAxisProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.properties.LineChartProperties;
import org.jCharts.properties.PropertyException;
import org.jCharts.test.TestDataGenerator;
import org.jCharts.types.ChartType;

public class NumericProbeGraph {

    private AxisChart axisChart;

    private OutputStream outputStream;

    private int fields; // the number of curves/fields

    private ChartProperties chartProperties;

    private AxisProperties axisProperties;

    private LegendProperties legendProperties;

    int length = 50;

    int height = 300;

    private static Paint[] paint;// TestDataGenerator.getRandomPaints(30);

    private Paint[] initPaint() {
        Paint[] tmp = new Paint[10];
        tmp[0] = Color.blue;
        tmp[1] = Color.cyan;
        tmp[2] = Color.darkGray;
        tmp[3] = Color.green;
        tmp[4] = Color.magenta;
        tmp[5] = Color.orange;
        tmp[6] = Color.pink;
        tmp[7] = Color.red;
        tmp[8] = Color.yellow;
        tmp[9] = Color.gray;

        return tmp;
    }

    private Paint[] getPaint(int field) {
        Paint[] tmp = new Paint[field];
        for (int i = 0; i < 30 && i < field; i++) {
            tmp[i] = paint[i];
        }
        return tmp;
    }

    public NumericProbeGraph() {
        super();

        chartProperties = new ChartProperties();
        axisProperties = new AxisProperties();
        legendProperties = new LegendProperties();

        // axisProperties.getXAxisProperties().setLabelRotationAngle(30);

        // allow to print Y axis with 0 to 100 %
        /*
         * axisProperties= new AxisProperties(); DataAxisProperties
         * dataAxisProperties= (DataAxisProperties)
         * axisProperties.getYAxisProperties(); try {
         * dataAxisProperties.setUserDefinedScale( 0, 10 ); } catch
         * (PropertyException e1) { e1.printStackTrace(); }
         * dataAxisProperties.setNumItems(11);
         */

    }

    public void createGraph(String[] xAxisLabels, String xAxisTitle,
            String yAxisTitle, String title, String[] legendLabels,
            double[][] data) {

        // get the number of fields
        fields = data.length;
        // length = data[0].length;

        DataSeries dataSeries = new DataSeries(xAxisLabels, xAxisTitle,
                yAxisTitle, title);

        paint = initPaint();
        Paint[] paints = getPaint(fields);
        Stroke[] strokes = new Stroke[fields];
        Shape[] shapes = new Shape[fields];

        for (int i = 0; i < fields; i++) {
            strokes[i] = LineChartProperties.DEFAULT_LINE_STROKE;
            shapes[i] = null;
        }
        LineChartProperties lineChartProperties = new LineChartProperties(
                strokes, shapes);

        AxisChartDataSet axisChartDataSet = null;
        try {
            axisChartDataSet = new AxisChartDataSet(data, legendLabels, paints,
                    ChartType.LINE, lineChartProperties);
        } catch (ChartDataException e) {
            e.printStackTrace();
        }
        dataSeries.addIAxisPlotDataSet(axisChartDataSet);

        // render the axis chart
        axisChart = new AxisChart(dataSeries, chartProperties, axisProperties,
                legendProperties, height, length);

        outputStream = new ByteArrayOutputStream();

        try {
            JPEGEncoder.encode(axisChart, 1.0f, outputStream);
        } catch (ChartDataException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (PropertyException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    public Image createImage(Display display) {
        InputStream is = null;

        try {
            is = new ByteArrayInputStream(
                    ((ByteArrayOutputStream) outputStream).toByteArray());
        } catch (Exception e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }

        // final Image image = new Image (display, 20, 20);
        Image image = new Image(display, is);

        return image;
    }

    public void createJPG() {

        try {
            PNGEncoder.encode(axisChart, outputStream);

            OutputStream fos = new FileOutputStream("/home/montagno/zzz.jpg");
            // PNGEncoder.encode(axisChart,fos);
            JPEGEncoder.encode(axisChart, 1.0f, outputStream);

        } catch (ChartDataException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (PropertyException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    public int getHeight() {
        return height;
    }

    public void setHeight(int height) {
        this.height = height;
    }

    public int getLength() {
        return length;
    }

    public void setLength(int length) {
        this.length = length;
    }

}
