/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.rules;

import java.io.File;
import java.io.IOException;

import javax.naming.NamingException;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.objectweb.jasmine.ui.loginAdressPort.FormUtil;

public class RulesView {
    public static void main(String[] args) {
        new RulesView().open();
    }

    private static int insetX = 2;

    private static int insetY = 2;

    private CTabFolder ctf;

    private Composite edit;

    private Composite rules;

    // Edit
    private Text editor;

    private Button activate;

    private Button save;

    private Button saveAs;

    private Button open;

    private Button newFile;

    private Combo choose;

    // Rules
    private Label description;

    private Button refresh;

    private List files;

    // Datas

    private int numItem;

    private CTabItem item[];

    ComRules cr;

    String filePath;

    String fileName;

    File file;

    String type;

    public RulesView() {
        super();
        cr = new ComRules();
    }

    public void createContent(Composite parent) {
        item = new CTabItem[2];

        // creation of tab foldres
        ctf = new CTabFolder(parent, SWT.BORDER);
        edit = newCompositeTab(ctf, "Edit");
        rules = newCompositeTab(ctf, "Rules");

        // Edit

        editor = new Text(edit, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.MULTI);

        activate = new Button(edit, SWT.PUSH);
        activate.setText("Activate");
        activate.setEnabled(false);

        newFile = new Button(edit, SWT.PUSH);
        newFile.setText("New");
        open = new Button(edit, SWT.PUSH);
        open.setText("Open");
        save = new Button(edit, SWT.PUSH);
        save.setText("Save");
        save.setEnabled(false);
        saveAs = new Button(edit, SWT.PUSH);
        saveAs.setText("Save as");

        choose = new Combo(edit, SWT.READ_ONLY);
        choose.add("XML");
        choose.add("DRL");
        choose.select(0);

        // Rules

        description = new Label(rules, SWT.BORDER);
        description.setText("Files XML and DRL.");

        files = new List(rules, SWT.BORDER);

        refresh = new Button(rules, SWT.PUSH);
        refresh.setText("Refresh");

        // Controler
        // Controler for edit
        newFile.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                // choose a file
                FileDialog fileDialog = new FileDialog(newFile.getShell(),
                        SWT.SAVE);
                fileDialog.open();
                filePath = fileDialog.getFilterPath();
                fileName = fileDialog.getFileName();
                if (!filePath.equals("") && !fileName.equals("")) {
                    // get the type of new file
                    type = choose.getText();

                    // add extension if file name isn't complet
                    if (type.equals("XML") && !fileName.endsWith(".xml")) {
                        fileName = fileName + ".xml";
                    } else if (type.equals("DRL") && !fileName.endsWith(".drl")) {
                        fileName = fileName + ".drl";
                    }
                    activate.setEnabled(true);
                    save.setEnabled(true);
                }
            }
        });

        open.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                // choose a file
                FileDialog fileDialog = new FileDialog(newFile.getShell(),
                        SWT.OPEN);
                fileDialog.open();
                filePath = fileDialog.getFilterPath();
                fileName = fileDialog.getFileName();

                if (!filePath.equals("") && !fileName.equals("")) {

                    // get the type of file
                    if (fileName.endsWith(".xml")) {
                        choose.select(0);
                    } else if (fileName.endsWith(".drl")) {
                        choose.select(1);
                    }

                    // dats datas of the file
                    file = new File(filePath + File.separator + fileName);
                    if (file.exists()) {
                        String tmp = "";
                        try {
                            tmp = Converter.readFile(file);
                        } catch (IOException e1) {
                            MessageDialog.openError(open.getShell(), "Error",
                                    "JASMINe can't open file " + filePath
                                            + File.separator + fileName);
                        }
                        editor.setText(tmp);
                    }
                    activate.setEnabled(true);
                    save.setEnabled(true);
                }
            }
        });

        save.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                try {
                    Converter.writeFile(file, editor.getText());
                } catch (IOException e1) {
                    MessageDialog.openError(open.getShell(), "Error",
                            "JASMINe can't save file " + filePath
                                    + File.separator + fileName);
                }
            }
        });

        saveAs.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                FileDialog fileDialog = new FileDialog(newFile.getShell(),
                        SWT.SAVE);
                fileDialog.open();
                filePath = fileDialog.getFilterPath();
                fileName = fileDialog.getFileName();

                type = choose.getText();

                if (type.equals("XML") && !fileName.endsWith(".xml")) {
                    fileName = fileName + ".xml";
                } else if (type.equals("DRL") && !fileName.endsWith(".drl")) {
                    fileName = fileName + ".drl";
                }

                file = new File(filePath + File.separator + fileName);

                try {
                    Converter.writeFile(file, editor.getText());
                } catch (IOException e1) {
                    MessageDialog.openError(open.getShell(), "Error",
                            "JASMINe can't save file " + filePath
                                    + File.separator + fileName);
                }
            }
        });

        activate.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                if (file.exists()) {
                    System.out.println("file exist");
                    byte[] tab = null;

                    try {
                        tab = Converter.convertFileToByte(file);
                    } catch (IOException e1) {
                        MessageDialog
                                .openError(open.getShell(), "Error",
                                        "JASMINe can't activate the rule : convertFileToByte error");
                    }
                    try {
                        if (tab != null) {
                            cr.save(tab, choose.getText(), fileName);
                        }
                    } catch (Exception e1) {
                        MessageDialog
                                .openError(open.getShell(), "Error",
                                        "JASMINe can't activate the rule : connection with EJB error");
                    }
                }
            }
        });

        // Controler for edit
        ctf.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (ctf.getSelectionIndex() == 1) {
                    refreshList();
                }
            }
        });

        refresh.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                refreshList();
            }
        });

        // layout
        parent.setLayout(new FillLayout());
        ctf.setLayout(new FillLayout());

        FormLayout formLayout;

        formLayout = new FormLayout();
        formLayout.marginWidth = insetX;
        formLayout.marginHeight = insetY;
        edit.setLayout(formLayout);

        formLayout = new FormLayout();
        formLayout.marginWidth = insetX;
        formLayout.marginHeight = insetY;
        rules.setLayout(formLayout);

        FormUtil.setFormLayout(edit, 0, 100, 0, 100);
        FormUtil.setFormLayout(rules, 0, 100, 0, 100);

        FormUtil.setFormLayout(newFile, 0, null, 0, null);
        FormUtil.setFormLayout(open, newFile, null, 0, null);
        FormUtil.setFormLayout(save, open, null, 0, null);
        FormUtil.setFormLayout(saveAs, save, null, 0, null);
        FormUtil.setFormLayout(choose, saveAs, null, 0, null);
        FormUtil.setFormLayout(editor, 0, 100, newFile, activate);
        FormUtil.setFormLayout(activate, 0, 100, null, 100);

        FormUtil.setFormLayout(description, 0, 100, 0, null);
        FormUtil.setFormLayout(files, 0, 100, description, refresh);
        FormUtil.setFormLayout(refresh, 0, null, null, 100);

    }

    private void refreshList() {
        files.removeAll();
        for (String tmp : cr.getFileUsed()) {
            files.add(tmp);
        }
    }

    private Composite newCompositeTab(CTabFolder ctf, String name) {

        item[numItem] = new CTabItem(ctf, SWT.NONE);

        Composite composite = new Composite(ctf, SWT.NONE);
        // composite.setLayout(new FillLayout());

        item[numItem].setControl(composite);
        item[numItem].setText(name);

        numItem++;

        return composite;
    }

    public void open() {
        /*cr = new ComRules();
        try {
            cr.connection("rmi://frecb003998:1099");
        } catch (NamingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }*/

        Display display = new Display();
        Shell shell = new Shell(display);
        shell.setLayout(new FillLayout());

        createContent(shell);

        shell.open();

        while (!shell.isDisposed()) {
            if (!display.readAndDispatch()) {
                display.sleep();
            }
        }
        display.dispose();
    }
}
