/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.model.discovery.popup.actions;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentEList;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequestFactory;
import org.eclipse.gmf.runtime.emf.type.core.commands.SetValueCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ocl.ecore.OCL;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.jasmine.ui.jasminemodel.ClusterContainer;
import org.objectweb.jasmine.ui.jasminemodel.JOnAS;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelFactory;
import org.objectweb.jasmine.ui.jasminemodel.JasminemodelPackage;
import org.objectweb.jasmine.ui.jasminemodel.diagram.edit.parts.DomainEditPart;
import org.objectweb.jasmine.ui.jasminemodel.diagram.part.JasmineDiagramEditorUtil;
import org.objectweb.jasmine.ui.jasminemodel.diagram.providers.JasmineElementTypes;
import org.objectweb.jasmine.ui.jasminemodel.diagram.providers.JasmineModelingAssistantProvider;
import org.objectweb.jasmine.ui.jasminemodel.impl.ClusterContainerImpl;
import org.objectweb.jasmine.ui.jasminemodel.impl.JasminemodelPackageImpl;
import org.objectweb.jonas_domain.api.DomainMapException;
import org.objectweb.jonas_domain.xml.Cluster;
import org.objectweb.jonas_domain.xml.Server;
import org.ow2.jasmine.model.discovery.JasmineDiscoveryPlugin;
import org.ow2.jasmine.model.discovery.Messages;
import org.ow2.jasmine.model.discovery.dialog.ResourceDialog;
import org.ow2.jasmine.model.domain.XMLDomainManager;

public class AutoDiscoveryAction implements IObjectActionDelegate {

	private static DomainEditPart selectedElement;
	private Shell shell;
	final private static int VERTICAL_OFFSET = 200;

	/**
	 * Constructor for Action1.
	 */
	public AutoDiscoveryAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {

		LoadResourceDialog loadResourceDialog = new LoadResourceDialog(
				shell, selectedElement.getEditingDomain());
		loadResourceDialog.open();

	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		selectedElement = null;
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			if (structuredSelection.size() == 1
					&& structuredSelection.getFirstElement() instanceof DomainEditPart) {
				selectedElement = (DomainEditPart) structuredSelection
				.getFirstElement();
			}
		}
		action.setEnabled(isEnabled());
	}

	private boolean isEnabled() {
		return selectedElement != null;
	}

	public static class LoadResourceDialog extends ResourceDialog
	{
		EditingDomain domain;

		public LoadResourceDialog(Shell parent)
		{
			this(parent, null);
		}

		public LoadResourceDialog(Shell parent, EditingDomain domain)
		{
			super(parent, Messages.LoadResourceDialog_title, SWT.OPEN | SWT.MULTI);
			this.domain = domain;
		}

		@Override
		protected boolean processResources()
		{
			if (domain != null)
			{
				for (URI uri : getURIs())
				{
					try
					{
						performDecorate(uri);
					}
					catch (RuntimeException exception)
					{
						JasmineDiscoveryPlugin.getDefault().logError(exception.getMessage());
						exception.printStackTrace();

//						MessageDialog
//						.openError(
//								null,
//								Messages.JasmineDiscoveryCreationError,
//								Messages.JasmineDiscoveryMessageCreationError);
					}
				}
			}
			return true;
		}


		private static void performDecorate(final URI uri) {
			final IRunnableWithProgress op = new IRunnableWithProgress() {

				public void run(IProgressMonitor monitor)
				throws InvocationTargetException, InterruptedException {

					File file = new File(uri.devicePath());
					XMLDomainManager.parse(file);
					final Collection<Server> servers = XMLDomainManager.getServers();
					final Collection<Cluster> clusters = XMLDomainManager.getClusters();

					if (!servers.isEmpty()) {
						// Create elements.
						createServers(selectedElement, servers);
					}

					if (!clusters.isEmpty()) {
						// Create elements.
						createClusters(selectedElement, clusters);
					}
				}
			};

			try {
				IProgressMonitor monitor = new NullProgressMonitor();
				op.run(monitor);


			} catch (InterruptedException e) {
				e.printStackTrace();

			} catch (InvocationTargetException e) {
				if (e.getTargetException() instanceof CoreException) {
					ErrorDialog.openError(null,
							Messages.JasmineDiscoveryCreationError, "Error during automatic discovery process "+e.getMessage(),
							((CoreException) e.getTargetException()).getStatus());
				} else {
					JasmineDiscoveryPlugin.getDefault().logError(
							"Error during automatic discovery process", e.getTargetException()); //$NON-NLS-1$
				}
				e.printStackTrace();
			} finally {
				XMLDomainManager.getClusters().clear();
				XMLDomainManager.getServers().clear();
				XMLDomainManager.getClusterDaemons().clear();
			}
		}


		private static void createServers(DomainEditPart selectedElement, Collection<Server> servers) {

			// Create components for the other end.
			Point p = selectedElement.getFigure().getBounds().getTopLeft().getCopy();
			p.x+=30; p.y+= 40 - VERTICAL_OFFSET;
			DomainEditPart domainEditPart = (DomainEditPart) selectedElement;

			for(Iterator<Server> it=servers.iterator(); it.hasNext();) {

				Server s=(Server)it.next();
				CompoundCommand cc = new CompoundCommand("Create JOnAS");
				CreateViewRequest jonasRequest = CreateViewRequestFactory.getCreateShapeRequest(
						JasmineElementTypes.JOnAS_1001, selectedElement.getDiagramPreferencesHint());

				jonasRequest.setLocation(p.translate(0, VERTICAL_OFFSET));
				Command createJonasCmd = domainEditPart.getCommand(jonasRequest);
				cc.add(createJonasCmd);
				selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(cc);

				IAdaptable adapter = (IAdaptable) ((List) jonasRequest.getNewObject()).get(0);

				// Add references and attributes
				View view = (View) adapter.getAdapter(View.class);
				if (view != null) {
					JOnAS newJOnAS = (JOnAS)view.getElement();
					SetRequest reqSet = new SetRequest(selectedElement.getEditingDomain(),
							newJOnAS, JasminemodelPackage.eINSTANCE.getJOnAS_Name(),s.getName());
					SetValueCommand operation = new SetValueCommand(reqSet);
					selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(new
							ICommandProxy(operation));
				}
			}
		}


		private static void createClusters(DomainEditPart selectedElement, Collection<Cluster> clusters) {

			ClusterContainer newClusterSet;
			Point p = selectedElement.getFigure().getBounds().getTopLeft().getCopy();
			p.x+=250; p.y+= 40 - VERTICAL_OFFSET;
			CompoundCommand cc = new CompoundCommand("Create ClusterSet");

			// Create the Cluster container for the other end.
			CreateViewRequest clusterSetRequest = CreateViewRequestFactory.getCreateShapeRequest(
					JasmineElementTypes.ClusterContainer_1007, selectedElement.getDiagramPreferencesHint());
			DomainEditPart domainEditPart = (DomainEditPart) selectedElement;
			Command createClusterSetCmd = domainEditPart.getCommand(clusterSetRequest);
			cc.add(createClusterSetCmd);
			selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(cc);

			IAdaptable adapter = (IAdaptable) ((List) clusterSetRequest.getNewObject()).get(0);
			View containerView = (View) adapter.getAdapter(View.class);

//			if (containerView!= null) {
				newClusterSet = (ClusterContainer)containerView.getElement();
//			} else {
//				EClass klass = JasminemodelPackage.eINSTANCE.getClusterContainer();
//
//				System.out.println("klass "+klass.getInstanceClass().getName());
//
//				newClusterSet = null;
//
//				System.out.println("eContainer() "+klass.eContents().get(0).eContainer().toString());
//				System.out.println("getClass() "+klass.getInstanceClass().getClass().toString());
//				System.out.println("getClass() "+klass.getInstanceClass().getClass().getName().toString());
//				newClusterSet = (ClusterContainer)klass.eContents().get(0).eContainer();
//			}

			// Create clusters
			for(Iterator<Cluster> it=clusters.iterator(); it.hasNext();) {

				Cluster c = (Cluster)it.next();
				org.objectweb.jasmine.ui.jasminemodel.Cluster newCluster =
					JasminemodelFactory.eINSTANCE.createCluster();

				SetRequest reqSet = new SetRequest(selectedElement.getEditingDomain(),
						newCluster, JasminemodelPackage.eINSTANCE.getCluster_Name(), c.getName());
				SetValueCommand operation = new SetValueCommand(reqSet);
				selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(
						new ICommandProxy(operation));

				reqSet = new SetRequest(selectedElement.getEditingDomain(),
						newClusterSet, JasminemodelPackage.eINSTANCE.getClusterContainer_Clusters(), newCluster);
				operation = new SetValueCommand(reqSet);
				selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(
						new ICommandProxy(operation));

				// Create all components (Server) for this cluster
				for(Iterator<Server> child=c.getServerList().iterator(); child.hasNext();) {
					Server s = (Server)child.next();

					cc = new CompoundCommand("Create Child");
					CreateViewRequest childRequest = CreateViewRequestFactory.getCreateShapeRequest(
							JasmineElementTypes.JOnAS_1001, selectedElement.getDiagramPreferencesHint());

					childRequest.setLocation(p.translate(0, VERTICAL_OFFSET));
					Command createJonasCmd = domainEditPart.getCommand(childRequest);
					cc.add(createJonasCmd);
					selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(cc);

					adapter = (IAdaptable) ((List) childRequest.getNewObject()).get(0);

					// Add references and attributes
					View view = (View) adapter.getAdapter(View.class);
					if (view != null) {
						JOnAS newJOnAS = (JOnAS)view.getElement();
						reqSet = new SetRequest(selectedElement.getEditingDomain(),
								newJOnAS, JasminemodelPackage.eINSTANCE.getJOnAS_Name(), s.getName());
						operation = new SetValueCommand(reqSet);
						selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(new
								ICommandProxy(operation));

						reqSet = new SetRequest(selectedElement.getEditingDomain(),
								newJOnAS, JasminemodelPackage.eINSTANCE.getJOnAS_Clusters(), newCluster);
						operation = new SetValueCommand(reqSet);
						selectedElement.getDiagramEditDomain().getDiagramCommandStack().execute(new
								ICommandProxy(operation));
					}
				}
			}
		}
	}

}
