/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import org.objectweb.jasmine.ui.jasminemodel.util.JasminemodelAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers.
 * The adapters generated by this factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets.
 * Note that most of the adapters are shared among multiple instances.
 * <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * @generated
 */
public class JasminemodelItemProviderAdapterFactory extends
        JasminemodelAdapterFactory implements ComposeableAdapterFactory,
        IChangeNotifier, IDisposable {
    /**
	 * This keeps track of the root adapter factory that delegates to this adapter factory.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected ComposedAdapterFactory parentAdapterFactory;

    /**
     * This is used to implement
     * {@link org.eclipse.emf.edit.provider.IChangeNotifier}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected IChangeNotifier changeNotifier = new ChangeNotifier();

    /**
	 * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}.
	 * <!-- begin-user-doc --> <!--
     * end-user-doc -->
	 * @generated
	 */
    protected Collection<Object> supportedTypes = new ArrayList<Object>();

    /**
     * This constructs an instance. <!-- begin-user-doc --> <!-- end-user-doc
     * -->
     *
     * @generated
     */
    public JasminemodelItemProviderAdapterFactory() {
		supportedTypes.add(IEditingDomainItemProvider.class);
		supportedTypes.add(IStructuredItemContentProvider.class);
		supportedTypes.add(ITreeItemContentProvider.class);
		supportedTypes.add(IItemLabelProvider.class);
		supportedTypes.add(IItemPropertySource.class);
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JASMINe} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JASMINeItemProvider jasmiNeItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JASMINe}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJASMINeAdapter() {
		if (jasmiNeItemProvider == null) {
			jasmiNeItemProvider = new JASMINeItemProvider(this);
		}

		return jasmiNeItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.ServerPool} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected ServerPoolItemProvider serverPoolItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.ServerPool}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createServerPoolAdapter() {
		if (serverPoolItemProvider == null) {
			serverPoolItemProvider = new ServerPoolItemProvider(this);
		}

		return serverPoolItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.Server} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected ServerItemProvider serverItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.Server}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createServerAdapter() {
		if (serverItemProvider == null) {
			serverItemProvider = new ServerItemProvider(this);
		}

		return serverItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JASMINeNode} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JASMINeNodeItemProvider jasmiNeNodeItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JASMINeNode}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJASMINeNodeAdapter() {
		if (jasmiNeNodeItemProvider == null) {
			jasmiNeNodeItemProvider = new JASMINeNodeItemProvider(this);
		}

		return jasmiNeNodeItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.Domain} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected DomainItemProvider domainItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.Domain}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createDomainAdapter() {
		if (domainItemProvider == null) {
			domainItemProvider = new DomainItemProvider(this);
		}

		return domainItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.Cluster} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected ClusterItemProvider clusterItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.Cluster}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createClusterAdapter() {
		if (clusterItemProvider == null) {
			clusterItemProvider = new ClusterItemProvider(this);
		}

		return clusterItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.DB} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected DBItemProvider dbItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.DB}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createDBAdapter() {
		if (dbItemProvider == null) {
			dbItemProvider = new DBItemProvider(this);
		}

		return dbItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JOnAS} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JOnASItemProvider jOnASItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.JOnAS}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createJOnASAdapter() {
		if (jOnASItemProvider == null) {
			jOnASItemProvider = new JOnASItemProvider(this);
		}

		return jOnASItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.Apache_HTTPd} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected Apache_HTTPdItemProvider apache_HTTPdItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.Apache_HTTPd}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createApache_HTTPdAdapter() {
		if (apache_HTTPdItemProvider == null) {
			apache_HTTPdItemProvider = new Apache_HTTPdItemProvider(this);
		}

		return apache_HTTPdItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.WebContainer} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected WebContainerItemProvider webContainerItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.WebContainer}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createWebContainerAdapter() {
		if (webContainerItemProvider == null) {
			webContainerItemProvider = new WebContainerItemProvider(this);
		}

		return webContainerItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.EJBContainer} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected EJBContainerItemProvider ejbContainerItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.EJBContainer}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createEJBContainerAdapter() {
		if (ejbContainerItemProvider == null) {
			ejbContainerItemProvider = new EJBContainerItemProvider(this);
		}

		return ejbContainerItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.EjbJar} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected EjbJarItemProvider ejbJarItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.EjbJar}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createEjbJarAdapter() {
		if (ejbJarItemProvider == null) {
			ejbJarItemProvider = new EjbJarItemProvider(this);
		}

		return ejbJarItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.ModJKModule} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected ModJKModuleItemProvider modJKModuleItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.ModJKModule}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createModJKModuleAdapter() {
		if (modJKModuleItemProvider == null) {
			modJKModuleItemProvider = new ModJKModuleItemProvider(this);
		}

		return modJKModuleItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.RepHTTPConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected RepHTTPConnectorItemProvider repHTTPConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.RepHTTPConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createRepHTTPConnectorAdapter() {
		if (repHTTPConnectorItemProvider == null) {
			repHTTPConnectorItemProvider = new RepHTTPConnectorItemProvider(this);
		}

		return repHTTPConnectorItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.HAConnector} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected HAConnectorItemProvider haConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.HAConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createHAConnectorAdapter() {
		if (haConnectorItemProvider == null) {
			haConnectorItemProvider = new HAConnectorItemProvider(this);
		}

		return haConnectorItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.IRMIRegistry} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected IRMIRegistryItemProvider irmiRegistryItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.IRMIRegistry}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createIRMIRegistryAdapter() {
		if (irmiRegistryItemProvider == null) {
			irmiRegistryItemProvider = new IRMIRegistryItemProvider(this);
		}

		return irmiRegistryItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.IIOP} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected IIOPItemProvider iiopItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.IIOP}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createIIOPAdapter() {
		if (iiopItemProvider == null) {
			iiopItemProvider = new IIOPItemProvider(this);
		}

		return iiopItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JRMP} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JRMPItemProvider jrmpItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.JRMP}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createJRMPAdapter() {
		if (jrmpItemProvider == null) {
			jrmpItemProvider = new JRMPItemProvider(this);
		}

		return jrmpItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.CMI} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected CMIItemProvider cmiItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.CMI}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createCMIAdapter() {
		if (cmiItemProvider == null) {
			cmiItemProvider = new CMIItemProvider(this);
		}

		return cmiItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.IRMI} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected IRMIItemProvider irmiItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.IRMI}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createIRMIAdapter() {
		if (irmiItemProvider == null) {
			irmiItemProvider = new IRMIItemProvider(this);
		}

		return irmiItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.CmiGroup} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected CmiGroupItemProvider cmiGroupItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.CmiGroup}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createCmiGroupAdapter() {
		if (cmiGroupItemProvider == null) {
			cmiGroupItemProvider = new CmiGroupItemProvider(this);
		}

		return cmiGroupItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.HaGroup} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected HaGroupItemProvider haGroupItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.HaGroup}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createHaGroupAdapter() {
		if (haGroupItemProvider == null) {
			haGroupItemProvider = new HaGroupItemProvider(this);
		}

		return haGroupItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.ClusterContainer} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected ClusterContainerItemProvider clusterContainerItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.ClusterContainer}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createClusterContainerAdapter() {
		if (clusterContainerItemProvider == null) {
			clusterContainerItemProvider = new ClusterContainerItemProvider(this);
		}

		return clusterContainerItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.HTTP_RepGroup} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected HTTP_RepGroupItemProvider httP_RepGroupItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.HTTP_RepGroup}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createHTTP_RepGroupAdapter() {
		if (httP_RepGroupItemProvider == null) {
			httP_RepGroupItemProvider = new HTTP_RepGroupItemProvider(this);
		}

		return httP_RepGroupItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.RMIGroup} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected RMIGroupItemProvider rmiGroupItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.RMIGroup}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createRMIGroupAdapter() {
		if (rmiGroupItemProvider == null) {
			rmiGroupItemProvider = new RMIGroupItemProvider(this);
		}

		return rmiGroupItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.DbLink} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected DbLinkItemProvider dbLinkItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.DbLink}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createDbLinkAdapter() {
		if (dbLinkItemProvider == null) {
			dbLinkItemProvider = new DbLinkItemProvider(this);
		}

		return dbLinkItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.RmiConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected RmiConnectorItemProvider rmiConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.RmiConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createRmiConnectorAdapter() {
		if (rmiConnectorItemProvider == null) {
			rmiConnectorItemProvider = new RmiConnectorItemProvider(this);
		}

		return rmiConnectorItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.AJP} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected AJPItemProvider ajpItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.AJP}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createAJPAdapter() {
		if (ajpItemProvider == null) {
			ajpItemProvider = new AJPItemProvider(this);
		}

		return ajpItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.HTTP} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected HTTPItemProvider httpItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.HTTP}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createHTTPAdapter() {
		if (httpItemProvider == null) {
			httpItemProvider = new HTTPItemProvider(this);
		}

		return httpItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.HTTPS} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected HTTPSItemProvider httpsItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.HTTPS}.
	 * <!-- begin-user-doc
     * --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createHTTPSAdapter() {
		if (httpsItemProvider == null) {
			httpsItemProvider = new HTTPSItemProvider(this);
		}

		return httpsItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JmsService} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JmsServiceItemProvider jmsServiceItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JmsService}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJmsServiceAdapter() {
		if (jmsServiceItemProvider == null) {
			jmsServiceItemProvider = new JmsServiceItemProvider(this);
		}

		return jmsServiceItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.DiscoveryService} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected DiscoveryServiceItemProvider discoveryServiceItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.DiscoveryService}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createDiscoveryServiceAdapter() {
		if (discoveryServiceItemProvider == null) {
			discoveryServiceItemProvider = new DiscoveryServiceItemProvider(this);
		}

		return discoveryServiceItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.MiddlewareComponent} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected MiddlewareComponentItemProvider middlewareComponentItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.MiddlewareComponent}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createMiddlewareComponentAdapter() {
		if (middlewareComponentItemProvider == null) {
			middlewareComponentItemProvider = new MiddlewareComponentItemProvider(this);
		}

		return middlewareComponentItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.ApacheProperties} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected ApachePropertiesItemProvider apachePropertiesItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.ApacheProperties}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createApachePropertiesAdapter() {
		if (apachePropertiesItemProvider == null) {
			apachePropertiesItemProvider = new ApachePropertiesItemProvider(this);
		}

		return apachePropertiesItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.ApacheMiddlewareProperties} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected ApacheMiddlewarePropertiesItemProvider apacheMiddlewarePropertiesItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.ApacheMiddlewareProperties}.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createApacheMiddlewarePropertiesAdapter() {
		if (apacheMiddlewarePropertiesItemProvider == null) {
			apacheMiddlewarePropertiesItemProvider = new ApacheMiddlewarePropertiesItemProvider(this);
		}

		return apacheMiddlewarePropertiesItemProvider;
	}

    /**
     * This keeps track of the one adapter used for all
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossAS} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    protected JBossASItemProvider jBossASItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossAS}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJBossASAdapter() {
		if (jBossASItemProvider == null) {
			jBossASItemProvider = new JBossASItemProvider(this);
		}

		return jBossASItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.JBossBinding} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected JBossBindingItemProvider jBossBindingItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossBinding}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJBossBindingAdapter() {
		if (jBossBindingItemProvider == null) {
			jBossBindingItemProvider = new JBossBindingItemProvider(this);
		}

		return jBossBindingItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.IJBossConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected IJBossConnectorItemProvider ijBossConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.IJBossConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createIJBossConnectorAdapter() {
		if (ijBossConnectorItemProvider == null) {
			ijBossConnectorItemProvider = new IJBossConnectorItemProvider(this);
		}

		return ijBossConnectorItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.JBossHAConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected JBossHAConnectorItemProvider jBossHAConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossHAConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJBossHAConnectorAdapter() {
		if (jBossHAConnectorItemProvider == null) {
			jBossHAConnectorItemProvider = new JBossHAConnectorItemProvider(this);
		}

		return jBossHAConnectorItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.JBossHTTPRepConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected JBossHTTPRepConnectorItemProvider jBossHTTPRepConnectorItemProvider;

    /**
	 * This creates an adapter for a {@link org.objectweb.jasmine.ui.jasminemodel.JBossHTTPRepConnector}.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter createJBossHTTPRepConnectorAdapter() {
		if (jBossHTTPRepConnectorItemProvider == null) {
			jBossHTTPRepConnectorItemProvider = new JBossHTTPRepConnectorItemProvider(this);
		}

		return jBossHTTPRepConnectorItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.JBossHAGroup} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected JBossHAGroupItemProvider jBossHAGroupItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossHAGroup}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJBossHAGroupAdapter() {
		if (jBossHAGroupItemProvider == null) {
			jBossHAGroupItemProvider = new JBossHAGroupItemProvider(this);
		}

		return jBossHAGroupItemProvider;
	}

    /**
	 * This keeps track of the one adapter used for all {@link org.objectweb.jasmine.ui.jasminemodel.JBossModJKConnector} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected JBossModJKConnectorItemProvider jBossModJKConnectorItemProvider;

    /**
     * This creates an adapter for a
     * {@link org.objectweb.jasmine.ui.jasminemodel.JBossModJKConnector}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    @Override
    public Adapter createJBossModJKConnectorAdapter() {
		if (jBossModJKConnectorItemProvider == null) {
			jBossModJKConnectorItemProvider = new JBossModJKConnectorItemProvider(this);
		}

		return jBossModJKConnectorItemProvider;
	}

    /**
     * This returns the root adapter factory that contains this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    public ComposeableAdapterFactory getRootAdapterFactory() {
		return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
	}

    /**
     * This sets the composed adapter factory that contains this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    public void setParentAdapterFactory(
            ComposedAdapterFactory parentAdapterFactory) {
		this.parentAdapterFactory = parentAdapterFactory;
	}

    /**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public boolean isFactoryForType(Object type) {
		return supportedTypes.contains(type) || super.isFactoryForType(type);
	}

    /**
	 * This implementation substitutes the factory itself as the key for the adapter.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Adapter adapt(Notifier notifier, Object type) {
		return super.adapt(notifier, this);
	}

    /**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    @Override
    public Object adapt(Object object, Object type) {
		if (isFactoryForType(type)) {
			Object adapter = super.adapt(object, type);
			if (!(type instanceof Class) || (((Class<?>)type).isInstance(adapter))) {
				return adapter;
			}
		}

		return null;
	}

    /**
	 * This adds a listener.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public void addListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.addListener(notifyChangedListener);
	}

    /**
	 * This removes a listener.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public void removeListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.removeListener(notifyChangedListener);
	}

    /**
     * This delegates to {@link #changeNotifier} and to
     * {@link #parentAdapterFactory}. <!-- begin-user-doc --> <!-- end-user-doc
     * -->
     *
     * @generated
     */
    public void fireNotifyChanged(Notification notification) {
		changeNotifier.fireNotifyChanged(notification);

		if (parentAdapterFactory != null) {
			parentAdapterFactory.fireNotifyChanged(notification);
		}
	}

    /**
     * This disposes all of the item providers created by this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    public void dispose() {
		if (jasmiNeItemProvider != null) jasmiNeItemProvider.dispose();
		if (serverPoolItemProvider != null) serverPoolItemProvider.dispose();
		if (serverItemProvider != null) serverItemProvider.dispose();
		if (jasmiNeNodeItemProvider != null) jasmiNeNodeItemProvider.dispose();
		if (domainItemProvider != null) domainItemProvider.dispose();
		if (clusterItemProvider != null) clusterItemProvider.dispose();
		if (dbItemProvider != null) dbItemProvider.dispose();
		if (jOnASItemProvider != null) jOnASItemProvider.dispose();
		if (apache_HTTPdItemProvider != null) apache_HTTPdItemProvider.dispose();
		if (webContainerItemProvider != null) webContainerItemProvider.dispose();
		if (ejbContainerItemProvider != null) ejbContainerItemProvider.dispose();
		if (ejbJarItemProvider != null) ejbJarItemProvider.dispose();
		if (modJKModuleItemProvider != null) modJKModuleItemProvider.dispose();
		if (repHTTPConnectorItemProvider != null) repHTTPConnectorItemProvider.dispose();
		if (haConnectorItemProvider != null) haConnectorItemProvider.dispose();
		if (irmiRegistryItemProvider != null) irmiRegistryItemProvider.dispose();
		if (iiopItemProvider != null) iiopItemProvider.dispose();
		if (jrmpItemProvider != null) jrmpItemProvider.dispose();
		if (cmiItemProvider != null) cmiItemProvider.dispose();
		if (irmiItemProvider != null) irmiItemProvider.dispose();
		if (cmiGroupItemProvider != null) cmiGroupItemProvider.dispose();
		if (haGroupItemProvider != null) haGroupItemProvider.dispose();
		if (clusterContainerItemProvider != null) clusterContainerItemProvider.dispose();
		if (httP_RepGroupItemProvider != null) httP_RepGroupItemProvider.dispose();
		if (rmiGroupItemProvider != null) rmiGroupItemProvider.dispose();
		if (dbLinkItemProvider != null) dbLinkItemProvider.dispose();
		if (rmiConnectorItemProvider != null) rmiConnectorItemProvider.dispose();
		if (ajpItemProvider != null) ajpItemProvider.dispose();
		if (httpItemProvider != null) httpItemProvider.dispose();
		if (httpsItemProvider != null) httpsItemProvider.dispose();
		if (jmsServiceItemProvider != null) jmsServiceItemProvider.dispose();
		if (discoveryServiceItemProvider != null) discoveryServiceItemProvider.dispose();
		if (middlewareComponentItemProvider != null) middlewareComponentItemProvider.dispose();
		if (apachePropertiesItemProvider != null) apachePropertiesItemProvider.dispose();
		if (apacheMiddlewarePropertiesItemProvider != null) apacheMiddlewarePropertiesItemProvider.dispose();
		if (jBossASItemProvider != null) jBossASItemProvider.dispose();
		if (jBossBindingItemProvider != null) jBossBindingItemProvider.dispose();
		if (ijBossConnectorItemProvider != null) ijBossConnectorItemProvider.dispose();
		if (jBossHAConnectorItemProvider != null) jBossHAConnectorItemProvider.dispose();
		if (jBossHTTPRepConnectorItemProvider != null) jBossHTTPRepConnectorItemProvider.dispose();
		if (jBossHAGroupItemProvider != null) jBossHAGroupItemProvider.dispose();
		if (jBossModJKConnectorItemProvider != null) jBossModJKConnectorItemProvider.dispose();
	}

}
