/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ui.jasminemodel.properties;

import java.net.InetAddress;
import java.net.UnknownHostException;

public class JasmineLoadProperties {

	/**
	 *
	 */
	private static final String DEFAULT_JADE_FRONT_PORT = "1099";
	/**
	 *
	 */
	private static final String DEFAULT_JADE_JMX_PORT = "9098";
	/**
	 *
	 */
	private static final String DEFAULT_JADE_RMI_PORT = "1238";
	/**
	 *
	 */
	private static final String DEFAULT_PROBE_FRONT_PORT = "1099";
	/**
	 *
	 */
	private static final String DEFAULT_FDF_FRONT_PORT = "1099";
	/**
	 *
	 */
	private static final String DEFAULT_LOG_FRONT_PORT = "1099";


	/**
	 * @return
	 */
	public static String getJadeFrontIp() {
		String prop = System.getProperty("jasmine.jade.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getJadeFrontPort() {
		String prop = System.getProperty("jasmine.jade.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_JADE_FRONT_PORT;
	}

	/**
	 * @return
	 */
	public static String getJadeJMXIp() {
		String prop = System.getProperty("jasmine.jade.jmx.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getJadeJMXPort() {
		String prop = System.getProperty("jasmine.jade.jmx.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_JADE_JMX_PORT;
	}

	/**
	 * @return
	 */
	public static String getJadeRMIIp() {
		String prop = System.getProperty("jasmine.jade.rmi.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getJadeRMIPort() {
		String prop = System.getProperty("jasmine.jade.rmi.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_JADE_RMI_PORT;
	}

	/**
	 * @return
	 */
	public static String getLogFrontIp() {
		String prop = System.getProperty("jasmine.rule.log.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getLogFrontPort() {
		String prop = System.getProperty("jasmine.rule.log.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_LOG_FRONT_PORT;
	}

	/**
	 * @return
	 */
	public static String getProbeFrontIp() {
		String prop = System.getProperty("jasmine.probe.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getProbeFrontPort() {
		String prop = System.getProperty("jasmine.probe.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_PROBE_FRONT_PORT;
	}

	/**
	 * @return
	 */
	public static String getFDFFrontIp() {
		String prop = System.getProperty("jasmine.fdf.ip");
		if (prop != null) {
			return prop;
		}
		return getLocalhostAddress();
	}

	/**
	 * @return
	 */
	public static String getFDFFrontPort() {
		String prop = System.getProperty("jasmine.fdf.port");
		if (prop != null) {
			return prop;
		}
		return DEFAULT_FDF_FRONT_PORT;
	}

	/**
	 * @return
	 */
	private static String getLocalhostAddress(){

		try {

			InetAddress localhost = InetAddress.getLocalHost();
			return localhost.getHostAddress();

		} catch (UnknownHostException e) {
			return "";
		}

	}
}
