/**
 * JASMINe
 * Copyright (C) 2006,2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.jade.application;

import java.util.HashMap;
import java.util.Map;

/**
 *
 *
 */
public class ApplicationImpl extends Application implements ApplicationIfc {

	/**
	 *
	 */
	private String adlId = null;
	/**
	 *
	 */
	protected Map<String, String> context = null;
	/**
	 *
	 */
	private int componentID;
	/**
	 *
	 */
	private boolean deployed = false;

	// ------------------------------------------------------------------------
	// Constructor
	// ------------------------------------------------------------------------

	public ApplicationImpl(String adlId) throws Exception {
		this.adlId = adlId;
		context = new HashMap<String, String>();
	}

	// ------------------------------------------------------------------------
	// Implementation of the ApplicationIfc interface
	// ------------------------------------------------------------------------

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#deploy(java.util.Map)
	 */
	public void deploy() throws Exception {

		testEJBConnexion();

		componentID = jadeBean.deploy(this.adlId, context);
		this.deployed = true;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#deploy(java.lang.String)
	 */
	public void deploy(String address) throws Exception {
		this.deploy(address, "0");
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#deploy(java.lang.String,
	 *      java.lang.String)
	 */
	public void deploy(String address, String jadeNodeNumber) throws Exception {

		testEJBConnexion();

		componentID = jadeBean.deploy(this.adlId, address, jadeNodeNumber);
		this.deployed = true;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#deploy(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public void deploy(String address, String dirLocal, String jadeNodeNumber)
			throws Exception {

		testEJBConnexion();

		componentID = jadeBean.deploy(this.adlId, address, dirLocal,
				jadeNodeNumber);
		this.deployed = true;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#start()
	 */
	public void start() throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			jadeBean.start(componentID);
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#stop()
	 */
	public void stop() throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			jadeBean.stop(componentID);
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#getAttribute(java.lang.String)
	 */
	public String getAttribute(String attribute) throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			return jadeBean.getAttribute(componentID, attribute);
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#setAttribute(java.lang.String, java.lang.String)
	 */
	public void setAttribute(String attribute, String value) throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			jadeBean.setAttribute(componentID, attribute, value);
		} else {
			throw new Exception("The application is not deployed.");
		}

	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#listAttributes()
	 */
	public String[] listAttributes() throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			return jadeBean.listAttributes(getName());
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#bind(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void bind(String component1, String itf1, String component2,
			String itf2) throws Exception {

		testEJBConnexion();

		jadeBean.bind(component1, itf1, component2, itf2);
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#unbind(java.lang.String, java.lang.String)
	 */
	public void unbind(String component1, String itf1) throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			jadeBean.unbind(component1, itf1);
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#listFc(java.lang.String)
	 */
	public String[] listFc(String component1) throws Exception {

		testEJBConnexion();

		if (deployed == true) {
			return jadeBean.listFc(component1);
		} else {
			throw new Exception("The application is not deployed.");
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.application.ApplicationIfc#getName()
	 */
	public String getName() {
		if (deployed == false) {
			return null;
		}
		return adlId + "_" + componentID;
	}

	// ------------------------------------------------------------------------
	//
	// ------------------------------------------------------------------------

	/**
	 * @param componentID
	 */
	public void setId(int componentID) {
		this.componentID = componentID;
	}

	/**
	 * @param deployed
	 * @throws Exception
	 */
	public void setDeployed(boolean deployed) throws Exception {
		this.deployed = deployed;
	}
}
